function seg = assign_occlusion_layer(this, seg)

% Assign the last occlusion layer to corresponding layers by spatial
% smoothness (modulated by intensity information)

% "inpainting" with side information
lambda = 1; % no need

% nL = this.nLayers;
nL = max(seg(:));

m   = size(seg, 1);
n   = size(seg, 2);

% Only process pixels in the mask
mask = seg == nL; 

% randInd = min(nL-1, max(1, round(rand(m,n)*nL)));
% seg(mask) = randInd(mask);

seg(mask) = 1;

labelset    = 1:nL-1;

% Index for row and column, needed for evluating data term for a label field
%[indr indc] = meshgrid(1:n, 1:m);
[indr indc] = ind2sub([m n], reshape(1:m*n, [m n]));

prior_func = @(u) pairwise_potential_table(u, this.pTable);

% Define cliques
vert = im2col((reshape(1:m*n,m,n)),[2 1])';%vertical
hor  = im2col((reshape(1:m*n,m,n)),[1 2])';%horizontal

% Initialize energy
current_energy = 1e32;
success = 0;

% Initialize labeling
%fp = ones(m,n)*labelset(end);
fp = seg;

% Computer intensity difference
% sigma_i = 1;
% im1  = this.seg;
% dif1 = im1(vert(:,1)) - im1(vert(:,2));
% dif2 = im1(hor(:,1))  - im1(hor(:, 2));
% dif  = [dif1(:)' dif2(:)'];
% dif  = exp(-dif.^2/sigma_i^2/2);
% %dif  = dif /max(dif);  % normalization
% dif  = dif /mean(dif);  % normalization
% dif  = repmat(dif, [4 1]);

% Computer segmentation difference
im1  = this.seg;
dif1 = im1(vert(:,1)) ~= im1(vert(:,2));
dif2 = im1(hor(:,1))  ~= im1(hor(:, 2));
dif  = [dif1(:)' dif2(:)'];
dif  = exp(-dif); % within segment e^0 =1, across segment e^-1 
dif  = repmat(dif, [4 1]);


%%% start alpha_expansion
while true
    for loop=1 : size(labelset,2)
        alpha = labelset(loop);
        current_proposal = repmat(alpha,m,n);
        
        % Do not change pixels out of mask
        current_proposal(~mask) = fp(~mask);
        
        %in case of pairwise mrfs, we do not split the potentials to unary
        %and pairwise terms, we simply use the energytable
        %[E00,E01,E10,E11], which can be represented by single pairwise
        %term
        
        pt1 = layer_generateEnergyTable(vert, prior_func, current_proposal, fp);
        pt2 = layer_generateEnergyTable(hor,  prior_func, current_proposal, fp);

        pairwise_terms  = [pt1 pt2];          
        %pairwise_terms(3:end,:) = lambda*pairwise_terms(3:end,:); 
        
        % Modulated by intensity/segmentation difference
        pairwise_terms(3:end,:) = lambda*pairwise_terms(3:end,:).*dif;
        
        %Seteup dataterm, these are only unary terms                

        % No unary term
        Dp_fp       = zeros(m*n,1);                          
        Dp_alpha    = zeros(m*n,1);                          
        
        %Arrange dataterm to a 3xn matrix
        data_unary = [(1:m*n)' Dp_alpha(:) Dp_fp(:)]';        
      
        [new_fp,new_energy] = QPBO_all_in_one(m*n, data_unary, pairwise_terms);
        
        if (new_energy - current_energy) > .5
            %if we use potentials of type double, it is
            %possible that the new energy grows, because of numerical
            %issues, see QPBO documentation

%             disp(strcat('WARNING: energy grows, (new_energy - current_energy) >'...
%                 ,num2str(new_energy - current_energy)));
        elseif (new_energy - current_energy) < -1
            success = 1;
            current_energy=new_energy;
        end
%         disp(strcat('alpha=',num2str(alpha),', energy=',num2str(current_energy)));
        
        new_fp = reshape(new_fp(1:m*n),m,n);%reshape new_fp to given image size
        fp = (1-new_fp).* current_proposal+ new_fp .* fp;%generate new labeling
        
    end
    if success == 0
%         disp('finished, energy did not shrink');
        break;
    end
    success = 0;
end

seg = fp;