function [pyramid_images, color_pyramid_images, grayImages] = build_image_pyramid(this)
%%

% Images
if size(this.images,4) > 1
    grayImages = [];
    labImages = [];
    for i = 1:size(this.images,4)
        grayImages = cat(3, grayImages, double(rgb2gray(uint8(this.images(:,:,:,i)))) );
        
        im1 = RGB2Lab(this.images(:,:,:,i));

        for j = 1:size(im1, 3);
            im1(:,:,j) = scale_image(im1(:,:,j), 0, 255);
        end;
        labImages = cat(4, labImages, im1);
    end;    
else
    grayImages = double(this.images);    
    % Duplicate gray for color-modulated hidden field prior
    labImages = [];        
    for i = 1:size(this.images,3)
        im1 = this.images(:,:,i);
        labImages = cat(4, labImages, repmat(im1, [1 1 3]));
    end;        
end;

% Perform structure texture decomposition to get the texture component
if this.texture == true;
    images  = structure_texture_decomposition_rof(grayImages, 1/8, 100, this.alp);
else
    images  = scale_image(grayImages, 0, 255);
end;

% Build image pyramid
factor            = sqrt(2);  % sqrt(3) worse
smooth_sigma      = sqrt(this.pyramid_spacing)/factor;   % or sqrt(3) recommended by Manuel Werlberger
f                 = fspecial('gaussian', 2*round(1.5*smooth_sigma) +1, smooth_sigma);
pyramid_images    = compute_image_pyramid(images, f, this.pyramid_levels, 1/this.pyramid_spacing);
color_pyramid_images = compute_image_pyramid(labImages, f, this.pyramid_levels, 1/this.pyramid_spacing);