function [layerLabels, layer_uvf, layer_uvb, pyramid_images, color_pyramid_images, layer_g, seg, incon] = compute_bad_init(this, init)

% only first layer is visible and takes all the good flow fields. all the
% rest flwo fields are constant flow fields

% Frame size
sz = [size(this.images, 1), size(this.images, 2)];


uvf = init{1};
uvb = init{2};
T   =  length(uvf)+1;
nL  = this.nLayers;

% Images
if size(this.images,4) > 1
    grayImages = [];
    labImages = [];
    for i = 1:size(this.images,4)
        grayImages = cat(3, grayImages, double(rgb2gray(uint8(this.images(:,:,:,i)))) );
        
        im1 = RGB2Lab(this.images(:,:,:,i));
        for j = 1:size(im1, 3);
            im1(:,:,j) = scale_image(im1(:,:,j), 0, 255);
        end;
        labImages = cat(4, labImages, im1);
    end;    
else
    grayImages = double(this.images);    
    % Duplicate gray for color-modulated hidden field prior
    labImages = [];        
    for i = 1:size(this.images,3)
        im1 = this.images(:,:,i);
        labImages = cat(4, labImages, repmat(im1, [1 1 3]));
    end;        
end;

% Perform structure texture decomposition to get the texture component
if this.texture == true;
    images  = structure_texture_decomposition_rof(grayImages, 1/8, 100, this.alp);
else
    images  = scale_image(grayImages, 0, 255);
end;

% Build image pyramid
factor            = sqrt(2);  % sqrt(3) worse
smooth_sigma      = sqrt(this.pyramid_spacing)/factor;   % or sqrt(3) recommended by Manuel Werlberger
f                 = fspecial('gaussian', 2*round(1.5*smooth_sigma) +1, smooth_sigma);
pyramid_images    = compute_image_pyramid(images, f, this.pyramid_levels, 1/this.pyramid_spacing);
color_pyramid_images = compute_image_pyramid(labImages, f, this.pyramid_levels, 1/this.pyramid_spacing);


% % % perform ROF on each image pyramid
% % gray_pyramid_images    = compute_image_pyramid(grayImages, f, this.pyramid_levels, 1/this.pyramid_spacing);
% % for l = 1:length(gray_pyramid_images)
% %     pyramid_images{l}  = structure_texture_decomposition_rof(gray_pyramid_images{l}, 1/8, 100, this.alp);
% % end;
% 
% 
% % Flow field
% layer_uvf = cell(1,T-1); layer_uvb = cell(1,T-1); 
% for t=1:T-1
%     i=1;
%     layer_uvf{t}{i}  = uvf{t};
%     layer_uvb{t}{i}  = uvb{t};    
%     for i=2:nL
%         layer_uvf{t}{i}  = zeros([sz 2]);
%         layer_uvb{t}{i}  = zeros([sz 2]);
%     end;
% end;
% 
% % only first layer visible
% seg = {ones(sz), ones(sz)};
% 
% %%%% Compute hidden fields for each layer
incon = cell(1,T);
for t=1:T    
    incon{t} = zeros([sz nL-1]);
end;



%% GT of bird
seg1 = imread('/data/vision/dqsun/program/iccv11_flow/data/bird_apple/seg004.png');
seg2 = imread('/data/vision/dqsun/program/iccv11_flow/data/bird_apple/seg005.png');
seg = {seg1, seg2};

[layer_g layerLabels] = convert_ini_seg_to_support_function(this, seg, incon);

s1 = load('/data/vision/dqsun/program/iccv11_flow/data/bird_apple/support004.mat');
s2 = load('/data/vision/dqsun/program/iccv11_flow/data/bird_apple/support005.mat');

sz = size(seg1);
uv1 = -10*ones([sz 2]); uv1(:,:,2) = 0; 
uv2 = 20*ones([sz 2]); uv2(:,:,2) = 0; 
uv3 = zeros([sz 2]);
layer_uvf{1} = {uv1, uv2, uv3};
layer_uvb{1} = {-uv1, -uv2, uv3};
layer_g = {s1.g(:,:,1:2)/10-0.05, s2.g(:,:,1:2)/10-0.05};

% this2 = this; this2.lambda4 = this.lambda4s(1, end);
% [en] = evaluate_energy(this2, layer_g, layer_uv, layer_uvb, pyramid_images{1}, colorImages2);
% fprintf('%3.3e\n', en);


%% collpaged everything into a single layer

% t = 1;
% uvf = composite_final_flow_g(layer_uvf{t}, layer_g{t});
% uvb = composite_final_flow_g(layer_uvb{t}, layer_g{t+1});
% %
% layer_uvf{t} = {uvf, uv3, uv3};
% layer_uvb{t} = {uvb, uv3, uv3};
% 
% layer_g = {0.05*cat(3, ones(sz), -ones(sz)), 0.05*cat(3, ones(sz), -ones(sz))};
 
% this2 = this; this2.lambda4 = this.lambda4s(1, end);
% [en] = evaluate_energy(this2, layer_g, layer_uv, layer_uvb, pyramid_images{1}, colorImages2);
% fprintf('%3.3e\n', en);
