function uv = compute_flow(this, init, gt)
%
%COMPUTE_FLOW   Compute flow field
%   UV = COMPUTE_FLOW(THIS[, INIT]) computes the flow field UV with
%   algorithm THIS and the optional initialization INIT.
%  
%   This is a member function of the class 'ba_optical_flow'. 

%   Author: Deqing Sun, Department of Computer Science, Brown University
%   Contact: dqsun@cs.brown.edu
%   $Date: 2007-- $

% Copyright 2007-2008, Brown University, Providence, RI.
%
%                         All Rights Reserved
%
% Permission to use, copy, modify, and distribute this software and its
% documentation for any purpose other than its incorporation into a
% commercial product is hereby granted without fee, provided that the
% above copyright notice appear in all copies and that both that
% copyright notice and this permission notice appear in supporting
% documentation, and that the name of the author and Brown University not be used in
% advertising or publicity pertaining to distribution of the software
% without specific, written prior permission.
%
% THE AUTHOR AND BROWN UNIVERSITY DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
% INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR ANY
% PARTICULAR PURPOSE.  IN NO EVENT SHALL THE AUTHOR OR BROWN UNIVERSITY BE LIABLE FOR
% ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
% WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
% ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
% OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.  


  % Frame size
  sz = [size(this.images, 1), size(this.images, 2)];

  % If we have no initialization argument, initialize with all zeros
  if (nargin < 2)
    uv = zeros([sz, 2]);
  else
    uv = init;
  end
   
  % Perform structure texture decomposition to get the texture component
  if this.texture == true;
%       images  = structure_texture_decomposition_rof( this.images);  % texture: structure 20:1
      images  = structure_texture_decomposition_rof( this.images, 1/8, 100, this.alp); 
%       % laplacian
%       D = [0 1 0; 1 -4 1; 0 1 0];
%       images = imfilter(this.images, D, 'symmetric');
%       images  = scale_image(images, 0, 255);
       
  elseif this.fc
      
      % 1 fixed filter response constancy
%       f = fspecial('gaussian', [5 5], 1.5);
%      Dx = [-1 0 1]; Dy = Dx';
%       f = Dx;
%       images  = this.images- imfilter(this.images, f, 'symmetric');

       % Real "Dx"
       Dx = [-1 0 1]/2; 
       %Dy = Dx';      
       images  = imfilter(this.images, Dx, 'symmetric');

        % Laplacian in flowfusion
%         f = fspecial('gaussian', [5 5], 1.5);
%         images  = this.images- imfilter(this.images, f, 'symmetric');
        % Gaussian pre-filering
%         f = fspecial('gaussian', [3 3], 0.5);  % Li & Huttenlocher
%         images  = imfilter(this.images, f, 'symmetric');

      % 1 - Dx - Dy      
%       Dx = [-1 0 1]/2; Dy = Dx';      
%       images  = this.images - imfilter(this.images, Dx, 'symmetric')...
%                             - imfilter(this.images, Dy, 'symmetric');
%         % Dx then Dy doesn't work
%         Dx = [-1 0 1]; Dy = Dx';
%         images  = imfilter(this.images, Dx, 'symmetric');
%         images  = imfilter(images, Dy, 'symmetric');
        
         % sobel edge magnitude
%         Dy = fspecial('sobel')/8;
%         Dx = Dy';        
%         Ix = imfilter(this.images, Dx, 'symmetric');
%         Iy = imfilter(this.images, Dy, 'symmetric');
%         images = sqrt(Ix.^2 + Iy.^2);         
        
        images  = scale_image(images, 0, 255);
  else
      images  = scale_image(this.images, 0, 255);      
  end;
    
  % Automatic determine pyramid level
  this.pyramid_levels  =  1 + floor( log(min(size(images, 1), size(images,2))/16) / log(this.pyramid_spacing) );  
  
  
% % To use initial value
% if sum(init(:) ~=0) > 0
%   this.pyramid_levels = 1;
%   this.alpha = 0;
%   this.gnc_iters = 1;
% end;

  % Construct image pyramid, using filter setting in Bruhn et al in "Lucas/Kanade.." (IJCV2005') page 218
  
  % For gnc stage 1    
  factor            = sqrt(2);  % sqrt(3) worse
  smooth_sigma      = sqrt(this.pyramid_spacing)/factor;   % or sqrt(3) recommended by Manuel Werlberger 
  f                 = fspecial('gaussian', 2*round(1.5*smooth_sigma) +1, smooth_sigma);        
  pyramid_images    = compute_image_pyramid(images, f, this.pyramid_levels, 1/this.pyramid_spacing);
% pyramid_images    = compute_image_pyramid2(images, smooth_sigma, this.pyramid_levels, 1/this.pyramid_spacing);


  % For segmentation purpose
  org_pyramid_images = compute_image_pyramid(this.images, f, this.pyramid_levels, 1/this.pyramid_spacing);
%   org_pyramid_images = compute_image_pyramid2(this.images, smooth_sigma, this.pyramid_levels, 1/this.pyramid_spacing);
  
  % For gnc stage 2 to gnc_iters  
  smooth_sigma      = sqrt(this.gnc_pyramid_spacing)/factor;
  f                 = fspecial('gaussian', 2*round(1.5*smooth_sigma) +1, smooth_sigma);        
  gnc_pyramid_images= compute_image_pyramid(images, f, this.gnc_pyramid_levels, 1/this.gnc_pyramid_spacing); 
% gnc_pyramid_images= compute_image_pyramid2(images, smooth_sigma, this.gnc_pyramid_levels, 1/this.gnc_pyramid_spacing); 
    
  % For segmentation purpose
  org_gnc_pyramid_images = compute_image_pyramid(this.images, f, this.gnc_pyramid_levels, 1/this.gnc_pyramid_spacing);   
%    org_gnc_pyramid_images = compute_image_pyramid2(this.images, smooth_sigma, this.gnc_pyramid_levels, 1/this.gnc_pyramid_spacing);   
   
  tic
  
  
  
%       if this.display
%           disp(['GNC stage: ', num2str(ignc)])
%       end
          
%       if ignc == 1
%           pyramid_levels = this.pyramid_levels;
%           pyramid_spacing = this.pyramid_spacing;
%           
%   
%       else
%           pyramid_levels = this.gnc_pyramid_levels;
%           pyramid_spacing = this.gnc_pyramid_spacing;
% 
%       end;
      
%   if ignc == this.gnc_iters     % if 2nd GNC produces good initialization
%       pyramid_levels = 1;
%   end;
      
      % Iterate through all pyramid levels starting at the top
      pyramid_levels  = this.pyramid_levels;      
      
      for l = pyramid_levels:-1:1
          
          if this.display
              disp(['-Pyramid level: ', num2str(l)])
          end
          
          nsz               = [size(pyramid_images{l}, 1) size(pyramid_images{l}, 2)];
          im1   = org_pyramid_images{l}(:,:,1);
          
          % Rescale the flow field
          uv        = resample_flow(uv, nsz);
          
          %           % adaptively determine half window size for the area term
          %           this.area_hsz        = min(10, max(2, ceil(min(nsz)/30)) );
          %           % this.area_hsz        = min(5, max(2, ceil(min(nsz)/30)) );
          %           this.affine_hsz      = min(4, max(2, ceil(min(nsz)/75)) );
          
          for ignc = 1:this.gnc_iters
              
              % Update GNC parameters (linearly)
              this.alpha = 1 - (ignc-1) / (this.gnc_iters-1);
              
              
              % Generate copy of algorithm with single pyramid level and the
              % appropriate subsampling
              small = this;
              small.images      = pyramid_images{l};
              small.max_linear  = 1;             % number of linearization performed per warping, 1 OK for quadratic formulation
              small.seg = im1;
              %           small.seg = small.images(:,:,1);
              
              
              % Run flow method on subsampled images
              uv = compute_flow_base(small, uv);
              if l == 1 %this.display
                  
                  fprintf('log-posterior\t%3.3e\n', evaluate_log_posterior(small, uv));
                  if nargin == 3
                      [aae stdae aepe] = flowAngErr(gt(:,:,1), gt(:,:,2), uv(:,:,1), uv(:,:,2), 0);        % ignore 0 boundary pixels
                      fprintf('AAE %3.3f STD %3.3f average end point error %3.3f \n', aae, stdae, aepe);
                  end;
                  
              end;

          end          
          
          figure; imshow(flowToColor(uv), []);
          toc/60
      end;
      
