function [small layer_uv layer_g layerLabels layer_uvb]  = compute_flow_hidden_field(this, layerLabels, layer_uv,...
    layer_uvb, pyramid_images, color_pyramid_images, layer_g)
% Subprogram to perform pyramid flow and hiddenf field estimation


T = length(layer_uv)+1;
% Interpolate invisible (invalid) pixels for each layer by affine motion model
% Redundant: to remove
% for t=1:T-1
%     layer_uv{t}  = affine_interpolate_invisible(layer_uv{t}, layerLabels{t});
%     layer_uvb{t} = affine_interpolate_invisible(layer_uvb{t}, layerLabels{t+1});
% end;

small = this;   

% Iterate through all pyramid levels starting at the top
for l = this.pyramid_levels:-1:1
    
    if this.display
        disp(['-Pyramid level: ', num2str(l)])
    end      

    nsz  = [size(pyramid_images{l}, 1) size(pyramid_images{l}, 2)];
    
    % Rescale the flow field 
    for t=1:T-1;
        for i1 = 1:this.nLayers;
            layer_uv{t}{i1}  = resample_flow(layer_uv{t}{i1}, nsz);
            layer_uvb{t}{i1} = resample_flow(layer_uvb{t}{i1}, nsz);
        end;
    end;    
    
    % Rescale the hidden field 
    for t=1:T;  layer_g{t} = imresize(layer_g{t}, nsz);  end;      
    
    % Rescale the soft label
    for t=1:T;
        layerLabels{t}   = imresize(layerLabels{t}, nsz);        
        % Does not normalize strictly
        layerLabels{t}   = layerLabels{t}./repmat(sum(layerLabels{t},3), [1 1 this.nLayers]);        
    end;
    
    
    for i = 1:this.max_iters
        
        % Anneal lambda4 (sigmoid)
        r = small.lambda4/this.lambda4s(l,i);
        small.lambda4 = this.lambda4s(l,i);        
        
        if this.anneal_same_s
            
            % Scale g accordingly so that sigma(lambda4*g) retains the same
            for t=1:T
                layer_g{t} = layer_g{t}*r;
            end;            

            small.lambda2 = small.lambda2/(r)^2;
            small.lambda3 = small.lambda3/(r)^2;
            small.lambda7 = small.lambda7/(r)^2;
        end;
        
        % anneal lambda2 for the learned model only, quick test
%         small.lambda2 = small.lambda2*18.8; fprintf('%3.3e\t', small.lambda2);
        

%% Update by cgd: flow and hidden fields together        
%         for t=1:T
%             layerLabels{t} = g2labels(layer_g{t}, small.lambda4);
%         end;        
%         for t=1:T-1
%             layer_mean_affine_uv{t}  = fit_affine_layer_motion(layer_uv{t}, layerLabels{t});
%             layer_mean_affine_uvb{t} = fit_affine_layer_motion(layer_uvb{t}, layerLabels{t+1});
%         end
% 
%         [layer_g layer_uv layer_uvb] = estimate_flow_hidden_field_3D_cgd(small, ...
%             layer_uv, layer_g, layer_uvb, layer_mean_affine_uvb, layer_mean_affine_uv, pyramid_images{l}, color_pyramid_images{l});
%         for t=1:T
%             layerLabels{t} = g2labels(layer_g{t}, small.lambda4);
%         end;  
%% end of update by cgd for both
        
        %Update hidden fields simultaneously given current flow estimate
        layer_g = estimate_hidden_field_3D_cgd(small, layer_uv, layer_g, layer_uvb, pyramid_images{l}, color_pyramid_images{l});
        for t=1:T
            layerLabels{t} = g2labels(layer_g{t}, small.lambda4);
        end;       

        for t=1:T-1           
        
            % Forward flow field
            small.images = cat(3, pyramid_images{l}(:,:,t), pyramid_images{l}(:,:,t+1));            
            if size(color_pyramid_images{l}, 4) > 1
                small.labImage    = color_pyramid_images{l}(:,:,:,t);
            else
                small.labImage    = color_pyramid_images{l}(:,:,t);                
            end;
            
            % Fit mean affine flow field
%             layer_mean_affine_uv = fit_affine_layer_motion(layer_uv{t}, layerLabels{t});

            switch small.meanMode
                case 'projective'
                    layer_mean_affine_uv = fit_projective_layer_motion_g(layer_uv{t}, layer_g{t});
                otherwise
                    layer_mean_affine_uv = fit_affine_layer_motion_g(layer_uv{t}, layer_g{t});
            end;
            
            %layer_uv{t} = layer_mean_affine_uv;    % affine flow field + hidden field
                       
            for i1 = 1:this.nLayers;
                
                if ~small.affine_mean 
                    % no affine mean
                    layer_mean_affine_uv{i1} = 0*layer_mean_affine_uv{i1};
                end;
                small.mean_uv     = layer_mean_affine_uv{i1};
                
                % Multiply warped labels at next frame for dataterm
                tmp = imwarp_grad_im(layer_uv{t}{i1}, layerLabels{t+1}(:,:,i1));
                small.layer_s     = cat(3, layerLabels{t}(:,:,i1), tmp);                
                tmp(isnan(tmp))   = 0;
                small.layerLabel  = layerLabels{t}(:,:,i1).*tmp;
                                
                % Run flow method on subsampled images
                if i1 < this.nLayers
                    % hidden field g is used in data term
                    layer_uv{t}{i1} = compute_flow_base(small, layer_uv{t}{i1}, layer_g{t}(:,:,i1), layer_g{t+1}(:,:,i1));
                else
                    layer_uv{t}{i1} = compute_flow_base(small, layer_uv{t}{i1});
                end;
                
            end;
            
            % Backward flow field            
            % Fit mean affine flow field
%             layer_mean_affine_uv = fit_affine_layer_motion(layer_uvb{t}, layerLabels{t+1}); 
            
            %layer_uvb{t} = layer_mean_affine_uv;            
            
            switch small.meanMode
                case 'projective'                            
                    layer_mean_affine_uv = fit_projective_layer_motion_g(layer_uvb{t}, layer_g{t+1}); 
                otherwise
                    layer_mean_affine_uv = fit_affine_layer_motion_g(layer_uvb{t}, layer_g{t+1}); 
            end;
            
            small.images = cat(3, pyramid_images{l}(:,:,t+1), pyramid_images{l}(:,:,t));            
            if size(color_pyramid_images{l}, 4) > 1
                small.labImage    = color_pyramid_images{l}(:,:,:,t+1);
            else
                small.labImage    = color_pyramid_images{l}(:,:,t+1);
            end;
            
            for i1 = 1:this.nLayers;

                if ~small.affine_mean
                    % no affine mean
                    layer_mean_affine_uv{i1} = 0*layer_mean_affine_uv{i1};
                end;
                small.mean_uv   = layer_mean_affine_uv{i1};
                
                %multiply warped labels
                tmp = imwarp_grad_im(layer_uvb{t}{i1}, layerLabels{t}(:,:,i1));
                small.layer_s     = cat(3, layerLabels{t+1}(:,:,i1), tmp);
                
                tmp(isnan(tmp)) = 0;
                small.layerLabel= layerLabels{t+1}(:,:,i1).*tmp;
                
                % Run flow method on subsampled images
                if i1 < this.nLayers
                    layer_uvb{t}{i1} = compute_flow_base(small, layer_uvb{t}{i1}, layer_g{t+1}(:,:,i1), layer_g{t}(:,:,i1));
                else
                    layer_uvb{t}{i1} = compute_flow_base(small, layer_uvb{t}{i1});
                end;
            end;

        end;

% % % % % Does not work        
% % %         layer_uv = process_composite_flow(layer_uv
% %         % forward
% %         for t=1:T-1           
% %             tmp_uv = composite_final_flow(layer_uv{t}, layerLabels{t});
% %             occ = detect_occlusion(tmp_uv, pyramid_images{l}(:,:,t:t+1));            
% %             
% %             if size(color_pyramid_images{l}, 4) > 1
% %                 labImage    = color_pyramid_images{l}(:,:,:,t);
% %             else
% %                 labImage    = color_pyramid_images{l}(:,:,t);
% %             end;
% %           
% %             figure; subplot(1,2,1); imshow(flowToColor(tmp_uv));
% %             
% %             tmp_uv = denoise_color_weighted_medfilt2(tmp_uv, labImage, occ, this.area_hsz_final, this.median_filter_size, this.sigma_mi, this.fullVersion);
% %             
% %             [tmp ind ] = max(layerLabels{t}, [], 3);
% %             ind = repmat(ind, [1 1 2]);
% %             
% %             for i1 = 1:length(layer_uv{t})
% %                 layer_uv{t}{i1}(ind ==i1) = tmp_uv(ind==i1);
% %             end;
% %             
% %             subplot(1,2,2); imshow(flowToColor(tmp_uv));
% %         end;
% %         
% %         
% %         
% %         % backward
% %         for t=1:T-1           
% %             tmp_uv = composite_final_flow(layer_uvb{t}, layerLabels{t+1});
% %             occ = detect_occlusion(tmp_uv, pyramid_images{l}(:,:,t+1:-1:t));            
% %             
% %             if size(color_pyramid_images{l}, 4) > 1
% %                 labImage    = color_pyramid_images{l}(:,:,:,t+1);
% %             else
% %                 labImage    = color_pyramid_images{l}(:,:,t+1);
% %             end;
% %             
% %             tmp_uv = denoise_color_weighted_medfilt2(tmp_uv, labImage, occ, this.area_hsz_final, this.median_filter_size, this.sigma_mi, this.fullVersion);
% %             
% %             [tmp ind ] = max(layerLabels{t+1}, [], 3);
% %             ind = repmat(ind, [1 1 2]);
% %             
% %             for i1 = 1:length(layer_uvb{t})
% %                 layer_uvb{t}{i1}(ind ==i1) = tmp_uv(ind==i1);
% %             end;
% %         end;
        
        
        % For debug;
%         fprintf('%3.3e\t, %3.3e\n', mean(layerLabels{1}(:)), mean(abs(layer_g{1}(:))));
%         tmp = composite_final_flow(layer_uv{1}, layerLabels{1});
%         figure; subplot(1,2,1); imshow(flowToColor(tmp));
%         tmp = composite_final_flow(layer_uvb{1}, layerLabels{2});
%         subplot(1,2,2); imshow(flowToColor(tmp));
%         tmp = composite_final_flow_g(layer_uv{1}, layer_g{1});
%         figure; subplot(1,2,1); imshow(flowToColor(tmp));
%         tmp = composite_final_flow_g(layer_uvb{1}, layer_g{2});
%         subplot(1,2,2); imshow(flowToColor(tmp));
        
        % NLM term on the composite flow field, update valid pixels of each
        % layer 
    end
end;
