function [uv seg]= compute_flow_nframes(this, init, gt)
%%
%COMPUTE_FLOW   Compute flow field
%   UV = COMPUTE_FLOW(THIS[, INIT]) computes the flow field UV with
%   algorithm THIS and the optional initialization INIT.
%  

%   Compute forward and backward flow fields at the same time, with
%   occlusion reasoning

%   This is a member function of the class 'layer_optical_flow'. 

%   Author: Deqing Sun, Department of Computer Science, Brown University
%   Contact: dqsun@cs.brown.edu
%   $Date: 2007-- $

% Copyright 2007-2008, Brown University, Providence, RI.
%
%                         All Rights Reserved
%
% Permission to use, copy, modify, and distribute this software and its
% documentation for any purpose other than its incorporation into a
% commercial product is hereby granted without fee, provided that the
% above copyright notice appear in all copies and that both that
% copyright notice and this permission notice appear in supporting
% documentation, and that the name of the author and Brown University not be used in
% advertising or publicity pertaining to distribution of the software
% without specific, written prior permission.
%
% THE AUTHOR AND BROWN UNIVERSITY DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
% INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR ANY
% PARTICULAR PURPOSE.  IN NO EVENT SHALL THE AUTHOR OR BROWN UNIVERSITY BE LIABLE FOR
% ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
% WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
% ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
% OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.  

tic

this.lambda4 = this.lambda4s(this.pyramid_levels, 1);
% Compute initial value and image pyramid
[ini_layerLabels, ini_layerLabels2, ini_layer_uv, ini_layer_uv2,...
    pyramid_images, color_pyramid_images, layer_g, layer_g2] = compute_init(this, init);

% test all possible permutations
% all_orders = perms(1:length(ini_layer_uv));

% fast to slow order
all_orders = 1:length(ini_layer_uv);

% slow to fast
% all_orders = length(ini_layer_uv):-1:1;

% two candidates
% all_orders = [ 1:length(ini_layer_uv); length(ini_layer_uv):-1:1];


minEn = 1e200;

for l = 1:size(all_orders, 1);

    % Test every possible order
    layerLabels = ini_layerLabels(:,:,all_orders(l,:));
    layerLabels2 = ini_layerLabels2(:,:,all_orders(l,:));
    for j = 1:size(all_orders, 2)
        layer_uv{j} = ini_layer_uv{all_orders(l,j)};
        layer_uv2{j} = ini_layer_uv2{all_orders(l,j)};
    end;
    layer_g = label2g(layerLabels, this.lambda4);
    layer_g2 = label2g(layerLabels2, this.lambda4);
    
    [small layer_uv layer_g layerLabels small2 layer_uv2 layer_g2 layerLabels2]  = ...
    compute_flow_hidden_field(this, layerLabels, layerLabels2, layer_uv,...
    layer_uv2, pyramid_images, color_pyramid_images, layer_g, layer_g2);


%     fn = sprintf([this.fileinfo 'flow_file_%03d.mat'], l);
%     save(fn, 'small', 'layer_uv', 'layer_g', 'layer_g2', 'small2', 'layer_uv2');

    en  = evaluate_energy(small, layer_uv, layer_g, layer_g2) + evaluate_energy(small2, layer_uv2, layer_g2, layer_g);
    if minEn > en
        minEn = en;
        min_layerLabels = layerLabels;
        min_layerLabels2 = layerLabels2 ;
        min_layer_uv = layer_uv;
        min_layer_uv2 = layer_uv2;
        min_layer_g = layer_g;
        min_layer_g2 = layer_g2;
    end;
    

    uv = composite_final_flow(layer_uv, layerLabels);
    
    % for debug
    fprintf('energy \t%3.3e\t', en);    
    if nargin == 3
        [aae stdae aepe] = flowAngErr(gt(:,:,1), gt(:,:,2), uv(:,:,1), uv(:,:,2), 0);
        fprintf('AAE %3.3f STD %3.3f average end point error %3.3f \n', aae, stdae, aepe);
    end;
%     [tmp ind ] = max(layerLabels, [], 3);
%     figure; subplot(1,2,1); imagesc(ind); axis square;
%     subplot(1,2,2); imshow(flowToColor(uv)); axis square;
end;

        

% this2 = this;
% this2.pyramid_levels = 1;
% this2.max_iters      = 1;

% swap depth order, perform 1 warping, compare energy

layerLabels = min_layerLabels;
layerLabels2 = min_layerLabels2 ;
layer_uv = min_layer_uv;
layer_uv2 = min_layer_uv2;
layer_g = min_layer_g;
layer_g2 = min_layer_g2;


% for debug      
% figure; 
% [tmp ind ] = max(layerLabels, [], 3);
% subplot(1,2,1); imagesc(ind); axis square;
% [tmp ind ] = max(layerLabels2, [], 3);
% subplot(1,2,2); imagesc(ind); axis square;
     
% Compose final flow field
uv = composite_final_flow(layer_uv, layerLabels);

%Detect occlusion by checking inconsistency between label and warped label
[tmp ind ] = max(layerLabels, [], 3);
seg(:,:,1) = ind;

[tmp2 ind2 ] = max(layerLabels2, [], 3);
wind2 = warp_backward(uv, ind2, 'nearest');

seg(:,:,2) = ind == wind2;    % dark region: occlusion

if true; %this.display    
    en = evaluate_energy(small, layer_uv, layer_g, layer_g2) + evaluate_energy(small2, layer_uv2, layer_g2, layer_g);
    fprintf('energy \t%3.3e\t', en);    
    [aae stdae aepe] = flowAngErr(gt(:,:,1), gt(:,:,2), uv(:,:,1), uv(:,:,2), 0);
    fprintf('AAE %3.3f STD %3.3f average end point error %3.3f \n', aae, stdae, aepe);
end;

fprintf('flow estimation finished, %3.2f minutes passed \t \t', toc/60);


% composte the illustration image
sz = size(uv);
nL = this.nLayers;
all = zeros(sz(1)*nL, sz(2)*3, 3);
 
for i = 1:nL; all((i-1)*sz(1)+1:sz(1)*i, 1:sz(2), :) = flowToColor(layer_uv{i}); end;
for i = 1:nL-1; all((i-1)*sz(1)+1:sz(1)*i, 1+sz(2):2*sz(2), :) = repmat(scale_image(layer_g(:,:,i), 0, 255), [1 1 3]); end;
all((nL-1)*sz(1)+1:sz(1)*nL, 1+sz(2):2*sz(2), :) = flowToColor(uv); 

f(:,:,1) = cos(ind*2*pi/nL);
f(:,:,2) = sin(ind*2*pi/nL);
all((nL-1)*sz(1)+1:sz(1)*nL, 1+sz(2)*2:sz(2)*3, :) = flowToColor(f);

for i = 1:nL-1; 
    tmp = layer_g(:,:,i)>=0;
    for j = 1:i-1;
        tmp = tmp & layer_g(:,:,j)<=0;
    end;    
    all((i-1)*sz(1)+1:sz(1)*i, 1+sz(2)*2:sz(2)*3, :) = repmat(tmp, [1 1 3])*255;
end;

% all(1:sz(1)*2, 1+sz(2)*3:sz(2)*4, :) = [this.colorImages(:,:,:,1); this.colorImages(:,:,:,2)];
% all(sz(1)*2+1:sz(1)*3, 1+sz(2)*3:sz(2)*4, :) = repmat(ind==wind2, [1 1 3])*255;
imwrite(uint8(all), 'illustration.png');



% for i = 1:this.nLayers
%     figure; imshow(flowToColor(layer_mean_affine_uv{i}));
% end;
% for debug      
% save([this.fileinfo '.mat'], 'layer_uv', 'layerLabels', 'layer_uv2', 'layerLabels2', 'layer_g2', 'layer_g');      

function [small layer_uv layer_g layerLabels small2 layer_uv2 layer_g2 layerLabels2]  = ...
    compute_flow_hidden_field(this, layerLabels, layerLabels2, layer_uv,...
    layer_uv2, pyramid_images, color_pyramid_images, layer_g, layer_g2)

% Interpolate invisible pixels for each layer by affine motion model
layer_uv = affine_interpolate_invisible(layer_uv, layerLabels);
layer_uv2 = affine_interpolate_invisible(layer_uv2, layerLabels2);
% fprintf('no affine interpolation');

% Iterate through all pyramid levels starting at the top
for l = this.pyramid_levels:-1:1
    
    if this.display
        disp(['-Pyramid level: ', num2str(l)])
    end
    % en = evaluate_energy(small, layer_uv, layer_g, layer_g2) + evaluate_energy(small2, layer_uv2, layer_g2, layer_g);
    % fprintf('energy \t%3.3e\n', en);
    [small layer_g layer_uv small2 layer_g2 layer_uv2] = scale_images(this, l,...
        pyramid_images, color_pyramid_images, layer_uv, layer_g, layer_uv2, layer_g2);
    for i = 1:this.max_iters
        
        % Anneal lambda4 (sigmoid)
        small.lambda4 = this.lambda4s(l,i);
        small2.lambda4 = this.lambda4s(l,i);
        
        % Update hidden fields simultaneously
        [layer_g layer_g2] = estimate_layer_labels_sym_frames_sim_gd(small, small2, layer_uv, layer_g, layer_uv2, layer_g2);
        layerLabels = g2labels(layer_g, small.lambda4);
        layerLabels2 = g2labels(layer_g2, small2.lambda4);
        
        % Fit mean affine flow field
        layer_mean_affine_uv = fit_affine_layer_motion(layer_uv, layerLabels);
        layer_uv = layer_mean_affine_uv;
        
%         for i1 = 1:this.nLayers;
%             
%             % no affine mean
%             %layer_mean_affine_uv{i1} = 0*layer_mean_affine_uv{i1};
%             
%             %multiply warped labels2
%             tmp = imwarp_grad_im(layer_uv{i1}, layerLabels2(:,:,i1));
%             tmp(isnan(tmp))   = 0;
%             small.layerLabel  = layerLabels(:,:,i1).*tmp;
%             small.mean_uv     = layer_mean_affine_uv{i1};
%             
%             
%             % Run flow method on subsampled images
%             if i1 < this.nLayers
%                 % hidden field g is used in data term
%                 layer_uv{i1} = compute_flow_base(small, layer_uv{i1}, layer_g(:,:,i1), layer_g2(:,:,i1));
%             else
%                 layer_uv{i1} = compute_flow_base(small, layer_uv{i1});
%             end;
%             
%         end;
        
        % Fit mean affine flow field
        layer_mean_affine_uv = fit_affine_layer_motion(layer_uv, layerLabels);
        layer_uv2 = layer_mean_affine_uv;
        disp('no deformation flow field');
%         for i1 = 1:this.nLayers;
% 
%             % no affine mean
%             %layer_mean_affine_uv{i1} = 0*layer_mean_affine_uv{i1};
%             
%             %multiply warped labels
%             tmp = imwarp_grad_im(layer_uv2{i1}, layerLabels(:,:,i1));
%             tmp(isnan(tmp)) = 0;
%             small2.layerLabel = layerLabels2(:,:,i1).*tmp;
%             small2.mean_uv     = layer_mean_affine_uv{i1};
%             
%             % Run flow method on subsampled images
%             if i1 < this.nLayers
%                 layer_uv2{i1} = compute_flow_base(small2, layer_uv2{i1}, layer_g2(:,:,i1), layer_g(:,:,i1));
%             else
%                 layer_uv2{i1} = compute_flow_base(small2, layer_uv2{i1});
%             end;
%         end;
    end
end;

function   [small layer_g layer_uv small2 layer_g2 layer_uv2] = scale_images(this, l, pyramid_images, color_pyramid_images, layer_uv, layer_g, layer_uv2, layer_g2)

% Generate copy of algorithm with single pyramid level and the
% appropriate subsampling
small = this;
small2 = this;

nsz               = [size(pyramid_images{l}, 1) size(pyramid_images{l}, 2)];
small.images      = pyramid_images{l};
small2.images     = pyramid_images{l}(:,:,end:-1:1);

if size(color_pyramid_images{l}, 4) >1
    im1   = color_pyramid_images{l}(:,:,:,1);
    im2   = color_pyramid_images{l}(:,:,:,2);
else
    im1   = color_pyramid_images{l}(:,:,1);
    im2   = color_pyramid_images{l}(:,:,2);
end;

% Rescale the flow field and labels
for i1 = 1:this.nLayers;
    layer_uv{i1}  = resample_flow(layer_uv{i1}, nsz);
    layer_uv2{i1} = resample_flow(layer_uv2{i1}, nsz);
end;

% Layer ownership / s
%           layerLabels   = imresize(layerLabels, nsz);
%           layerLabels   = layerLabels./repmat(sum(layerLabels,3), [1 1 this.nLayers]);
%
%           layerLabels2  = imresize(layerLabels2, nsz);
%           layerLabels2  = layerLabels2./repmat(sum(layerLabels2,3), [1 1 this.nLayers]);

layer_g       = imresize(layer_g, nsz);
layer_g2      = imresize(layer_g2, nsz);

% Segment static image
%           [tmp small.seg]  = msseg(im1,7,6.5, round(50/pyramid_spacing.^(l-1)) );
%           [tmp small2.seg] = msseg(im2,7,6.5, round(50/pyramid_spacing.^(l-1)) );

% Convert color from RGB to LAB space
if size(im1, 3) > 1
    im1 = RGB2Lab(im1);          % to Lab space
    for j = 1:size(im1, 3);
        im1(:,:,j) = scale_image(im1(:,:,j), 0, 255);
    end;
    
    im2 = RGB2Lab(im2);          % to Lab space
    for j = 1:size(im2, 3);
        im2(:,:,j) = scale_image(im2(:,:,j), 0, 255);
    end;
end;
small.labImage    = im1;
small2.labImage   = im2;