function [uv all uvb allb]= compute_flow_sym(this, init)

%%
%   UV = COMPUTE_FLOW_SYM(THIS[, INIT]) computes forward (and backward
%   flow, this.is_symmetric==true) fields at the same time, with occlusion
%   reasoning 

%   This is a member function of the class 'layer_optical_flow_3D'. 

tic

if length(init{1}) ==1
    % always uses symmetric for 2 frames
    this.is_symmetric = true;
end;

outputDir = this.extParams.contDir;

this.lambda4 = this.lambda4s(this.pyramid_levels, 1);

[pyramid_images, color_pyramid_images, grayImages] = build_image_pyramid(this);

this.extParams.grayImages = grayImages;
%%
[ini_layer_uv, ini_layer_uvb, seg, incon, ini_layer_g] =....
    compute_init_layer_support_all(this, init, ...
    pyramid_images, color_pyramid_images);

% T frames have T-1 forward flow fields
T           = length(ini_layer_uv)+1; 

if ~this.cont_ref
    
    if nargout >2
        [uv all uvb allb] = composite_final_output([], ...
            ini_layer_uv, ini_layer_uvb, ini_layer_g, T, this);
    else
        [uv all ] = composite_final_output([], ...
            ini_layer_uv, ini_layer_uvb, ini_layer_g, T, this);
    end;
    
    return;
end;


% discrete optimization may change the number of layers to use
this.nLayers = length(ini_layer_uv{1});

all_orders  = obtain_depth_orders_to_enumerate(this.depth_order,this.nLayers);

minEn       = 1e200;
txtfn       = [outputDir 'energy.txt'];
txtfp       = fopen(txtfn, 'w');

fprintf(txtfp, 'depth order \t \t \t \t data\t spatial g\t consistent \t spatial flow\t nlm\t total\n');    

for l = 1:size(all_orders, 1);
    
    fprintf(txtfp, 'near to far\t');    
    for ti1 =1:size(all_orders,2)
        fprintf(txtfp, '%d\t', all_orders(l, ti1));
    end;
%
    % Test selected depth orderings, move to discrete optimzation?
    if ~this.gc_init
        % does not use discrete optimization
        tmp_seg = seg;
        for t=1:T
            for l1=1:this.nLayers
                tmp_seg{t}(seg{t}==all_orders(l, l1)) = l1;
            end;
        end;
        [layer_g layerLabels] = convert_ini_seg_to_support_function(this, tmp_seg, incon);
    else
        % use discrete optimization
        layer_g = ini_layer_g;
    end;
    
    for t=1:T
        layerLabels{t} = g2labels(layer_g{t}, this.lambda4);
    end;
%    
    for t=1:T-1
        for j = 1:this.nLayers
            layer_uv{t}{j}  = ini_layer_uv{t}{all_orders(l,j)};
            layer_uvb{t}{j} = ini_layer_uvb{t}{all_orders(l,j)};
        end;
    end;    
       
    % Save Initial results
    if ~isempty(outputDir)
        for t=1:T-1
            tmp_uv = composite_final_flow_g(layer_uv{t}, layer_g{t});            
            fn = [outputDir sprintf('init_t%03d_depth%03d.flo', t, l)];
            writeFlowFile(tmp_uv, fn);
            
            tmp_all= composite_illustration_image(tmp_uv, this.nLayers,...
                     layer_uv{t}, layer_g{t});
            fn = [outputDir sprintf('_ini_t%03d_depth%03d.png', t, l)];
            imwrite(uint8(tmp_all), fn);
        end;
    end;    
   
    % Evaluate energy of initial solutions
    
    if this.cont_ref 
    
        this2 = this; this2.lambda4 = this.lambda4s(1, end);
        fprintf('Init energy ');
        en = evaluate_energy(this2, layer_g, layer_uv, layer_uvb,...
            pyramid_images{1}, color_pyramid_images{1});
        fprintf('Init energy %3.3e\n', en);
    else

        if nargout >2
            [uv all uvb allb] = composite_final_output(layerLabels, ...
                layer_uv, layer_uvb, layer_g, T, this);
        else
            [uv all ] = composite_final_output(layerLabels, ...
                layer_uv, layer_uvb, layer_g, T, this);
        end;

        return;
    end;
    
    
    this.fileinfo = [this.extParams.contDir sprintf('order_%d', l)];
    if ~exist(this.fileinfo, 'file'); 
        mkdir(this.fileinfo); 
    end;

    
    
%% Call subprogram to estimate flow and hidden fields           
    if this.cont_ref 
        % for debug    
        [small layer_uv layer_g layerLabels layer_uvb]  = ...
            compute_flow_hidden_field(this, layerLabels, layer_uv,...
            layer_uvb, pyramid_images, color_pyramid_images, layer_g);
    else
        small = this;
        small.lambda4 = this.lambda4s(1, end);
    end;
%%
    % note lambda2 and lambda3 are adjusted during annealing
    % Evaluate energy of final solutions
    
    if this.cont_ref 
    
        fprintf('Final energy ');
        [en enind] = evaluate_energy(small, layer_g, layer_uv, layer_uvb,...
            pyramid_images{1}, color_pyramid_images{1});        
        %fprintf('Final energy %3.3e\n', en);        
    else 
        en = minEn -1e1;
    end;
    
    
    % Keep lowest-energy solution
    if minEn > en
        minEn = en;
        min_layerLabels = layerLabels;
        min_layer_uv = layer_uv;
        min_layer_uvb = layer_uvb;
        min_layer_g = layer_g;
    end;
    
    % Save intermediate results
    if ~isempty(outputDir)                

        for t=1:T-1    
            tmp_uv = composite_final_flow_g(layer_uv{t}, layer_g{t});
            fn = [outputDir sprintf('t%03d_depth%03d.flo', t, l)];
            writeFlowFile(tmp_uv, fn);
            tmp_all= composite_illustration_image(tmp_uv, this.nLayers,...
                layer_uv{t}, layer_g{t});
            fn = [outputDir sprintf('t%03d_depth%03d.png', t, l)];
            imwrite(uint8(tmp_all), fn);       
            
            % Save occulusion detection results
            seg1 = threshold_g(layer_g{t});
            seg2 = threshold_g(layer_g{t+1});            

            wseg2 = warp_backward(tmp_uv, seg2, 'nearest');
            tmp_occ = (seg1 == wseg2)*255;
            fn = [outputDir sprintf('_occ_t%03d_depth%03d.png', t, l)];
            imwrite(uint8(tmp_occ), fn);                                    

            if this.is_symmetric
                tmp_uv = composite_final_flow_g(layer_uvb{t}, layer_g{t+1});
                fn = [outputDir sprintf('t%03d_depth%03d_back.flo', t, l)];
                writeFlowFile(tmp_uv, fn);
                tmp_all= composite_illustration_image(tmp_uv, this.nLayers,...
                    layer_uvb{t}, layer_g{t+1});
                fn = [outputDir sprintf('t%03d_depth%03d_back.png', t, l)];
                imwrite(uint8(tmp_all), fn);
            end;
        end;

    end;
    
    % Output individual energy term
    for ti = 1:length(enind)
        fprintf(txtfp, '%3.3e\t', enind(ti));
    end;    
    % Output total energy
    fprintf(txtfp, '%3.3e\n', en);
    
end;
fclose(txtfp);

% Composite flow fields / illustration image with segmentation information 
if nargout >2 
    [uv all uvb allb] = composite_final_output(min_layerLabels, ...
        min_layer_uv, min_layer_uvb, min_layer_g, T, this);
else
    [uv all ] = composite_final_output(min_layerLabels, ...
        min_layer_uv, min_layer_uvb, min_layer_g, T, this);
end;

% initial value
t = 1;
ini_uv  = composite_final_flow_g(ini_layer_uv{t}, ini_layer_g{t});
ini_all = composite_illustration_image(ini_uv, this.nLayers,...
    ini_layer_uv{t}, ini_layer_g{t});
all{T} = ini_all; % initial value for the first frame

fprintf('flow estimation finished, %3.2f minutes passed \n', toc/60);

for t=1:T
    seg{t} = threshold_g(min_layer_g{t});
end;

outputDir = [this.extParams.finalDir this.extParams.info '/'];
if ~exist(outputDir, 'file')
    mkdir(outputDir);
end;

if this.is_save_mat
    fn = [outputDir 'result.mat'];
    % save(fn, 'min_layer_g', 'seg');
    save(fn, 'min_layer_uv', 'min_layer_uvb', 'min_layer_g', 'seg');
end;