function [layer_uv, layer_uvb, layer_g]= compute_support_flow(this, init)

%%
%   UV = COMPUTE_FLOW_SYM(THIS[, INIT]) computes forward (and backward
%   flow, this.is_symmetric==true) fields at the same time, with occlusion
%   reasoning 

%   This is a member function of the class 'layer_optical_flow_3D'. 

%keyboard;

if length(init{1}) ==1
    % always uses symmetric for 2 frames
    this.is_symmetric = true;
end;

this.lambda4 = this.lambda4s(this.pyramid_levels, 1);

% data = build_image_pyramid(this);
[pyramid_images, color_pyramid_images, grayImages] = build_image_pyramid(this);

this.extParams.grayImages = grayImages;
%keyboard;
[ini_layer_uv, ini_layer_uvb, seg, incon, ini_layer_g] =....
    compute_init_layer_support_all(this, init, ...
    pyramid_images, color_pyramid_images);

if ~this.cont_ref
    layer_uv = ini_layer_uv;
    layer_uvb= ini_layer_uvb;
    layer_g  = ini_layer_g;    
    return;
end

% T frames have T-1 forward flow fields
T   = length(ini_layer_uv)+1; 
% discrete optimization may change the number of layers to use
this.nLayers = length(ini_layer_uv{1}); 

all_orders   = obtain_depth_orders_to_enumerate(this.depth_order,this.nLayers);

minEn   = 1e200;

fprintf(['depth order \t \t \t \t data\t spatial' ... 
    ' g\t consistent \t spatial flow\t nlm\t total\n']);    

for l = 1:size(all_orders, 1);
    
    fprintf('near to far\t');    
    fprintf('%d\t', all_orders(l, :));
    
    
    %[layer_uv layer_uvb layer_g]= set_init_specified_order();
    
    % Test selected depth orderings, move to discrete optimzation
    if ~this.gc_init
        % does not use discrete optimization
        tmp_seg = seg;
        for t=1:T
            for l1=1:this.nLayers
                tmp_seg{t}(seg{t}==all_orders(l, l1)) = l1;
            end;
        end;
        layer_g = convert_ini_seg_to_support_function(this, tmp_seg, incon);
    else
        % use discrete optimization
        layer_g = ini_layer_g;
    end;
    
    for t=1:T-1
        for j = 1:this.nLayers
            layer_uv{t}{j}  = ini_layer_uv{t}{all_orders(l,j)};
            layer_uvb{t}{j} = ini_layer_uvb{t}{all_orders(l,j)};
        end;
    end;           
  
    % Evaluate energy of initial solutions    
    this2 = this; this2.lambda4 = this.lambda4s(1, end);
    en = evaluate_energy(this2, layer_g, layer_uv, layer_uvb,...
        pyramid_images{1}, color_pyramid_images{1});
    fprintf('Init energy %3.3e\n', en);    
       
    
    %%%%%%  Call subprogram to estimate flow and hidden fields
    [small layer_uv layer_g layer_uvb]  = ...
        compute_support_flow_c2f(this, layer_uv,...
        layer_uvb, pyramid_images, color_pyramid_images, layer_g);
    
    % Evaluate energy of new solution
    [en enind] = evaluate_energy(small, layer_g, layer_uv, layer_uvb,...
        pyramid_images{1}, color_pyramid_images{1});

    % Output individual energy term  & total energy
    fprintf('%3.3e\t', enind);        
    fprintf('%3.3e\n', en);
    
    % Keep lowest-energy solution
    if minEn > en
        minEn = en;
        min_layer_uv    = layer_uv;
        min_layer_uvb   = layer_uvb;
        min_layer_g     = layer_g;
    end;     
    
end;

layer_uv    = min_layer_uv;
layer_uvb   = min_layer_uvb;
layer_g     = min_layer_g;
