function [layer_g layerLabels] = convert_ini_seg_to_support_function(this, seg, incon)
%%
% Convert the initial hard segmentation to the soft support functions
% the depth order is (from near to far): segment 1, segment 2, ...
%   *visible pixels in each layer is assigned support function this.this.visG,
%   *invisible pixels are assigned -this.this.visG
%   *inconsistency (occluded) pixels: support function divided by
%       this.ratioG>1 to reduce the influence of occlusion 

T  = length(seg);
sz = size(seg{1});
nL = size(incon{1},3)+1;

layer_g     = cell(1,T);
layerLabels = cell(1,T);

for t=1:T 
    layer_g{t} = - ones([sz nL-1]) * this.visG;
    for l=1:nL-1
        tmp = layer_g{t}(:,:,l);
        tmp(seg{t} ==l)   =  this.visG;
        layer_g{t}(:,:,l) = tmp;     
        % for pixels who are occluded by previous layers, set the support
        % function to be large
        % for l1=l+1:size(layer_g{t},3)
        %     tmp = layer_g{t}(:,:,l1);
        %     tmp(seg{t} ==l) =  0;
        %     layer_g{t}(:,:,l1) = tmp;
        % end;        
    end;       
    % mark inconsistency    
    layer_g{t}(incon{t} ==1) = layer_g{t}(incon{t} ==1)/this.ratioG;    
    layerLabels{t} = g2labels(layer_g{t}, this.lambda4);
end;
