function [layer_uvf, layer_uvb, layer_g, incon] =...
            convert_init_seg_to_support(this, init, seg)
%%
% compute affine clustering of flow field at center key frame, propagate
% the resutls to other frames

uvf = init{1};
uvb = init{2};
T   =  length(uvf)+1;
nL  = max(seg{1}(:));


% if there's a prior, use it 
if ~isempty(this.prior)
    if this.isstuff
        uvf{2} = this.prior{1};
        uvb{2} = this.prior{2};
    else
        uvf{1} = this.prior{1};
        uvb{1} = this.prior{2};
    end
end

% compute initial flow field for each layer
%   visible pixels: use initial flow field
%   invisible ones: use affine motion to interpolate
layer_uvf = cell(1,T-1); 
layer_uvb = cell(1,T-1); 
for t=1:T-1
    for i=1:nL
        layer_uvf{t}{i}  = uvf{t};
        layer_uvb{t}{i}  = uvb{t};
    end;
end;

% order layer by fast to slow order
this.is_hand_seg = 1;

% Compute hidden fields (support function) for each layer
incon = cell(1,T-1);
for t=1:T-1
    tmp      = warp_backward_segmentation(seg{t+1}, uvf{t});
    incon{t} = repmat(tmp~=seg{t}, [1 1 nL-1]);
end;
tmp = warp_backward_segmentation(seg{T-1}, uvb{T-1});
t   = T; 
incon{t} = repmat(tmp~=seg{t}, [1 1 nL-1]);

layer_g  = convert_ini_seg_to_support_function(this, seg, incon); 
  

for t=1:T-1

    if this.affine_ini_all
        
        % Use affine motion for all the pixels
        layer_uvf{t} = fit_affine_layer_motion_g(layer_uvf{t}, layer_g{t}, this.dataset_name);
        layer_uvb{t} = fit_affine_layer_motion_g(layer_uvb{t}, layer_g{t+1}, this.dataset_name);
    
    else
        
        % Interpolate invisible pixels for each layer
        if isempty(this.prior) || this.isstuff || t==2 

            tmp = layer_g{t}; 
            layer_uvf{t} = affine_interpolate_invisible_g(layer_uvf{t},...
                tmp, this.meanMode, this.rho_affine, this.dataset_name);

            tmp = layer_g{t+1};
            layer_uvb{t} = affine_interpolate_invisible_g(layer_uvb{t},...
                tmp, this.meanMode, this.rho_affine, this.dataset_name);

        end

    end;
end;
