function E = evaluate_energy_ba(this, im1, im2, u, v)
%EVALUATE_ENERGY_BA computes the energy function of the flow field U

%   Author: Deqing Sun, Department of Computer Science, Brown University
%   Contact: dqsun@cs.brown.edu
%   $Date: 2007-11-30 $
%
% Copyright 2007-2008, Brown University, Providence, RI.
%
%                         All Rights Reserved
%
% Permission to use, copy, modify, and distribute this software and its
% documentation for any purpose other than its incorporation into a
% commercial product is hereby granted without fee, provided that the
% above copyright notice appear in all copies and that both that
% copyright notice and this permission notice appear in supporting
% documentation, and that the name of the author and Brown University not be used in
% advertising or publicity pertaining to distribution of the software
% without specific, written prior permission.
%
% THE AUTHOR AND BROWN UNIVERSITY DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
% INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR ANY
% PARTICULAR PURPOSE.  IN NO EVENT SHALL THE AUTHOR OR BROWN UNIVERSITY BE LIABLE FOR
% ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
% WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
% ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
% OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.  

sigma_d = this.sigma_d;
sigma_s = this.sigma_s;
lambda  = this.lambda; 

% spatial term
S = {[1 -1], [1; -1], [0; -1; 1]};

E_s = 0;

for i = 1:length(S)

    u_ = conv2(u, S{i}, 'valid');
    v_ = conv2(u, S{i}, 'valid');

    E_s = E_s + sum(log(1+0.5*u_(:).^2/sigma_s^2))...
              + sum(log(1+0.5*v_(:).^2/sigma_s^2));
          
end;

% data term
imw = backward_warp(im2, u, v);
It  = imw - im1;
% Ignore pixels out of the boundary
It(isnan(It)) = 0; 
    
E_d = sum(log(1+0.5*It(:).^2/sigma_d^2));

E = E_d + lambda*E_s;

function im = backward_warp(im2, u, v)
% backward warp the input image im2 using the flow field from im1 to im2
% using the bi-linear interpolation method

H   = size(im2, 1);
W   = size(im2, 2);

[x,y]   = meshgrid(1:W,1:H);
x2      = x + u;        
y2      = y + v;        

im  = interp2(x,y,im2,x2,y2,'linear', NaN);