function [layer_uv z] = fuse_layer_motion(this, z, can_uv, lambdaZ, lambdaL)

% choose between two candidate layers, which a pixel belongs to (z), and
% for every pixel in each layer, choose its motion vector among a candidate
% set of motion vectors (l1/l2)

% layer_uv{1}(i,j, :) = can_uv{l1(i,j)}(i,j, :)

% note that these two layers may not cover the entire image plane


% lambdaZ = 5.5e1;  % works for quarter size birdapple
% lambdaL = 5e0;    % 5
if nargin<3
    lambdaZ = 5.5e1*3.1/1.8*2;  % %  5.5e1*3.1/1.8*2 roughly good for half birdapple
end;
if nargin<4
    lambdaL = 5e0/2*3.1/1.8;    % 5e0/2*3.1/1.8
end;
% lambdaA = 1e8;                % data term, deviation from the affine flow fields

% lambdaL = lambdaL/2

[H W]= size(z);
ind = find(z>0); % initial value, 1: layer1, 2: layer2, 0: other layers
nvalid  = numel(ind);
z0 = z(ind);
l1 = ones(H, W);
l2 = ones(H, W)*3;
%[z0 l1 l2] is the initial value of the solution
  
nZ = 2;              % only two layers
%nC = length(can_uv); % 1: current flow of layer1, 2: affien flow of layer1, 3 current flow of % laery 2, 4 affine flow of layer 2, to add: 5c+nl, sift, block matching...)
nC = size(can_uv, 4); 

% Precompute the data term for each candidate flow field
aIt2 = zeros([H W nC]);
for i = 1:nC   
    It          = partial_deriv(this.images, can_uv(:,:,:,i), this.interpolation_method, this.deriv_filter);               
    aIt2(:,:,i) = reshape(evaluate(this.rho_data, It(:)), size(It));
    % TODO: occlusion handling
end;

% mean(abs(aIt2(:)));


% The flow field are not fixed, 
% Precompute the flow derivative penalty 
% Dh = [-1 1]; Dv = Dh'; 
% pen_h = zeros([H W nC]); pen_v = zeros([H W nC]);
% for i = 1:nC   
%     u_ = conv2(can_uv{i}(:,:,1), Dh, 'valid');
%     v_ = conv2(can_uv{i}(:,:,1), Dh, 'valid');
%     % last column will never be touched, for easy manipulcation
%     pen_h(:,1:W-1,i) = reshape(evaluate(this.rho_spatial_u{1}, u_(:)), size(u_))+...
%                    reshape(evaluate(this.rho_spatial_v{1}, v_(:)), size(v_));
%     u_ = conv2(can_uv{i}(:,:,1), Dv, 'valid');
%     v_ = conv2(can_uv{i}(:,:,1), Dv, 'valid');   
%     % last row will never be touched, for easy manipulcation
%     pen_v(1:H-1,:,i) = reshape(evaluate(this.rho_spatial_u{2}, u_(:)), size(u_))+...
%                    reshape(evaluate(this.rho_spatial_v{2}, v_(:)), size(v_));    
% end;

% cliques
vert = im2col((reshape(1:H*W,H,W)),[2 1])';%vertical
hor  = im2col((reshape(1:H*W,H,W)),[1 2])';%horizontal

% valid cliques for z
tmp = (z(1:end-1,:)>0) & (z(2:end,:)>0);
tind = find(tmp);
valid_vert_z = vert(tind, :);       % TODO: double check

tmp = (z(:,1:end-1)>0) & (z(:,2:end)>0);
tind = find(tmp);
valid_hor_z = hor(tind, :);

%%
% compute weights for Ising model of z
if size(this.labImage, 3) >1
    dif2 = zeros(size(valid_vert_z,1),1);
    for j=1:size(this.labImage, 3)
        tmp = this.labImage(valid_vert_z+(j-1)*H*W);
        dif2 = dif2 + (tmp(:,1)-tmp(:,2)).^2;
    end;
    dif2 = dif2*size(this.labImage, 3)/3;    
    dif2(dif2>this.Ti2) = this.Ti2;
    weightZv = exp(-dif2/2/this.sigma_i^2);

    dif2 = zeros(size(valid_hor_z,1),1);
    for j=1:size(this.labImage, 3)
        tmp = this.labImage(valid_hor_z+(j-1)*H*W);
        dif2 = dif2 + (tmp(:,1)-tmp(:,2)).^2;
    end;
    dif2 = dif2*size(this.labImage, 3)/3;
    dif2(dif2>this.Ti2) = this.Ti2;
    weightZh = exp(-dif2/2/this.sigma_i^2);

else
    tmp = this.labImage(valid_vert_z);
    dif2 = (tmp(:,1)-tmp(:,2)).^2 *3;
    dif2(dif2>this.Ti2) = this.Ti2;
    weightZv = exp(-dif2/2/this.sigma_i^2);

    tmp = this.labImage(valid_hor_z);
    dif2 = (tmp(:,1)-tmp(:,2)).^2 *3;
    dif2(dif2>this.Ti2) = this.Ti2;
    weightZh = exp(-dif2/2/this.sigma_i^2);        
end;

% [mean(weightZv(:)) mean(weightZh(:)) ]

weightZv = repmat(weightZv', [4 1]);
weightZh = repmat(weightZh', [4 1]);

tmpz = z;
tmpz(ind) = 1:nvalid;
valid_vert_z = tmpz(valid_vert_z);
valid_hor_z  = tmpz(valid_hor_z);
% collect valid pairwise term in z

vertl1 = vert+nvalid;
vertl2 = vert+nvalid+H*W;
horl1  = hor + nvalid;
horl2  = hor +nvalid+H*W;

pwzl1 = [(1:nvalid)' ind+nvalid];
pwzl2 = [(1:nvalid)' ind+nvalid+H*W];


prior_funcZ = @(u) pairwise_potential_tableZ(u);
prior_funcZL1 = @(u) pairwise_potential_tableZL(u, aIt2, nvalid, 1);
prior_funcZL2 = @(u) pairwise_potential_tableZL(u, aIt2, nvalid+H*W, 2);
prior_funcL1V = @(u) pairwise_potential_tableL(u, can_uv, nvalid, this.rho_spatial_u{1});
prior_funcL1H = @(u) pairwise_potential_tableL(u, can_uv, nvalid, this.rho_spatial_u{1});
prior_funcL2V = @(u) pairwise_potential_tableL(u, can_uv, nvalid+H*W, this.rho_spatial_u{1});
prior_funcL2H = @(u) pairwise_potential_tableL(u, can_uv, nvalid+H*W, this.rho_spatial_u{1});

% Compute deviations from the affine motion
% dataD = zeros(4, nvalid);
% 
% tmp = zeros(4, H*W);
% tmp1 = sum(abs(can_uv(:,:,1:2)-can_uv(:,:,3:4)), 3);
% tmp(1,:) = tmp1(:)';
% tmp1 = sum(abs(can_uv(:,:,5:6)-can_uv(:,:,7:8)), 3);
% tmp(3,:) = tmp1(:)';
% dataD = [dataD tmp tmp]*lambdaA;
% 
data_unary = [1:nvalid+2*H*W; zeros(2, H*W*2+nvalid)]; % no data unary term

% %%%%%%%%%%%%%%%%%%%%%%l1->l2->z
% 
% fp = [z0(:); l1(:); l2(:)];
% current_energy = 1e32;
% success = 0;
% % start alpha expansion
% % while true
%     current_proposal = ones(size(fp));
%         
%     for il1 = 1:nC
%         current_proposal(1+nvalid:nvalid+H*W) = il1;
%        
%         % add proper shift to the second row (index to l1)
%         
%         %pairwise terms of l1
%         pl1v = layer_generateEnergyTable(vertl1, prior_funcL1V, current_proposal, fp);
%         pl1h = layer_generateEnergyTable(horl1,  prior_funcL1H, current_proposal, fp);
%         pl1v(3:end, :) = lambdaL*pl1v(3:end, :);
%         pl1h(3:end, :) = lambdaL*pl1h(3:end, :);
%         
%         % add proper shift to the first and second row (index to l1)
%         
%         for il2 = 1:nC
%             current_proposal(1+nvalid+H*W:nvalid+H*W*2) = il2;
%             
%             %pairwise terms of l2
%             pl2v = layer_generateEnergyTable(vertl2, prior_funcL2V, current_proposal, fp);
%             pl2h = layer_generateEnergyTable(horl2,  prior_funcL2H, current_proposal, fp);
%             pl2v(3:end, :) = lambdaL*pl2v(3:end, :);
%             pl2h(3:end, :) = lambdaL*pl2h(3:end, :);
%             
%             for iz=1:nZ
%                 current_proposal(1:nvalid) = iz;
%                 
%                 %pairwise terms between z and l1
%                 pzl1 = layer_generateEnergyTable(pwzl1, prior_funcZL1, current_proposal, fp);
% 
%                 %pairwise terms between z and l2
%                 pzl2 = layer_generateEnergyTable(pwzl2, prior_funcZL2, current_proposal, fp);
%                 
%                 % current_proposal_z = ones(1,nvalid)*iz;
%                 % pairwise term of z in valid regions
%                 pzv = layer_generateEnergyTable(valid_vert_z, prior_funcZ, current_proposal, fp);
%                 pzh = layer_generateEnergyTable(valid_hor_z,  prior_funcZ, current_proposal, fp);
%                 pzv(3:end, :) = lambdaZ*pzv(3:end, :).*weightZv;
%                 pzh(3:end, :) = lambdaZ*pzh(3:end, :).*weightZh;
%                 
%                 pairwise_terms  = [pl1v pl1h pl2v pl2h pzl1 pzl2 pzv pzh];
%                 
%                 [new_fp,new_energy] = QPBO_all_in_one(H*W*2+nvalid, data_unary, pairwise_terms);
%                 
%                 if (new_energy - current_energy) > .5
%                     %if we use potentials of type double, it is
%                     %possible that the new energy grows, because of numerical
%                     %issues, see QPBO documentation
%                     
%                     %             disp(strcat('WARNING: energy grows, (new_energy - current_energy) >'...
%                     %                 ,num2str(new_energy - current_energy)));
%                 elseif (new_energy - current_energy) < -1
%                     success = 1;
%                     current_energy=new_energy;
%                 end
%                 
%                 fprintf('%3.3f ', current_energy);
%                 %disp(strcat('energy=',num2str(current_energy)));
%                 
%                 % To change
%                 % new_fp = reshape(new_fp(1:m*n),m,n);%reshape new_fp to given image size
%                 fp = (1-new_fp).* current_proposal+ new_fp .* fp;%generate new labeling
%                 
%             end            
%         end;
%         fprintf('\n');
%     end;
%     
% %     if success == 0
% %         disp('finished, energy did not shrink');
% %         break;
% %     end
% %     success = 0;
% % end;

%%%%%%%%%%%%%%%%% z->l1->l2
fp = [z0(:); l1(:); l2(:)];
current_energy = 1e32;
success = 0;
% start alpha expansion
% while true
    current_proposal = ones(size(fp));
    for iz=1:nZ
        current_proposal(1:nvalid) = iz;
        
        % current_proposal_z = ones(1,nvalid)*iz;        
        % pairwise term of z in valid regions
        pzv = layer_generateEnergyTable(valid_vert_z, prior_funcZ, current_proposal, fp);
        pzh = layer_generateEnergyTable(valid_hor_z,  prior_funcZ, current_proposal, fp);
        pzv(3:end, :) = lambdaZ*pzv(3:end, :).*weightZv;
        pzh(3:end, :) = lambdaZ*pzh(3:end, :).*weightZh;
        
        for il1 = 1:nC
            current_proposal(1+nvalid:nvalid+H*W) = il1;
            %pairwise terms between z and l1
            pzl1 = layer_generateEnergyTable(pwzl1, prior_funcZL1, current_proposal, fp);
            
            % add proper shift to the second row (index to l1)
            
            %pairwise terms of l1
            pl1v = layer_generateEnergyTable(vertl1, prior_funcL1V, current_proposal, fp);
            pl1h = layer_generateEnergyTable(horl1,  prior_funcL1H, current_proposal, fp);
            pl1v(3:end, :) = lambdaL*pl1v(3:end, :);
            pl1h(3:end, :) = lambdaL*pl1h(3:end, :);
            
            % add proper shift to the first and second row (index to l1)
            
            for il2 = 1:nC
                current_proposal(1+nvalid+H*W:nvalid+H*W*2) = il2;
                %pairwise terms between z and l2
                pzl2 = layer_generateEnergyTable(pwzl2, prior_funcZL2, current_proposal, fp);
                
                %pairwise terms of l2
                pl2v = layer_generateEnergyTable(vertl2, prior_funcL2V, current_proposal, fp);
                pl2h = layer_generateEnergyTable(horl2,  prior_funcL2H, current_proposal, fp);
                pl2v(3:end, :) = lambdaL*pl2v(3:end, :);
                pl2h(3:end, :) = lambdaL*pl2h(3:end, :);
               
                pairwise_terms  = [pl1v pl1h pl2v pl2h pzl1 pzl2 pzv pzh];

                % Data term penalize deviation from affine flow field
%                 ind_curr = sub2ind([4 nvalid+2*H*W], current_proposal', 1:nvalid+2*H*W);
%                 ind_fp   = sub2ind([4 nvalid+2*H*W], fp', 1:nvalid+2*H*W);                
%                 data_unary = [1:nvalid+2*H*W; dataD(ind_curr); dataD(ind_fp)]; 
%                 
%     fprintf('%3.3e\t%3.3e\n', max(dataD(ind_curr)), max(dataD(ind_fp))); % for debug
    
                
                [new_fp,new_energy] = QPBO_all_in_one(H*W*2+nvalid, data_unary, pairwise_terms);
                
                if (new_energy - current_energy) > .5
                    %if we use potentials of type double, it is
                    %possible that the new energy grows, because of numerical
                    %issues, see QPBO documentation
                    
                    %             disp(strcat('WARNING: energy grows, (new_energy - current_energy) >'...
                    %                 ,num2str(new_energy - current_energy)));
                elseif (new_energy - current_energy) < -1
                    success = 1;
                    current_energy=new_energy;
                end
                
%                 fprintf('%3.3f ', current_energy);
                %disp(strcat('energy=',num2str(current_energy)));
                
                % To change
                % new_fp = reshape(new_fp(1:m*n),m,n);%reshape new_fp to given image size
                fp = (1-new_fp).* current_proposal+ new_fp .* fp;%generate new labeling

            end            
%             fprintf('\n');
        end;               
    end;
    
%     if success == 0
%         disp('finished, energy did not shrink');
%         break;
%     end
%     success = 0;
% end;
z(ind) = fp(1:nvalid);
l1     = fp(nvalid+1:nvalid+H*W);
tind   = [1:H*W]' + (l1-1)*H*W*2;
layer_uv{1}(:,:,1) = reshape(can_uv(tind), [H W]);
layer_uv{1}(:,:,2) = reshape(can_uv(tind+H*W), [H W]);
l2     = fp(nvalid+1+H*W:nvalid+H*W*2);
tind   = [1:H*W]' + (l2-1)*H*W*2;
layer_uv{2}(:,:,1) = reshape(can_uv(tind), [H W]);
layer_uv{2}(:,:,2) = reshape(can_uv(tind+H*W), [H W]);

% figure; imagesc(reshape(l1, [H W]));
% figure; imagesc(reshape(l2, [H W]));
% Arrange solution into normal format