function [ouv oz] = fuse_seg_motion(this, zs, can_uvs, lambdaZ)

% choose from several flow candidates, which the pixel should take at time
% t and t+1

% The data term will consider the occlusion relationship by checking the
% segementation consistency over time

if nargin<4
    lambdaZ = 1;    %
end;

lambdaD = this.lambda5; 
lambdaFB = lambdaD+2; % penalty for chaning from the front layers to the back layers

z  = zs(:,:,1);

can_uv  = can_uvs{1};
can_uv2 = can_uvs{2};

[H W]= size(z);
  
nC = size(can_uv, 4); 
nZ = nC;

% Precompute the data term for each candidate flow field
aIt2 = zeros([H W nC]); aIt22 = aIt2;
for i = 1:nC  
    % Data term at time 1
    It          = partial_deriv(this.images, can_uv(:,:,:,i), this.interpolation_method, this.deriv_filter);               
    aIt2(:,:,i) = reshape(evaluate(this.rho_data, It(:)), size(It));    
    % Data term at time 2
    It          = partial_deriv(this.images(:,:,2:-1:1), can_uv2(:,:,:,i), this.interpolation_method, this.deriv_filter);
    aIt22(:,:,i) = reshape(evaluate(this.rho_data, It(:)), size(It));
end;
aIt2 = aIt2-lambdaD; aIt22 = aIt22-lambdaD;

% Record and set out-of-boundary penalty to be 0
for i=1:nC
    OOB1(:,:,i) = compute_out_of_boundary_pixels(can_uv(:,:,:,i));
    OOB2(:,:,i) = compute_out_of_boundary_pixels(can_uv2(:,:,:,i));
end;
aIt2(OOB1) =0; aIt22(OOB2) =0; 

% cliques
vert = im2col((reshape(1:H*W,H,W)),[2 1])';%vertical
hor  = im2col((reshape(1:H*W,H,W)),[1 2])';%horizontal


% Temporal cliques for time t
[cliqueT1 aIt2]= construct_temporal_neighbor(can_uv, H, W, aIt2);
[cliqueT2 aIt22]= construct_temporal_neighbor(can_uv2, H, W, aIt22);
% index of cliqueT2 should be flipped
cliqueT2(:,1) = cliqueT2(:,1)+H*W;
cliqueT2(:,2) = cliqueT2(:,2)-H*W;

%%
% compute weights for Ising model of z
wv1 = compute_color_weight(this.labImage{1}, vert, this.Ti2, this.sigma_i);
wh1 = compute_color_weight(this.labImage{1}, hor, this.Ti2, this.sigma_i);
wv2 = compute_color_weight(this.labImage{2}, vert, this.Ti2, this.sigma_i);
wh2 = compute_color_weight(this.labImage{2}, hor, this.Ti2, this.sigma_i);
weightZv = repmat([wv1' wv2'], [4 1]);
weightZh = repmat([wh1' wh2'], [4 1]);

prior_funcZ   = @(u) pairwise_potential_tableZ(u);
% prior_funcZ1  = @(u) pairwise_potential_tableZUV(u, can_uv, lambdaB, H, W, this.rho_spatial_u{1});
% prior_funcZ2  = @(u) pairwise_potential_tableZUV(u, can_uv2, lambdaB, H, W, this.rho_spatial_u{1});

% prior_funcZT1 = @(u) pairwise_potential_tableZT(u, aIt2,  H, W, nC, lambdaD);
% prior_funcZT2 = @(u) pairwise_potential_tableZT(u, aIt22, H, W, nC, lambdaD);
prior_funcZT1 = @(u) pairwise_potential_tableZT_order(u, aIt2,  H, W, OOB1, nC, lambdaD);
prior_funcZT2 = @(u) pairwise_potential_tableZT_order(u, aIt22, H, W, OOB2, nC, lambdaD);



data_unary = [1:2*H*W; zeros(2, 2*H*W)]; % no data unary term

%%%%%%%%%%%%%%%%% 
fp = zs(:);
current_energy = 1e32;
success = 0;
% start alpha expansion
while true    
    for iz=1:nZ
        current_proposal = iz *ones(size(fp));
                
%         % spatial pairwise term         
%         pzv1 = layer_generateEnergyTable(vert, prior_funcZ1, current_proposal(1:H*W), fp(1:H*W));
%         pzh1 = layer_generateEnergyTable(hor,  prior_funcZ1, current_proposal(1:H*W), fp(1:H*W));
% 
%         pzv2 = layer_generateEnergyTable(vert+H*W, prior_funcZ2, current_proposal(1+H*W:2*H*W), fp(1+H*W:2*H*W));
%         pzh2 = layer_generateEnergyTable(hor+H*W,  prior_funcZ2, current_proposal(1+H*W:2*H*W), fp(1+H*W:2*H*W));        

        % Potts model for z smoothness
        pzv1 = layer_generateEnergyTable(vert, prior_funcZ, current_proposal(1:H*W), fp(1:H*W));
        pzh1 = layer_generateEnergyTable(hor,  prior_funcZ, current_proposal(1:H*W), fp(1:H*W));

        pzv2 = layer_generateEnergyTable(vert, prior_funcZ, current_proposal(1+H*W:2*H*W), fp(1+H*W:2*H*W));
        pzh2 = layer_generateEnergyTable(hor,  prior_funcZ, current_proposal(1+H*W:2*H*W), fp(1+H*W:2*H*W));        
        pzv2(1:2,:) = pzv2(1:2,:)+H*W;
        pzh2(1:2,:) = pzh2(1:2,:)+H*W;        
        
        pzv  = [pzv1 pzv2];
        pzh  = [pzh1 pzh2];        
        pzv(3:end, :) = pzv(3:end, :).*weightZv;
        pzh(3:end, :) = pzh(3:end, :).*weightZh;        
        
%         % flow smoothness
%         pzv1 = layer_generateEnergyTable(vert, prior_funcZ1, current_proposal(1:H*W), fp(1:H*W));
%         pzh1 = layer_generateEnergyTable(hor,  prior_funcZ1, current_proposal(1:H*W), fp(1:H*W));
%         
%         pzv2 = layer_generateEnergyTable(vert, prior_funcZ2, current_proposal(1+H*W:2*H*W), fp(1+H*W:2*H*W));
%         pzh2 = layer_generateEnergyTable(hor,  prior_funcZ2, current_proposal(1+H*W:2*H*W), fp(1+H*W:2*H*W));
%         pzv2(1:2,:) = pzv2(1:2,:)+H*W;
%         pzh2(1:2,:) = pzh2(1:2,:)+H*W;       
%         
%         tmp_pzv  = [pzv1 pzv2];
%         tmp_pzh  = [pzh1 pzh2];    
%         pzv(3:end, :) = pzv(3:end, :) + tmp_pzv(3:end, :);
%         pzh(3:end, :) = pzh(3:end, :) + tmp_pzh(3:end, :);

        % Multiply the weight
        pzv(3:end, :) = lambdaZ*pzv(3:end, :);
        pzh(3:end, :) = lambdaZ*pzh(3:end, :);

        
        
        % data pariwise term (temporal)
        pzt1 = layer_generateEnergyTable(cliqueT1,  prior_funcZT1, current_proposal, fp);
        pzt2 = layer_generateEnergyTable(cliqueT2,  prior_funcZT2, current_proposal, fp);
                      
        pairwise_terms  = [pzv pzh pzt1 pzt2];
        
        [new_fp,new_energy] = QPBO_all_in_one(H*W*2, data_unary, pairwise_terms);
        
        if (new_energy - current_energy) > .5
            %if we use potentials of type double, it is
            %possible that the new energy grows, because of numerical
            %issues, see QPBO documentation
            
            %             disp(strcat('WARNING: energy grows, (new_energy - current_energy) >'...
            %                 ,num2str(new_energy - current_energy)));
        elseif (new_energy - current_energy) < -1
            success = 1;
            current_energy=new_energy;
        end
        
%         fprintf('%3.3f ', current_energy);
        
        fp = (1-new_fp).* current_proposal+ new_fp .* fp;%generate new labeling
        
    end
%     fprintf('\n');
    
    if success == 0
%         disp('finished, energy did not shrink');
        break;
    end
    success = 0;
end;
fprintf('%3.3f \n', current_energy);
oz = reshape(fp, [H W 2]);
ouv{1} = composite_final_flow_seg(can_uv, oz(:,:,1));
ouv{2} = composite_final_flow_seg(can_uv2,oz(:,:,2));

% figure; imagesc([flowToColor(ouv{1}) flowToColor(ouv{2})]);
% figure; imshow(flowToColor(ouv{1}));
% figure; imshow(flowToColor(ouv{2}));
% figure; imagesc([oz(:,:,1) oz(:,:,2)]);
% figure; imagesc();
%%
% z2 = oz(:,:,2);
% seg2 = oz(cliqueT1);

% plot(seg2(:,1)); hold on; plot(seg2(:,2), 'r');

% seg2 = warp_integer_seg(oz(:,:,2), ouv{1}, H, W);
% figure; imagesc(oz(:,:,1)~=seg2);
% seg1 = warp_integer_seg(oz(:,:,1), ouv{2}, H, W);
% figure; imagesc(oz(:,:,2)~=seg1);

function seg = warp_integer_seg(z, uv, H, W)
[X Y] = meshgrid(1:W, 1:H);
x = round(X+uv(:,:,1));
x(x>W) = W; x(x<1) = 1;
y = round(Y+uv(:,:,2));
y(y>H) = H; y(y<1) = 1;
ind = sub2ind([H W], y, x);

seg = reshape(z(ind), [H W]);

% ind = sub2ind([H W], y, x);
% seg2 = reshape(oz(ind+H*W), [H W]);
% % figure; imagesc(seg2);
% % figure; imagesc(oz(:,:,1));
% 
% figure; imagesc(oz(:,:,1)~=seg2);
% % Observe segmentation consistency
% 
% It = partial_deriv(this.images, ouv{1}, this.interpolation_method, this.deriv_filter);               
% figure; imagesc(It < lambdaD);

%%
function weight = compute_color_weight(labImage, cliques, Ti2, sigma_i)

H = size(labImage,1);
W = size(labImage,2);

if size(labImage, 3) >1
    dif2 = zeros(size(cliques,1),1);
    for j=1:size(labImage, 3)
        tmp = labImage(cliques+(j-1)*H*W);
        dif2 = dif2 + (tmp(:,1)-tmp(:,2)).^2;
    end;
    dif2 = dif2*size(labImage, 3)/3;    
    dif2(dif2>Ti2) = Ti2;
    weight = exp(-dif2/2/sigma_i^2);
else
    tmp = labImage(cliques);
    dif2 = (tmp(:,1)-tmp(:,2)).^2 *3;
    dif2(dif2>Ti2) = Ti2;
    weight = exp(-dif2/2/sigma_i^2);
end;

function [cliqueT aIt2]= construct_temporal_neighbor(can_uv, H, W, aIt2)

[X Y] = meshgrid(1:W, 1:H);
cliqueT = [];
ind1  = reshape(1:W*H, [H W]);
for i=1:size(can_uv,4);
    x = round(X + can_uv(:,:,1,i));
    y = round(Y + can_uv(:,:,2,i));
    valid = (x>0) & (x<=W) & (y>0) & (y<=H);
    
    x(~valid) = X(~valid);
    y(~valid) = Y(~valid);
    
    ind2 = sub2ind([H W], y, x);
    % Set data term of invalid flow candidate to be zero
    tmp = aIt2(:,:,i);
    tmp(~valid) = 0;
    aIt2(:,:,i) = tmp;
    %cliqueT = [cliqueT; [ind1(valid) ind2(valid)+H*W]];    
    % include out-of-boundary mapping
    cliqueT = [cliqueT; [ind1(:) ind2(:)+H*W]];    
end;

