function [oluvs oz] = fuse_seg_motion_libDAI_sep_candidate_set_complete_clique_mex(this, zs, can_uvs, layer_id, luvs, lambdaZ, lambdaT)

% choose from several flow candidates, which the pixel should take at time
% t and t+1

% The data term will consider the occlusion relationship by checking the
% segementation consistency over time

% layer_id specifis the two layers to switch

if nargin<6
    lambdaZ = 1;    %
end;

if nargin<7
    lambdaT = 1;        % weight on the temporal term
end;

lambdaL     = this.lambda;
lambdaD     = this.lambda5;    %
lambdaD     = lambdaD*lambdaT;
lambdaFB    = lambdaD*2;       % penalty for front layer to change to back layers at next frame 
lambdaFB    = lambdaD;       % penalty for front layer to change to back layers at next frame 

validZ = (zs==layer_id(1)) | (zs==layer_id(2));
%%
[H W]   = size(zs(:,:,1));
can_uv  = can_uvs{1};
can_uv2 = can_uvs{2};
nC      = size(can_uv, 4); 

% Precompute the data term for each candidate flow field
aIt2 = zeros([H W nC]); aIt22 = aIt2;
for i = 1:nC  
    % Data term at time 1
    It          = partial_deriv(this.images, can_uv(:,:,:,i), this.interpolation_method, this.deriv_filter);               
    aIt2(:,:,i) = reshape(evaluate(this.rho_data, It(:)), size(It));    
    % Data term at time 2
    It          = partial_deriv(this.images(:,:,2:-1:1), can_uv2(:,:,:,i), this.interpolation_method, this.deriv_filter);
    aIt22(:,:,i) = reshape(evaluate(this.rho_data, It(:)), size(It));
end;

aIt2 = aIt2*lambdaT;
aIt22 = aIt22*lambdaT;

% Record and set out-of-boundary penalty to be 0
for i=1:nC
    OOB1(:,:,i) = compute_out_of_boundary_pixels(can_uv(:,:,:,i));
    OOB2(:,:,i) = compute_out_of_boundary_pixels(can_uv2(:,:,:,i));
end;
aIt2(OOB1) =lambdaD; aIt22(OOB2) =lambdaD; 
OOB1 = double(OOB1);OOB2 = double(OOB2);

% spatial cliques
vert = im2col((reshape(1:H*W,H,W)),[2 1])';%vertical
hor  = im2col((reshape(1:H*W,H,W)),[1 2])';%horizontal

% compute weights for Ising model of z
wv1 = compute_color_weight(this.labImage{1}, vert, this.Ti2, this.sigma_i); wv1 = [reshape(wv1, [H-1 W]); ones(1, W)];
wh1 = compute_color_weight(this.labImage{1}, hor, this.Ti2, this.sigma_i);  wh1 = [reshape(wh1, [H W-1]) ones(H, 1)];
wv2 = compute_color_weight(this.labImage{2}, vert, this.Ti2, this.sigma_i); wv2 = [reshape(wv2, [H-1 W]); ones(1, W)];
wh2 = compute_color_weight(this.labImage{2}, hor, this.Ti2, this.sigma_i);  wh2 = [reshape(wh2, [H W-1]) ones(H, 1)];

wv1 = wv1*lambdaZ;
wh1 = wh1*lambdaZ;
wv2 = wv2*lambdaZ;
wh2 = wh2*lambdaZ;


flowPTables1V = compute_can_flow_spatial_potential_min_sum_bp(can_uv, this.rho_spatial_u{1}, vert,lambdaL, 'V');
flowPTables1H = compute_can_flow_spatial_potential_min_sum_bp(can_uv, this.rho_spatial_u{1}, hor, lambdaL, 'H');

flowPTables2V = compute_can_flow_spatial_potential_min_sum_bp(can_uv2, this.rho_spatial_u{1}, vert, lambdaL, 'V');
flowPTables2H = compute_can_flow_spatial_potential_min_sum_bp(can_uv2, this.rho_spatial_u{1}, hor, lambdaL, 'H');
%%
% Temporal cliques for time t

z1neighbors = compute_temporal_neighbors(can_uv, H, W);
z2neighbors = compute_temporal_neighbors(can_uv2, H, W);
%%
% tic
% layer_bp_inference(aIt2, aIt22, flowPTables1V, flowPTables1H, flowPTables2V, flowPTables2H, wv1, wh1, wv2, wh2, zs-1, OOB1, OOB2, lambdaD, lambdaFB, layer_id-1, z1neighbors-1, z2neighbors-1);
            
% #define AIT2_IN    prhs[0]
% #define AIT22_IN   prhs[1] 
% #define PTABLEV1_IN prhs[2]
% #define PTABLEH1_IN prhs[3]
% #define PTABLEV2_IN prhs[4]
% #define PTABLEH2_IN prhs[5]

% #define WEIGHTVZ1_IN prhs[6]
% #define WEIGHTHZ1_IN prhs[7]
% #define WEIGHTVZ2_IN prhs[8]
% #define WEIGHTHZ2_IN prhs[9]

% #define Z_IN         prhs[10]
% #define OOB_IN       prhs[11]
% #define OOB2_IN      prhs[12]
% #define LAMBDAD_IN   prhs[13]
% #define LAMBDAFB_IN  prhs[14]
% #define ZID_IN       prhs[15]

% #define Z1NEIGHBORS_IN prhs[16]
% #define Z2NEIGHBORS_IN prhs[17]

[qmapZ qmapL]   = layer_bp_inference(aIt2, aIt22, flowPTables1V, flowPTables1H, flowPTables2V, flowPTables2H, ...
                wv1, wh1, wv2, wh2, zs-1, OOB1, OOB2, lambdaD, lambdaFB, layer_id-1, z1neighbors-1, z2neighbors-1);

qmap = cat(3, qmapZ, qmapL);
%%
tmp = reshape(qmap+1, [H W 6]);

oz = tmp(:,:,1:2);
tmp_oz = oz;
oz(tmp_oz==1) = layer_id(1);
oz(tmp_oz==2) = layer_id(2);
oz(~validZ) = zs(~validZ);
% figure; imagesc(oz(:,:,1));
%
% Update flow of each layer
l11 = tmp(:,:,3);
l12 = tmp(:,:,4);
uv1 = composite_final_flow_seg(can_uv, l11);
uv2 = composite_final_flow_seg(can_uv, l12);
l21 = tmp(:,:,5);
l22 = tmp(:,:,6);
uvb1 = composite_final_flow_seg(can_uv2, l21);
uvb2 = composite_final_flow_seg(can_uv2, l22);

oluvs = luvs;
oluvs{1}(:,:,:,layer_id(1)) = uv1;
oluvs{1}(:,:,:,layer_id(2)) = uv2;
oluvs{2}(:,:,:,layer_id(1)) = uvb1;
oluvs{2}(:,:,:,layer_id(2)) = uvb2;

%%
figure; imagesc([l11 l12]); % for debug
% %%
% figure; imagesc(flowToColor(uv1));
% %%
% figure; imagesc(oz(:,:,1));
% %%
% figure; imagesc(tmp(:,:,1));
% %%
% figure; imagesc(validZ(:,:,1));
%%
function seg = warp_integer_seg(z, uv, H, W)
[X Y] = meshgrid(1:W, 1:H);
x = round(X+uv(:,:,1));
x(x>W) = W; x(x<1) = 1;
y = round(Y+uv(:,:,2));
y(y>H) = H; y(y<1) = 1;
ind = sub2ind([H W], y, x);

seg = reshape(z(ind), [H W]);


function [neighbors]= compute_temporal_neighbors(can_uv, H, W)
% two corresponding neighbors are valid if they are both in the valid
% regions, as specified by validZ
% onInd(i) records the flow candidate that connect clique(i,1) and clique(i,2)
% oob records the selected pixels in image 1 that are projected out of boundary

nL = size(can_uv, 4);
[X Y] = meshgrid(1:W, 1:H);

neighbors = ones(H, W, nL*2);

for i=1:size(can_uv,4);
    x = round(X + can_uv(:,:,1,i));
    y = round(Y + can_uv(:,:,2,i));
    valid = (x>0) & (x<=W) & (y>0) & (y<=H);
    
    x(~valid) = X(~valid);
    y(~valid) = Y(~valid);    

    neighbors(:,:,2*i-1) = x;
    neighbors(:,:,2*i)   = y;
end;



