function potential_tables = compute_can_flow_spatial_potential(can_uv, rho, vert, hor, lambdaL, isLogDomain)

% Compute the potential table for flow candidate spatial terms

% tic;
% H = 400;W = 400;
% % H = 100;W = 100;
% can_uv = rand([H W 2 6]);
% rho = robust_function('generalized_charbonnier', 1e-3, 0.45);

% cliques
% vert = im2col((reshape(1:H*W,H,W)),[2 1])';%vertical
% hor  = im2col((reshape(1:H*W,H,W)),[1 2])';%horizontal

H  = size(can_uv,1);
W  = size(can_uv,2);

nV = size(vert,1);
nH = size(hor,1);

nC = size(can_uv, 4); 
potential_tables = zeros(nC, nC, nV+nH);


% By default, there are only two flow candidates for each layer and the
% first is the mean affine flow fields
% This is the affine+deviation model 
% meanuv = can_uv(:,:,:,1);
% for i=1:nC
%     can_uv(:,:,:,i)=can_uv(:,:,:,i)-meanuv;
% end;

for c1=1:nC;
    for c2 = 1:nC;
                
        % horitonzal flow field                
        ind1 = [vert(:,1); hor(:,1)] + (c1-1)*2*H*W;
        ind2 = [vert(:,2); hor(:,2)] + (c2-1)*2*H*W;
        dif = can_uv(ind1)-can_uv(ind2);        
        potential_tables(c1, c2, :) = evaluate(rho, dif);
        
        % vertical flow field       
        ind1 = ind1+H*W;
        ind2 = ind2+H*W;
        dif = can_uv(ind1)-can_uv(ind2);               
        potential_tables(c1, c2, :) = squeeze(potential_tables(c1, c2, :))+evaluate(rho, dif);
    end;
end;

% convert energy function to log comatability function 
if isLogDomain
    potential_tables = -lambdaL*potential_tables;
else
    potential_tables = exp(-lambdaL*potential_tables);
end;

% toc