function [cliqueT aIt2 oob onInd flip flipZ]= construct_temporal_neighbor_forward(can_uv, aIt2, zs, layer_id, lambdaD)
% two corresponding neighbors are valid if they are both in the valid
% regions, as specified by validZ
% onInd(i) records the flow candidate that connect clique(i,1) and clique(i,2)
% oob records the selected pixels in image 1 that are projected out of boundary

% flip records pixels that moves to other layers
% flipZ records the corresponding layer # for pixels in flip

H = size(can_uv,1);
W = size(can_uv,2);
validZ = zs==layer_id(1) | zs==layer_id(2);

[X Y]   = meshgrid(1:W, 1:H);

oob     = [];   % record pixels that are projected out of boundary
flip    = [];   % record pixels that change to other layers
flipZ   = [];
ind1    = reshape(1:W*H, [H W]);

sel     = validZ(:,:,1);
nSel    = sum(sel(:));
% record pixels that are in layer "1" or "2"
onInd   = repmat(1:size(can_uv,4), [nSel 1]);   
onInd   = onInd(:);

cliqueT= ind1(sel);

for i=1:size(can_uv,4);
    x = round(X + can_uv(:,:,1,i));
    y = round(Y + can_uv(:,:,2,i));
    valid = (x>0) & (x<=W) & (y>0) & (y<=H);
    
    % record the selected pixels that are projected out of image boundary
    OB = (~valid)&validZ(:,:,1);
    oob = [oob; ind1(OB)];   

    % Set data term of out-of-boundary piexls be lambdaD
    tmp = aIt2(:,:,i);
    tmp(~valid) = lambdaD;
    aIt2(:,:,i) = tmp;    
    
    x(~valid) = X(~valid);
    y(~valid) = Y(~valid);    
       
    ind2 = sub2ind([H W], y, x);
           
    % these pixels are selected in frame 1 but their corresponding pixels
    % become other layers in frame2
    wz2     = validZ(ind2+H*W);    
    % oob take precedence over changing to other layers
    changed = validZ(:,:,1)&(~wz2) & (~OB);
    
    
    flip    = [flip; ind1(changed)];
    flipZ   = [flipZ; zs(ind2(changed)+H*W)];    
    
    
    
    % clique only contains the selected pixels in the first image    
    cliqueT = [cliqueT ind2(sel)+H*W];        
end;

