function layer_uv = fit_affine_layer_motion(layer_uv, layerLabels)

% fit affine motion to the visible pixels of each layer, interpolate the
% motion of the invisible using affine motion model; motion of visible
% pixels remain unchanged

nL = length(layer_uv);
sz = size(layer_uv{1}); sz = sz(1:2);

nData = prod(sz);
[X Y] = meshgrid(1:sz(2), 1:sz(1));
x = X(:); y = Y(:);
xy1 = [x y ones(nData, 1)];

[tmp ind ] = max(layerLabels, [], 3);


for i = 1:nL
    
    uv = layer_uv{i};
    u = uv(:,:,1); u = u(:);    
    v = uv(:,:,2); v = v(:);
    
    % fit affine motion to visible pixels
    tmp_u = u(ind==i);
    tmp_v = v(ind==i);
    tmp_xy1 = xy1(ind==i, :);
    A(:,1) = L2_fit(tmp_u, tmp_xy1);
    A(:,2) = L2_fit(tmp_v, tmp_xy1);
    
    % interpolate invisble pixels by affine motion model
    u = X*A(1,1)+Y*A(2,1)+A(3,1);
    v = X*A(1,2)+Y*A(2,2)+A(3,2);
    
    u = reshape(u, sz);
    v = reshape(v, sz);
    
    layer_uv{i} = cat(3, u, v);

end;

function a = L2_fit(u, xy1)    
% least square fit u = a1x + a2y + a3;
a = (xy1'*xy1)\(xy1'*u);

% function [layer_uv, layer_uv2, layer_g_layer_g2]= determin_depth_order(small, small2, layer_uv, layer_uv2, layer_g_layer_g2)
% % swap order and check which gives the lower energy
% nL = length(layer_uv);
% cur_order = 1:nL;
% cor_order = cur_order;
% 
% % en = evaluate_energy(small, layer_uv, layer_g, layer_g2) + evaluate_energy(small2, layer_uv2, layer_g2, layer_g);
% % fprintf('energy \t%3.3e\n', en);