function layer_uv = fit_projective_layer_motion_g(layer_uv, layer_g, rho)
% Fit projective (8 parameters) motion to the visible pixels of each layer,
% interpolate the motion of the invisible using the motion model; 

if nargin < 2
    rho = robust_function('quadratic', 1);
end;

nL = length(layer_uv);
sz = size(layer_uv{1}); sz = sz(1:2);

nData = prod(sz);
[X Y] = meshgrid(1:sz(2), 1:sz(1));
x   = X(:); 
y   = Y(:);
ze  = zeros(nData, 1);
on  = ones(nData, 1);

xy1 = [x y on ze ze ze x.^2 x.*y;...
       ze ze ze x y on x.*y y.^2 ];

% Obtain hard segmentation by thresholding the hidden fields
ind = threshold_g(layer_g);
ind = repmat(ind, [1 1 2]);


for i = 1:nL
    
    uv = layer_uv{i}(:);
    
    % fit projective motion to visible pixels
    tmp_uv = uv(ind==i);    
    tmp_xy1 = xy1(ind==i, :);
    %A = L2_fit(tmp_uv, tmp_xy1);     
    A  = RLS_fit(tmp_uv, tmp_xy1, rho);
    
    % compute projective motion field
    u = X*A(1)+Y*A(2)+A(3)+ X.^2*A(7)+X.*Y*A(8);
    v = X*A(4)+Y*A(5)+A(6)+ X.*Y*A(7) +Y.^2*A(8);
    
    layer_uv{i} = cat(3, u, v);
end;

% layer_uv = projective2layer(A, sz);


