function [AO score seg layer_uv ]= robust_fit_projective_motion(uv, K, iniA)

% segment the motion field UV into K groups, each group consistent with an
% affine motion model
% u = a1x + a2y + a3 + a7*x^2 + a8 xy;
% v = a4x + a5y + a6 + a7 xy  + a8 y^2;

uv = readFlowFile('/u/dqsun/research/program/nips10_flow/data/init/classic+nl/train_018.flo');
K  = 4;

niters = 100;
wsz = 15;       % size of window for estimating initial affine parameters

sz = size(uv); sz = sz(1:2);

nData = prod(sz);

u = uv(:,:,1); u = u(:);
v = uv(:,:,2); v = v(:);

[X Y] = meshgrid(1:sz(2), 1:sz(1));
x   = X(:); 
y   = Y(:);
ze  = zeros(nData, 1);
on  = ones(nData, 1);

xy1 = [x y on ze ze ze x.^2 x.*y;...
       ze ze ze x y on x.*y y.^2 ];
   
if nargin >= 3
    % use initial starting point
    A = iniA;    
else
    % K-means++ initialization
    A = randn(8,K);
    cx  = min( max( round(rand(1,K)*sz(2)), 1), sz(2)-wsz);
    cy  = min( max( round(rand(1,K)*sz(1)), 1), sz(1)-wsz);
    
    for j = 1:K
        
        if j>=2
            dist = zeros(nData, j-1);
            for j1 = 1:j
                dist(:,j1) = (u(:)-xy1(1:nData,:)*A(:,j1)).^2 +  (v(:)-xy1(nData+1:end,:)*A(:,j1)).^2;
            end;
            % closest distance to nearest assigned center
            tmp = min(dist, [], 2);
            pp = tmp/sum(tmp(:));
            
            % generate cumulative distribution of previous time posterior
            cpp = cumsum(pp);
            
            % generate random point according to weighted probability
            idx = find(cpp>=rand());
            cx(j) = min( max( x(idx(1)), 1), sz(2)-wsz);
            cy(j) = min( max( y(idx(1)), 1), sz(1)-wsz);
            
        end;
        
        % Fit projective motion to the small patch
        
        tmp_u = uv(cy(j):cy(j)+wsz, cx(j):cx(j)+wsz, 1);
        tmp_v = uv(cy(j):cy(j)+wsz, cx(j):cx(j)+wsz, 2);
        tmp_uv = [tmp_u(:); tmp_v(:)];
        
        tmp_x = X(cy(j):cy(j)+wsz, cx(j):cx(j)+wsz);
        tmp_y = Y(cy(j):cy(j)+wsz, cx(j):cx(j)+wsz);
        
        tmp_x = tmp_x(:); tmp_y = tmp_y(:);
        ze  = zeros(size(tmp_x));
        on  = ones(size(tmp_x));
        
        tmp_xy1 = [tmp_x tmp_y on ze ze ze tmp_x.^2 tmp_x.*tmp_y;...
               ze ze ze tmp_x tmp_y on tmp_x.*tmp_y tmp_y.^2 ];
        
        A(:,j) = L2_fit(tmp_uv(:), tmp_xy1);
        
    end;
end;


dist = zeros(nData, K);
% Initialize the assignment
for j = 1:K
    dist(:,j) = (u(:)-xy1(1:nData,:)*A(:,j)).^2 +  (v(:)-xy1(nData+1:end,:)*A(:,j)).^2;
end;
[tmp, y] = min(dist, [], 2);


% "K-means" Iteration
for i = 1:niters

    prev_y = y;
    
    rep_y = [y;y];
    % Compute new motion parameters for each group    
    for j = 1:K
       tmp_u = u(y==j);
       tmp_v = v(y==j);
       tmp_xy1 = xy1(rep_y==j, :); 
       A(:,j) = L2_fit([tmp_u; tmp_v], tmp_xy1);       
    end    
    
    % Compute distance to each motion group center (fitting error)
    for j = 1:K
        dist(:,j) = (u(:)-xy1(1:nData,:)*A(:,j)).^2 +  (v(:)-xy1(nData+1:end,:)*A(:,j)).^2;
    end;   
       
    % Re group each motion point
    [tmp, y] = min(dist, [], 2);
    
    if sum(abs(y-prev_y)) == 0
        %fprintf('robust_fit_affine_motion converges after %d iteration, error %3.3e\n', i, sum(tmp(:)) );
        break;
    end;

end;

% figure; imagesc(reshape(y, sz));


if nargout >=1
    AO = A;
end;

if nargout >= 2    
    score = sum(tmp(:));
    %title(sprintf('%3.3e', score));
end;

if nargout >= 3
    seg = reshape(y, sz);
end;

if nargout >=4
    for j = 1:K
        uv2 = xy1*A(:,j);
        layer_uv{j} = reshape(uv2, [sz 2]);
    end;
end;

%% observe fitted flow fields for debug

for j = 1:K
    uv2 = xy1*A(:,j);
    layer_uv{j} = reshape(uv2, [sz 2]);
end;
%%
layer_uv = projective2layer(A, sz);

ind = reshape(y, sz);
uv2  = layer_uv{1};
for i1 = 2:length(layer_uv)
    uv2(ind ==i1) = layer_uv{i1}(ind==i1);
end;
figure; imshow(flowToColor(uv2));