function [layer_uv, layer_uvb, seg_o, incon_o, layer_g_o] =...
    compute_init_mf(model, ini_layer_uv, ini_layer_uvb, ...
    pyramid_images, color_pyramid_images, ini_layer_g, init_uvs)


% convert continuous initial value to discrete 
T = length(ini_layer_uv)+1;
nLayers = length(ini_layer_uv{1});
layer_g = [];
for t=1:T-1
    layer_g = cat(4, layer_g, ini_layer_g{t});
    layer_uv{t} =[];
    layer_uvb{t} =[];
    for layerID=1:nLayers
        layer_uv{t}  = cat(4, layer_uv{t}, ini_layer_uv{t}{layerID});
        layer_uvb{t} = cat(4, layer_uvb{t}, ini_layer_uvb{t}{layerID});
    end;
end;
t = T;
layer_g = cat(4, layer_g, ini_layer_g{t});

% convert continuous support to binary mask
layer_g = double(layer_g>0);
mfParams = load_mean_field_parameters(model.dataset_name);

% To easy switch between pre-allocated inference and original inference
if mfParams.seq_update
    mf_inference_2layers = @perform_mean_field_inference_2layers_prealloc;
else
    mf_inference_2layers = @perform_mean_field_inference_2layers_prealloc_FB;
end;


for t = 1:length(model.class_mask)
    mfParams.sseg_layer_q(:,:,1,t,1) = model.class_mask{t}==2;
    mfParams.sseg_layer_q(:,:,1,t,2) = model.class_mask{t}==1;
end

layer_q = [];
if mfParams.possionInt 
    [layer_uv,layer_uvb]=inpaint_flow_possion_mf(layer_g,layer_uv,layer_uvb);
end

for mfIter = 1:mfParams.mfOuterIters
    
    % given candidate flow fields, perform mean field approximation to
    % solve for the approximate marginals for the support functions
    layer_q = mf_inference_2layers(mfParams, pyramid_images,...
        color_pyramid_images, layer_uv, layer_uvb, layer_g, layer_q);
    layer_g = layer_q(:,:,:,:,1) >= layer_q(:,:,:,:,2);
    
    % given the max model for the mean field approximation results,
    % estimate the motion for each layer
    
    if mfIter ==1
        % fuse initial non-layer flow with affine flow field of each layer
        mfParams.isFlowFusion  = true;
        [layer_uv, layer_uvb] = estimate_flow_given_layer_mf(mfParams, ...
            pyramid_images{1}, color_pyramid_images{1}, layer_q,...
            layer_uv, layer_uvb, model, init_uvs);
        
        if mfParams.possionIntTwice
            [layer_uv, layer_uvb] = inpaint_flow_possion_mf(layer_g,...
                layer_uv, layer_uvb);
        end;        
    else        
        mfParams.isFlowFusion  = false;
        [layer_uv, layer_uvb] = estimate_flow_given_layer_mf(mfParams, ...
            pyramid_images{1}, color_pyramid_images{1}, layer_q,...
            layer_uv, layer_uvb, model);
    end;

end;

ini_layer_uv  = layer_uv;
ini_layer_uvb = layer_uvb;

% convert discrete solution to continuous 
layer_seg   = support2seg(double(layer_g));
for t=1:T
    seg_o{t} = layer_seg(:,:,t);
    layer_g_o{t} = (2*layer_g(:,:,:,t)-1)*model.visG;
end;
for t=1:T-1
    uvf = composite_final_flow_seg(layer_uv{t},  seg_o{t});    
    tmp = warp_backward_segmentation(seg_o{t+1}, uvf);
    incon_o{t} = repmat(tmp~=seg_o{t}, [1 1 nLayers-1]);
end;
layer_uv = cell(T-1,1);
layer_uvb= layer_uv;
for t=1:T-1
    for layerID=1:nLayers
        layer_uv{t}{layerID}  = ini_layer_uv{t}(:,:,:,layerID);
        layer_uvb{t}{layerID} = ini_layer_uvb{t}(:,:,:,layerID);
    end;
end;




