function [outputData layer_seg flow_sel can_uv] = parse_data(data, params)


inputDownRatio = params.inputDownRatio;

seg = data.seg;
layer_g = data.layer_g;
color_pyramid_images = data.color_pyramid_images;
pyramid_images  = data.pyramid_images;
ini_layer_uv   = data.ini_layer_uv;
ini_layer_uvb  = data.ini_layer_uvb;

inputSZ = size(data.seg{1});
downSZ  = round(inputSZ/inputDownRatio);

%% 
layer_uv = ini_layer_uv; 
layer_mean_affine_uv = fit_affine_layer_motion_g(layer_uv{1}, layer_g{1});
layer_uvb = ini_layer_uvb; 
layer_mean_affine_uvb = fit_affine_layer_motion_g(layer_uvb{1}, layer_g{2});

% down sample
sseg{1} = imresize(seg{1}, downSZ, 'nearest');
sseg{2} = imresize(seg{2}, downSZ, 'nearest');

for i=1:length(layer_uv{1})
    layer_uv_s{i} = imresize(layer_uv{1}{i}, downSZ)/inputDownRatio;
    layer_mean_affine_uv_s{i} = imresize(layer_mean_affine_uv{i}, downSZ)/inputDownRatio;
    layer_uvb_s{i} = imresize(layer_uvb{1}{i}, downSZ)/inputDownRatio;
    layer_mean_affine_uvb_s{i} = imresize(layer_mean_affine_uvb{i}, downSZ)/inputDownRatio;
end;

images = imresize(pyramid_images{1}, downSZ);

if size(color_pyramid_images{1}, 4)>1
    labImages = {imresize(color_pyramid_images{1}(:,:,:,1), downSZ), imresize(color_pyramid_images{1}(:,:,:,2), downSZ)};
else
    labImages = {imresize(color_pyramid_images{1}(:,:,1), downSZ), imresize(color_pyramid_images{1}(:,:,2), downSZ)};
end;
%

layer_seg = cat(3, sseg{1}, sseg{2});

uvf = composite_final_flow_seg(layer_uv_s,  layer_seg(:,:,1));
uvb = composite_final_flow_seg(layer_uvb_s, layer_seg(:,:,2));

% candidate flow : affine plus single-layered result
can_uvf = [];
can_uvb = [];
for i=1:length(layer_uv_s)    
    can_uvf = cat(4, can_uvf, layer_mean_affine_uv_s{i});
    can_uvb = cat(4, can_uvb, layer_mean_affine_uvb_s{i});

    % comment above and uncomment below for sanity check
%     can_uvf = cat(4, can_uvf, layer_uv_s{i});
%     can_uvb = cat(4, can_uvb, layer_uvb_s{i});
end;    
can_uvf = cat(4, can_uvf, uvf);
can_uvb = cat(4, can_uvb, uvb);

can_uv = {can_uvf, can_uvb};

% luv = [];luv2 = [];
% for i=1:length(layer_uv_s)
%     luv = cat(4, luv, layer_uv_s{i});
%     luv2 = cat(4, luv2, layer_uvb_s{i});
% end;    

for t=1:2;
    flow_sel{t} = [];
    for i=1:length(layer_uv_s)
        tmp = ones(downSZ)*i;
        % comment for sanity check, together with the flow field above
        tmp(layer_seg(:,:,t)==i) = 4;
        flow_sel{t} = cat(3, flow_sel{t}, tmp);
    end;
end;

outputData.images = {images, images(:,:,2:-1:1)};
outputData.labImages = labImages;