function data = precompute(data, can_uv, params)
%%
sz      = size(can_uv{1});
H       = sz(1); 
W       = sz(2);
nCan    = sz(4);

% compute data term cost, temporal neighbors, out-of-boundary mask
for t=1:length(can_uv)
    
    data.matchCost{t}     = zeros([H W nCan]);
    data.tempNeighbors{t} = zeros([H W nCan]);
    data.OOB{t}           = zeros([H W nCan]);
    
    images = data.images{t};
    
    for iCan=1:nCan
        
        % OOB & temporal structure
        [neighbors OOB] = compute_temporal_neigbhors(can_uv{t}(:,:,:,iCan));
        data.OOB{t}(:,:,iCan)           = OOB;
        data.tempNeighbors{t}(:,:,iCan) = neighbors;
 
        % matching cost
        It   = partial_deriv(images, can_uv{t}(:,:,:,iCan),...
             params.interpolation_method, params.deriv_filter);               
        
        cost = reshape(evaluate(params.rho_data, It(:)), [H W]);        
        % treat out-of-boundaries pixels as being occluded
        cost(OOB) = params.lambdaOcc;        
        data.matchCost{t}(:,:,iCan)     = cost;
    end;
    
end;

% spatial cliques, weights for layer segmentation
vert = im2col((reshape(1:H*W,H,W)),[2 1])';
hor  = im2col((reshape(1:H*W,H,W)),[1 2])';

% compute weights for layer segmentation
for t=1:params.T
    wv = compute_color_weight(data.labImages{t}, vert, params.T2_lab,...
        params.sigma_lab, params.rho_color);
    wh = compute_color_weight(data.labImages{t}, hor,  params.T2_lab,...
        params.sigma_lab, params.rho_color);
    
    weight = repmat([wv(:)' wh(:)'], [4 1]);
    weight = weight(:)';
    
    data.weight{t} = weight;
    
end;