function layer_seg = segment_layers_binary_gc(layer_seg, flow_sel,...
        all_can_uv, seg_id, params, data)
%% Given with-layer flow field, assign pixels to different layers


selected = (layer_seg == seg_id(1) ) | (layer_seg == seg_id(2) );

[H W nCan] = size(flow_sel{1});

nCan = length(seg_id);

nStates  = nCan;

ind = reshape(1:H*W, [H W]);


for t=1:params.T
    
    for i = 1:nCan
        
        id = seg_id(i);
        
%         flow_seg = flow_sel{t}(:,:,id);
%         can_uv{t}(:,:,:,i) = composite_final_flow_seg(all_can_uv{t},...
%             flow_seg);
%         
%         init_flow_sel{t}(:,:,i) = flow_seg;
        
        tmp_ind = ind + (flow_sel{t}(:,:,id)-1)*H*W;        
        matchCost{t}(:,:,i)= reshape(data.matchCost{t}(tmp_ind), [H W]);
        OOB{t}(:,:,i)      = reshape(data.OOB{t}(tmp_ind), [H W]);                   
        
        neigh = data.tempNeighbors{t}(:,:,1);        
        for iL = 2:size(data.tempNeighbors{t}, 3)
%         for iL = 2:length(all_can_uv)
            tmp = data.tempNeighbors{t}(:,:,iL);
            mask = flow_sel{t}(:,:,id) == iL;
            neigh(mask) = tmp(mask);                        
        end;
        tempNeighbors{t}(:,:,i) = neigh;       
    end;        
end;

% construct pairwise spatial term
vert = im2col((reshape(1:H*W,H,W)),[2 1])';
hor  = im2col((reshape(1:H*W,H,W)),[1 2])';

for t=1:params.T
    spatial_func{t}   = @(u) compute_pw_spatial_layer_potential(u,...
        selected(:,:,t), data.weight{t}, params);
end;

% construct pairwise temporal data term
indexInc = params.startIndex-1;

ind = reshape(1:H*W, [H W]);
for t=1:params.T
    ind_sel = ind(selected(:,:,t));
    temporal{t} = [];
    for iL = 1:nCan
        tmp = tempNeighbors{t}(:,:,iL);
        tmp = tmp(ind_sel);
        temporal{t} = [temporal{t};...
            ind_sel(:)+indexInc(t) tmp(:)+indexInc(t+1)];
    end;    
end;

t=1;
temp_fun{t} = @(u) compute_pw_t_layer_potential(u, ...
    matchCost{t}, layer_seg(:,:,t+1), seg_id, params, ...
    OOB{t}, selected(:,:,t+1));

t=2;
temp_fun{t} = @(u) compute_pw_t_layer_potential(u, ...
    matchCost{t}, layer_seg(:,:,t-1), seg_id, params, ...
    OOB{t}, selected(:,:,t-1));


%%%%%%%%%%%%%%% alpha expansion

% no data unary term
data_unary = [1:2*H*W; zeros(2, 2*H*W)]; 

% initial state
% tmp_seg = layer_seg;   
% tmp_seg(~selected) = seg_id(1);  

tmp_seg = zeros(size(layer_seg));   
tmp_seg(layer_seg==seg_id(2)) = 1;  

fp = tmp_seg(:);

% fp = invoke_gc_solver(fp, vert, hor, spatial_func, temporal, nStates, H, W, data, params);

current_energy = 1e32;
success = 0;
% start alpha expansion
while true    
    for iState=0:nStates-1
        current_proposal = iState *ones(size(fp));

        % pairwise spatial term
        spa1 = generateEnergyTable([vert; hor], spatial_func{1},...
            current_proposal(1:H*W), fp(1:H*W));
        spa2 = generateEnergyTable([vert; hor], spatial_func{2},...
            current_proposal(1+H*W:end), fp(1+H*W:end));
        spa2(1:2,:) = spa2(1:2,:) + H*W;
        
        % data pariwise term (temporal)
        t = 1;
        temp1 = generateEnergyTable(temporal{t}, temp_fun{t},...
                current_proposal, fp);
        t = 2;
        temp2 = generateEnergyTable(temporal{t}, temp_fun{t},...
                current_proposal, fp);
            
        pairwise_terms  = [spa1 spa2 temp1 temp2];
        
        [new_fp,new_energy] = QPBO_all_in_one(H*W*2, data_unary, pairwise_terms);
        
        if (new_energy - current_energy) > .5
            %if we use potentials of type double, it is
            %possible that the new energy grows, because of numerical
            %issues, see QPBO documentation
            
            %             disp(strcat('WARNING: energy grows, (new_energy - current_energy) >'...
            %                 ,num2str(new_energy - current_energy)));
        elseif (new_energy - current_energy) < -1
            success = 1;
            current_energy=new_energy;
        end
        
        %fprintf('%3.3f ', current_energy);
        
        fp = (1-new_fp).* current_proposal+ new_fp .* fp;%generate new labeling
        
    end
    %fprintf('\n');
    
    if success == 0
%         disp('finished, energy did not shrink');
        break;
    end
    success = 0;
end;
%fprintf('%3.3f \n', current_energy);

% parse collapsed states to individual variables
fp = reshape(fp, [H W 2]);

% tmp = fp+1;
% layer_seg(selected) = tmp(selected);

tmp_seg = fp;
tmp_seg(fp==1) = seg_id(2);
tmp_seg(fp==0) = seg_id(1);
layer_seg(selected) = tmp_seg(selected);


