function [matchCost OOB  tempNeighbors] = compute_data_match_cost_oob(...
    layer_uv, images, params)
%%
% compute data term matching cost, out-of-boundary mask, and temporal
% neighbors (integer)

[H W nF nCan] = size(layer_uv);

matchCost     = zeros([H W nCan]);
OOB           = zeros([H W nCan]);
if nargout >2
    tempNeighbors = zeros([H W nCan]);
end;

for iCan=1:nCan

    if strcmp(params.downOption, 'hires_0.5')
       
        % compute data match cose on original images
        [neighbors oob] = compute_temporal_neigbhors(...
            layer_uv(:,:,:,iCan)/params.inputDownRatio);
        OOB(:,:,iCan)   = oob;
        
        if nargout >2
            tempNeighbors(:,:,iCan) = neighbors;
        end;
        
        % matching cost
        sz = size(images);
        sz = sz(1:2);
        uv  = imresize(layer_uv(:,:,:,iCan), sz,...
            'nearest');
        
        It  = partial_deriv(images, uv,...
            params.interpolation_method, params.deriv_filter);
        
        It  = imresize(It, 1/params.inputDownRatio, 'nearest');
        
        cost = reshape(evaluate(params.rho_data, It(:)), [H W]);
        
        % treat out-of-boundaries pixels as being occluded
        cost(oob)               = params.lambdaOcc;
        matchCost(:,:,iCan)     = cost;        
        
        
    else
        % OOB & temporal structure
        [neighbors oob] = compute_temporal_neigbhors(layer_uv(:,:,:,iCan));
        OOB(:,:,iCan)   = oob;
        
        if nargout >2
            tempNeighbors(:,:,iCan) = neighbors;
        end;
        
        % matching cost
        It   = partial_deriv(images, layer_uv(:,:,:,iCan),...
            params.interpolation_method, params.deriv_filter);
        
        cost = reshape(evaluate(params.rho_data, It(:)), [H W]);
        
        % treat out-of-boundaries pixels as being occluded
        cost(oob)               = params.lambdaOcc;
        matchCost(:,:,iCan)     = cost;
    end;
end;
