function [data solution params] = compute_init_gc_all_asym(data, solution, params, this)
%%
%%
%  visiblity per layer
%  visibility and flow per layer
%  support per layer
%  affine and continuous flow per layer

en = evaluate_overall_energy_si_asym(solution, data, params, params.display);
fprintf('init energy %3.3e\n', en);

minData     = data;
minSolution = solution;
minParams   = params;
minEn       = en;

for iter=1:params.nGCiters
    
    if ~params.isBasicMove
        
        % these moves involves changing the states of several
        % binary MRFs simultaneously
        
        %%%%%%%%%% layer segmentation
        solution = segment_layers_gc_si_asym(solution, params, data);
        
        % remove unnecessary layers
        [data solution params] = prune_layers_asym(data, solution, params);
        data    = precompute_asym(data, params);
        en      = evaluate_overall_energy_si_asym(solution,...
            data, params, params.display);
        fprintf('seg move energy %3.3e\n', en);
        
        if ~strcmp(params.supMFType, 'no_mf')
            %%% (weighted) median filtering
            solution = postprocess_support_functions(solution, data, params);
        else
            % only accepts lower-energy solution
            if en < minEn
                minData     = data; minSolution = solution;
                minParams   = params; minEn       = en;
            else
                data = minData; solution = minSolution;
                params = minParams;
            end;
        end;
        
        %%%%%%%%%% simultaneous layer segmentation + flow selection
        [solution data]= segment_layers_select_flow_gc_si_asym(...
            solution, params, data);
        
        % remove unnecessary layers
        [data solution params] = prune_layers_asym(data, solution, params);
        %update data.matchCost since the layer flow fields (layer_uv) change
        %in segment_layers_select_flow_gc_si_asym
        data    = precompute_asym(data, params);
        
        en = evaluate_overall_energy_si_asym(solution,...
            data, params, params.display);
        fprintf('seg & flow move energy %3.3e\n', en);
        
        if ~strcmp(params.supMFType, 'no_mf')
            %%% (weighted) median filtering
            solution = postprocess_support_functions(solution, data, params);
        else
            % only accepts lower-energy solution
            if en < minEn
                minData     = data; minSolution = solution;
                minParams   = params; minEn       = en;
            else
                data = minData; solution = minSolution;
                params = minParams;
            end;
        end;
        
    end;
    
    %%%%%%%%%% update support function for each layer
    solution = segment_layers_change_support_gc_si_asym(solution, params, data);
    
    % remove unnecessary layers
    [data solution params] = prune_layers_asym(data, solution, params);
    data    = precompute_asym(data, params);
    en      = evaluate_overall_energy_si_asym(solution,...
        data, params, params.display);
    fprintf('support move energy %3.3e\n', en);
    
    if ~strcmp(params.supMFType, 'no_mf')
        %%% (weighted) median filtering
        solution = postprocess_support_functions(solution, data, params);
    else
        % only accepts lower-energy solution
        if en < minEn
            minData     = data; minSolution = solution;
            minParams   = params; minEn       = en;
        else
            data = minData; solution = minSolution;
            params = minParams;
        end;
    end;
    
    %%%%%%%%%% flow refinement
    [data solution params] = refine_flow_move(data, solution, params, this);
    %update data.matchCost since the layer flow fields (layer_uv) change
    data    = precompute_asym(data, params);
    en      = evaluate_overall_energy_si_asym(solution,...
        data, params, params.display);
    fprintf('cont flow refinement energy %3.3e\n', en);
    
    % only accepts lower-energy solution
    if en < minEn
        minData     = data;     minSolution = solution;
        minParams   = params;   minEn       = en;
    else
        data = minData; solution = minSolution;
        params = minParams;
    end;
    
end

data     = minData;
params   = minParams;
solution = minSolution;