function out = compute_pw_t_potential_si_asym(vals, ...
                matchCost, params, OOB, prevP, currP, t1, isForward)
%%
% compute the temporal pairwise term for layer segmentation with the
% sequential Ising model. the input states, vals(3:4,:), select which group
% of layer support (3D array) to use  

% only for FORWARD flow field temporal terms

% 0: curr proposal used for each layer
% 1: prev proposal used for each layer

% matchCost is the matching cost from t to t+1
% PrevP and currP are the support functions at time t and t+1
% t1 is t
%
%
% %%
% H = 10; W = 10; nP = 3; nLayers = nP+1; T = 2;
% 
% prevP = round(rand(H, W, nP, T));
% currP = round(rand(H, W, nP, T));
% OOB   = round(rand(H, W, nLayers));
% 
% % prevP = (ones(H, W, nP, T));
% % currP = (ones(H, W, nP, T));
% % OOB   = (zeros(H, W, nLayers));
% matchCost = 20*(rand(H, W, nLayers));%round(rand(H, W, nLayers));
% 
% vals = round(rand(4, H*W*nLayers*4))+1;
% vals(1,:) = round(rand(1, H*W*nLayers*4)*(H*W-2))+1;
% vals(2,:) = round(rand(1, H*W*nLayers*4)*(H*W-2))+1+H*W;
% vals(3:4, :) = double(vals(3:4, :) ==1);
% 
% t1          = 1;
% isForward   = true;
% lambdaOcc   = 1;
% lambdaIncon = 0;
% params.lambdaOcc    = lambdaOcc;
% params.lambdaIncon  = lambdaIncon;
% 
% %%
% tic;
% out = compute_pw_t_potential_si_asym(vals, ...
%                 matchCost, params, OOB, prevP, currP, t1, isForward);
% toc
% %
% tic;
% out2 = compute_pw_t_potential_si_asym_mex(vals, ...
%                 matchCost, lambdaOcc, lambdaIncon,...
%                 OOB, prevP, currP, t1, isForward);
% toc
% %
% figure; plot(out~=out2, '.');
% %%
% out = compute_pw_t_potential_si_asym_mex(vals, matchCost, lambdaOcc, lambdaIncon,OOB, prevP, currP, t1, isForward);
%

if ~exist('isForward', 'var');
    isForward = true;
end;

[H W nP T] = size(prevP);
% n Layers have n-1 support functions
nLayers    = nP+1;

HP    = size(vals,2)/nLayers;

prevV = support2visibility(prevP);
currV = support2visibility(currP);

if isForward
    ind1  = vals(1,:) - (t1-1)*H*W;
    % for indexing visibility masks, layer_ID will be added later
    indV1 = ind1;
    % for indexing support
    indP1 = ind1(1:HP*(nLayers-1));
    
    ind2  = vals(2,:) - t1*H*W ;
    % for indexing visibility masks
    indV2 = ind2 + nLayers*H*W;
    % for indexing support
    indP2 = ind2(1:HP*(nLayers-1)) + (nLayers-1)*H*W;
else
    ind1  = vals(1,:) - t1*H*W;
    % for indexing visibility masks
    indV1 = ind1 + nLayers*H*W;
    % for indexing support
    indP1 = ind1(1:HP*(nLayers-1)) + (nLayers-1)*H*W;
    
    ind2  = vals(2,:) - (t1-1)*H*W ;
    % for indexing visibility masks
    indV2 = ind2;
    % for indexing support
    indP2 = ind2(1:HP*(nLayers-1));
end;    

% obtain layer IDs for each state
layerID = repmat([1:nLayers], [HP 1]);
layerID = layerID(:)';
  
%%%%%%%%%%%% data matching cost
tmp_ind  = ind1 + (layerID-1)*H*W;  
out      = matchCost(tmp_ind);

% out of boundary
% oob      = OOB(tmp_ind);
% out(oob==1) = params.lambdaOcc;

tmp_ind = indV2 + (layerID-1)*H*W;  
vis2    = prevV(tmp_ind).*vals(4,:) + ...
          currV(tmp_ind).*(1-vals(4,:));
            
% % invisible at other frame
out(vis2==0) = params.lambdaOcc;
% 
% % invisible at current (center) frame
tmp_ind = indV1 + (layerID-1)*H*W;  
vis1    = prevV(tmp_ind).*vals(3,:) + ...
          currV(tmp_ind).*(1-vals(3,:));
out(vis1==0) = 0;      

%%%%%%%%%%% time term for support functions
% out = out*0; 

% corresponding nodes at frame t+1
% tmp_ind = indP2;
% for iL =1:nLayers-1
%     tmp_ind( (iL-1)*HP+1:iL*HP ) = ...
%         tmp_ind( (iL-1)*HP+1:iL*HP ) + (iL-1)*H*W;
% end;
tmp_ind = indP2 + (layerID(1:HP*(nLayers-1))-1)*H*W;
sup2    = prevP(tmp_ind).*vals(4,1:HP*(nLayers-1)) + ...
          currP(tmp_ind).*(1-vals(4,1:HP*(nLayers-1)));

% corresponding nodes at frame t      
% tmp_ind = indP1;
% for iL =1:nLayers-1
%     tmp_ind( (iL-1)*HP+1:iL*HP ) =...
%         tmp_ind( (iL-1)*HP+1:iL*HP ) + (iL-1)*H*W;
% end;
tmp_ind = indP1 + (layerID(1:HP*(nLayers-1))-1)*H*W;
sup1    = prevP(tmp_ind).*vals(3,1:HP*(nLayers-1)) + ...
          currP(tmp_ind).*(1-vals(3,1:HP*(nLayers-1)));      
      
% add temporal consistency term for the nLayers-1 support functions
out(1:HP*(nLayers-1)) = out(1:HP*(nLayers-1)) +...
    params.lambdaIncon*(sup1~=sup2);