function [data solution params ] = determine_depth_ordering_disc_init(...
data, solution, params, this, outputDir);

%%

if nargin < 5
    outputDir   = [this.extParams.discDir];
end;

if this.is_save_mat
    fn = [outputDir 'stage0.mat'];
    save(fn, 'data', 'solution', 'params');
end

% [H W nP T]  = size(solution.layer_support); 
nLayers     = size(solution.layer_support, 3) + 1;
T           = size(solution.layer_support, 4);

%%%%%%%%%% determine depth ordering
%save a copy for depth order reasoning
data2       = data;
solution2   = solution;

%%%%%%%%%% reverse depth ordering, slow-to-fast
layer_seg            = support2seg(solution.layer_support);
for t=1:T-1;
    data.aff_uv{t}   = data2.aff_uv{t}(:,:,:, end:-1:1);
    data.layer_uv{t} = data2.layer_uv{t}(:,:,:, end:-1:1);
end;

if params.is_symmetric
    for t=1:T-1;
        data.aff_uvb{t}   = data2.aff_uvb{t}(:,:,:, end:-1:1);
        data.layer_uvb{t} = data2.layer_uvb{t}(:,:,:, end:-1:1);
    end;
end;

% TODO: switch support does not guarantee the same effect on segmentation
% tmp_layer_seg          = nLayers+1-layer_seg;
% solution.layer_support =  seg2DistLayers(tmp_layer_seg);
solution.layer_support(:,:,1, :) = layer_seg == nLayers;
solution.layer_support(:,:,2:nLayers-1, :) = ...
    solution2.layer_support(:,:,end:-1:2, :);

data       = precompute_asym(data, params);



% [inv_data inv_solution inv_params] = load_or_compute_discrete_init(...
% [outputDir 'inverse.mat'], data, solution, params, this);


fn = [outputDir 'inverse.mat'];
if exist(fn, 'file');
    load(fn);
    inv_paramsTmp       = inv_params;
    inv_params          = params;
    inv_params.nLayers  = inv_paramsTmp.nLayers;
else    
    [inv_data inv_solution inv_params] = compute_init_gc_all_asym(data,...
        solution, params, this);
    if ~strcmp(params.supMFType, 'no_mf')
        %(weighted) median filtering
        inv_solution = postprocess_support_functions(inv_solution,...
            inv_data, inv_params);
    end;
    
    inv_en          = evaluate_overall_energy_si_asym(inv_solution,...
        inv_data, inv_params, 1);
    fprintf('inverse energy %3.3e\n', inv_en);
    
    if this.is_save_mat
        save(fn, 'inv_data', 'inv_solution', 'inv_params', 'inv_en');
    end;   

end;
fprintf('%3.3f min\n', toc/60);

if ~exist('inv_en', 'var');    
    inv_en          = evaluate_overall_energy_si_asym(inv_solution,...
        inv_data, inv_params, 1);
    fprintf('inverse energy %3.3e\n', inv_en);    
    if this.is_save_mat
        save(fn, 'inv_data', 'inv_solution', 'inv_params', 'inv_en');
    end;
end;

% output_intermediate_result(data, solution, 'inverse_init.png');
img_inv         = display_results_si_asym(data, solution);
fn              = [outputDir 'inverse_init.png'];
imwrite(uint8(img_inv), fn);
clear data solution;


img             = display_results_si_asym(inv_data, inv_solution);
fn              = [outputDir 'inverse.png'];
imwrite(uint8(img), fn);


%%%%%%%%% forward depth ordering, fast-to-slow
% [for_data for_solution for_params] = load_or_compute_discrete_init(...
% [outputDir 'forward.mat'], data2, solution2, params, this);

fn              = [outputDir 'forward.mat'];
if exist(fn, 'file');
    load(fn);
    for_paramsTmp = for_params;
    for_params  = params;
    for_params.nLayers = for_paramsTmp.nLayers;
    
else        
    [for_data for_solution for_params] = compute_init_gc_all_asym(...
        data2, solution2, params, this);
    if ~strcmp(params.supMFType, 'no_mf')
        % (weighted) median filtering
        for_solution = postprocess_support_functions(for_solution,...
            for_data, for_params);
    end;
    
    for_en = evaluate_overall_energy_si_asym(for_solution,...
        for_data, for_params, 1);
    fprintf('forward energy %3.3e\n', for_en);
    
    if this.is_save_mat
        save(fn, 'for_data', 'for_solution', 'for_params', 'for_en');
    end;

end;

if ~exist('for_en', 'var');
    for_en = evaluate_overall_energy_si_asym(for_solution,...
        for_data, for_params, 1);
    fprintf('forward energy %3.3e\n', for_en);
    
    if this.is_save_mat
        save(fn, 'for_data', 'for_solution', 'for_params', 'for_en');
    end;

end;

fprintf('%3.3f min\n', toc/60);

% output intermediate result
fn      = [outputDir 'forward_init.png'];
img_for = display_results_si_asym(data2, solution2);
imwrite(uint8(img_for), fn);
clear data2 solution2;

fn      = [outputDir 'forward.png'];
img     = display_results_si_asym(for_data, for_solution);
imwrite(uint8(img), fn);

%%%%%%%% select the ordering with lower energy
if for_en <= inv_en
    img      = img_for; %display_results_si_asym(data2, solution2);
    data     = for_data;
    solution = for_solution;
    params   = for_params;    
else    
    img      = img_inv; %display_results_si_asym(data, solution);
    data     = inv_data;
    solution = inv_solution;
    params   = inv_params;    
end;

% clear memory
clear for_data for_solution for_params;
clear inv_data inv_solution inv_params;

fn = [outputDir 'stage0.png'];
imwrite(uint8(img), fn);


% output intermediate result
img     = display_results_si_asym(data, solution);
fn      = [outputDir 'stage1.png'];
imwrite(uint8(img), fn);

if this.is_save_mat
    fn = [outputDir 'stage1.mat'];
    save(fn, 'data', 'solution', 'params');
end;