function data = estimate_affine_mean_asym(data, solution, params)
%%
% Fit parametric flow field for each layer using the visible pixels
%  set flow fields

layer_seg = support2seg(solution.layer_support);
 
for iL=1:params.nLayers
    
    if sum(layer_seg(:)==iL) < params.nIgnore
        % no or very few visible pixels, set flow to zero        
        for t=1:params.T-1
            %forward flow field
            data.aff_uv{t}(:,:,:,iL)   = 0*data.aff_uv{t}(:,:,:,iL);
            data.layer_uv{t}(:,:,:,iL) = 0*data.aff_uv{t}(:,:,:,iL);
            
            if params.is_symmetric
                %backward flow field
                data.aff_uvb{t}(:,:,:,iL) = 0*data.aff_uvb{t}(:,:,:,iL);
                data.layer_uvb{t}(:,:,:,iL) = 0*data.layer_uvb{t}(:,:,:,iL);
            end;
        end

    else

        % use only visible pixels, replace invisible ones immediately
        for t=1:params.T-1           
            
            %forward flow field
            data.aff_uv{t}(:,:,:,iL) = fit_affine_robust(...
                data.layer_uv{t}(:,:,:,iL), params.rho_affine,...
                layer_seg(:,:,t)==iL );           
            
            data.layer_uv{t}(:,:,:,iL) = replace_flow_mask(...
                data.layer_uv{t}(:,:,:,iL), ...
                data.aff_uv{t}(:,:,:,iL), layer_seg(:,:,t)~=iL);
            
%             mask = layer_seg(:,:,t)~=iL;
%             for iF = 1:size(data.aff_uv{t},3)
%                 uaff = data.aff_uv{t}(:,:,iF,iL);
%                 u    = data.layer_uv{t}(:,:,iF,iL);
%                 u(mask) = uaff(mask);
%                 data.layer_uv{t}(:,:,iF,iL) = u;
%             end;
            
            
            if params.is_symmetric
                %backward flow field
                data.aff_uvb{t}(:,:,:,iL) = fit_affine_robust(...
                    data.layer_uvb{t}(:,:,:,iL), params.rho_affine, ...
                    layer_seg(:,:,t+1)==iL);
                
                data.layer_uvb{t}(:,:,:,iL) = replace_flow_mask(...
                    data.layer_uvb{t}(:,:,:,iL), ...
                    data.aff_uvb{t}(:,:,:,iL), layer_seg(:,:,t+1)~=iL);
            end;
        end            
            
            
          %%% use all the pixels
%         for t=1:params.T-1            
%             data.aff_uv{t}(:,:,:,iL) = fit_affine_robust(...
%                 data.layer_uv{t}(:,:,:,iL), params.rho_affine);
%             
%             if params.is_symmetric
%                 %backward flow field
%                 data.aff_uvb{t}(:,:,:,iL) = fit_affine_robust(...
%                     data.layer_uvb{t}(:,:,:,iL), params.rho_affine);
%             end;
%         end
        
    end;
end;

function uv = replace_flow_mask(uv, uvRef, mask)
mask = repmat(mask, [1 1 2]);
uv(mask) = uvRef(mask);

function uvo = fit_affine_robust(uv, rho, mask, nRLSiters)
sz = size(uv); 
sz = sz(1:2);

if nargin < 4
    nRLSiters = 5; 
end;

%nData = prod(sz);

if nargin <3
    mask = ones(sz)==1;
end;

nData = sum(mask(:)==1);
if nData <20    
    uvo = 0*uv;
else
    
    [X Y] = meshgrid(1:sz(2), 1:sz(1));
    x = X(mask);
    y = Y(mask);
    x = x(:);
    y = y(:);
    xy1 = [x y ones(nData, 1)];
    
    u = uv(:,:,1);
    u = u(mask);
    u = u(:);
    
    v = uv(:,:,2);
    v = v(mask);
    v = v(:);
    
    A(:,1) = L2_fit(u, xy1);
    A(:,2) = L2_fit(v, xy1);
    
    A(:,1) = RLS_fit(u, xy1, rho, nRLSiters, A(:,1));
    A(:,2) = RLS_fit(v, xy1, rho, nRLSiters, A(:,2));
    
    % interpolate invisble pixels by affine motion model
    u = X*A(1,1)+Y*A(2,1)+A(3,1);
    v = X*A(1,2)+Y*A(2,2)+A(3,2);
    
    u = reshape(u, sz);
    v = reshape(v, sz);
    
    uvo = cat(3, u,v);
end;

function a = L2_fit(u, xy1)    
% least square fit u = a1x + a2y + a3;

A = (xy1'*xy1);
b = (xy1'*u);

% if sum(isnan(A(:)))>0
%     fprintf('.\n');
% end

if rank(A) <3
    A = A + 1e-12*max(abs(A(:)))*eye(3,3);
end

a = A\b;

%a = (xy1'*xy1)\(xy1'*u);

function s = RLS_fit(u, xy1, rho, niters, s)

for iter = 1:niters;
    err = u - s(1)*xy1(:,1) -s(2)*xy1(:,2)-s(3);
    w = deriv_over_x(rho, err);
    A = (xy1'* [w.*xy1(:,1) w.*xy1(:,2) w] );
    b = (xy1'* (w.*u));
    
%     if sum(isnan(A(:)))>0
%        fprintf('.\n'); 
%     end
    
    if rank(A) <3
        A = A + 1e-12*max(abs(A(:)))*eye(3,3);
    end
    s = A\b;
end;