function  en = evaluate_overall_energy_si_asym(solution, data,...
                params, display)
%% Evaluate energy for sequential Ising layered models
%  some computations are redundant for sanity check reasons

if nargin < 4
    display = false;
end;

layer_vis = support2visibility(solution.layer_support);

% parse data                        
[H W nF nLayers] = size(data.layer_uv{1}); 

% flow 
[enFlow enAff] = compute_flow_energy_asym(data.layer_uv, data.aff_uv, params);
       
% support function smoothness term
enSeg = 0;
vert = im2col((reshape(1:H*W,H,W)),[2 1])';
hor  = im2col((reshape(1:H*W,H,W)),[1 2])';
for t=1:params.T    
    
    % compute color weight
    wv = compute_color_weight(data.labImages{t}, vert, params.T2_lab, params.sigma_lab);
    wh = compute_color_weight(data.labImages{t}, hor,  params.T2_lab, params.sigma_lab);

    for iL=1:nLayers-1
        tmp = solution.layer_support(:,:,iL,t);
        tmpx = tmp(hor(:,1)) ~= tmp(hor(:,2));
        tmpy = tmp(vert(:,1)) ~= tmp(vert(:,2));
        
        enSeg = enSeg + sum(tmpx(:).*wh(:)) + sum(tmpy(:).*wv(:));
    end;        
end;

enMF = 0;
if isfield(params, 'supMFType') && (~strcmp(params.supMFType, 'no_mf'));
    for t=1:params.T
        for iL=1:nLayers-1
            enMF = enMF + evaluate_weighted_energy_support_function(...
                solution.layer_support(:,:,iL,t), data.labImages{t}, ...
                7, [5 5], params.sigma_lab, params.supMFType);
        end
    end
    
    enMF = enMF * params.lambdaMF;
    %enMF = enMF * params.lambdaSeg/3;
    %fprintf('non-local term %3.5e\n', enMF);
end;


% temporal (data) term
enData = 0;
enIncon = 0;
for t=1:params.T-1  
    for iL=1:nLayers
    
        % matching cost
        
        if strcmp(params.downOption, 'hires_0.5')            
            uv = data.layer_uv{t}(:,:,:,iL);
            [neighbors OOB] = compute_temporal_neigbhors(...
                uv/params.inputDownRatio);        
            sz = size(data.images);
            sz = sz(1:2);
            uv  = imresize(uv, sz, 'nearest');
            It  = partial_deriv(data.images(:,:,t:t+1), uv,...
                params.interpolation_method, params.deriv_filter);            
            It  = imresize(It, 1/params.inputDownRatio, 'nearest');
        else
            uv = data.layer_uv{t}(:,:,:,iL);
            [neighbors OOB] = compute_temporal_neigbhors(uv);
            It = partial_deriv(data.images(:,:,t:t+1), uv,...
                params.interpolation_method, params.deriv_filter);            
        end;
        
        It = reshape(evaluate(params.rho_data, It(:)), [H W]);
        
        % out of bounaries
        It(OOB) = params.lambdaOcc;        
        % occlusion reasoning by checking visibility        
        vis1 = layer_vis(:,:,iL,t);
        vis2 = layer_vis(:,:,iL,t+1);
        
        warpV = vis2(neighbors);        
        It(warpV==0) = params.lambdaOcc;
        It(vis1==0)  = 0;        

        enData = enData+ sum(It(:));
% fprintf('%3.5e, ', enData);
        % time term for layer support        
        if iL < nLayers
            sup1 = solution.layer_support(:,:,iL,t);           
            sup2 = solution.layer_support(:,:,iL,t+1);            
            warpSup = sup2(neighbors);            
            enIncon = enIncon+ params.lambdaIncon*sum(sup1(:)~=warpSup(:));            
        end;
        
    end;
end;
% fprintf('\n');

if params.is_symmetric

    % flow
%     [enFlowb enAffb] = compute_flow_energy_asym(data.layer_uvb,...
%     data.aff_uvb, params, params.t0b);
    [enFlowb enAffb] = compute_flow_energy_asym(data.layer_uvb,...
    data.aff_uvb, params);
    enFlow = enFlow+enFlowb;
    enAff  = enAff + enAffb;
    
    % BACKWARD temporal (data) term
    enDatab = 0;
    enInconb = 0;
    for t=1:params.T-1
        for iL=1:nLayers
            
            % matching cost
            
            if strcmp(params.downOption, 'hires_0.5')
                uv = data.layer_uvb{t}(:,:,:,iL);
                [neighbors OOB] = compute_temporal_neigbhors(...
                    uv/params.inputDownRatio);                
                sz = size(data.images);
                sz = sz(1:2);                
                uv  = imresize(uv, sz, 'nearest');                
                It  = partial_deriv(data.images(:,:,t+1:-1:t), uv,...
                    params.interpolation_method, params.deriv_filter);
                It  = imresize(It, 1/params.inputDownRatio, 'nearest');
            else
                uv = data.layer_uvb{t}(:,:,:,iL);
                [neighbors OOB] = compute_temporal_neigbhors(uv);
                It = partial_deriv(data.images(:,:,t+1:-1:t), uv,...
                    params.interpolation_method, params.deriv_filter);                
            end;
            It = reshape(evaluate(params.rho_data, It(:)), [H W]);
            
            % out of bounaries
            It(OOB) = params.lambdaOcc;
            % occlusion reasoning by checking visibility
            vis1 = layer_vis(:,:,iL,t+1);
            vis2 = layer_vis(:,:,iL,t);
            
            warpV = vis2(neighbors);
            It(warpV==0) = params.lambdaOcc;
            It(vis1==0)  = 0;
            
            enDatab = enDatab+ sum(It(:));
% fprintf('%3.5e, ', enDatab);            
            % time term for layer support
            if iL < nLayers
                sup1 = solution.layer_support(:,:,iL,t+1);
                sup2 = solution.layer_support(:,:,iL,t);
                warpSup = sup2(neighbors);
                enInconb = enInconb+ params.lambdaIncon*sum(sup1(:)~=warpSup(:));
            end;
            
        end;
    end;

    enData  = enData+enDatab;
    enIncon = enIncon+enInconb;

end;
% fprintf('\n');

%%% for debug, must mask off for normal runs
% en = enFlow*params.lambdaFlow + enSeg*params.lambdaSeg ...
%     +enAff*params.lambdaAff + enIncon + enMF; 

en = enFlow*params.lambdaFlow + enSeg*params.lambdaSeg + enData...
    +enAff*params.lambdaAff + enIncon + enMF; 
if display
%     fprintf('flow smoothness %3.5e\t flow deviation %3.5e\t', ...
%         enFlow*params.lambdaFlow, enAff*params.lambdaAff);
%     fprintf('sup nlm %3.3e\t', enMF);
%     fprintf('sup smoothness %3.3e\t data %3.3e\t time %3.3e\t\n', ...
%         enSeg*params.lambdaSeg,enData, enIncon);

    fprintf('flow spa \t flow dev \t', ...
        enFlow*params.lambdaFlow, enAff*params.lambdaAff);
    fprintf('sup nlm \t', enMF);
    fprintf('sup spa \t data \t time \t\n', ...
        enSeg*params.lambdaSeg,enData, enIncon);

    fprintf(' %3.5e, %3.5e, ', ...
        enFlow*params.lambdaFlow, enAff*params.lambdaAff);
    fprintf('%3.5e, ', enMF);
    fprintf('%3.5e,  %3.5e,  %3.5e, \n', ...
        enSeg*params.lambdaSeg,enData, enIncon);
    
    
    %display_results_si_asym(data, solution); 
end;

function [enFlow enAff] = compute_flow_energy_asym(layer_uv, aff_uv, params, t0)

% flow smoothness + deviation from affine

if nargin < 4;
    t0 = 1;
end;

nLayers = size(layer_uv{1}, 4);

enFlow=0;
Dx = [1 -1];
Dy = Dx';
for t=t0:params.T-1
    for i = 1:nLayers
        for j=1:2
            if strcmp(params.flowPrior, 'mrfOnDev')
                tmp = layer_uv{t}(:,:,j,i) - aff_uv{t}(:,:,j,i);
            else
                tmp = layer_uv{t}(:,:,j,i);
            end;
            
            % horizontal derivative
            tmpx = conv2(tmp, Dx, 'valid');
            enFlow = enFlow + sum( evaluate(params.rho_spatial, tmpx(:)) );
            
            % vertical derivative
            tmpy = conv2(tmp, Dy, 'valid');
            enFlow = enFlow + sum( evaluate(params.rho_spatial, tmpy(:)) );
            
        end;

    end;
end;

% deviation from mean affine flow fields
enAff=0;
for t=t0:params.T-1    
    tmp = layer_uv{t}-aff_uv{t};    
    tmp = evaluate(params.rho_affine, tmp);
    enAff=enAff+sum(tmp(:));    
end;