function uv = fuse_flow_gc_si_asym(solution, t_curr, layer_id, params,...
    data, isForward)
                
%%
% given layer segmentation, select flow field for each layer between the
% current continuous flow field and its affine mean flow field
% temporal consistency of layer support, deviation from affine flow fields

if ~exist('isForward', 'var');
    isForward = true;
end;
layer_vis   = support2visibility(solution.layer_support);
nLayers     = size(layer_vis,3);

if isForward    
    
    if isfield(params, 'useAaffine') && ~params.useAaffine
        can_uv = data.layer_uv{t_curr}(:,:,:,layer_id);
    else
        can_uv = cat(4, data.aff_uv{t_curr}(:,:,:,layer_id),...
            data.layer_uv{t_curr}(:,:,:,layer_id));
    end;
    aff_uv = data.aff_uv{t_curr}(:,:,:,layer_id);

    if params.flowFusionFeature
        % TODO: only selected relevant feature matching in selected regions
        % Add candidate flow fields from feature matching
        can_uv = cat(4, can_uv, data.feature_uv{t_curr});
    end;
   
    [matchCost, OOB tmpIndex] = compute_data_match_cost_oob(can_uv,...
        data.images(:,:,t_curr:t_curr+1), params);
    
    if layer_id < nLayers
        sup1 = solution.layer_support(:,:,layer_id, t_curr);
        sup1 = sup1(:);
        sup2 = solution.layer_support(:,:,layer_id, t_curr+1);        
    end;
    
    vis1  = layer_vis(:,:,layer_id, t_curr);
    vis1  = vis1(:);    
    vis2  = layer_vis(:,:,layer_id, t_curr+1);
    
else
    
    if isfield(params, 'useAaffine') && ~params.useAaffine
        can_uv = data.layer_uvb{t_curr}(:,:,:,layer_id);
    else        
        can_uv = cat(4, data.aff_uvb{t_curr}(:,:,:,layer_id),...
            data.layer_uvb{t_curr}(:,:,:,layer_id));
    end;
    aff_uv = data.aff_uvb{t_curr}(:,:,:,layer_id);

    if params.flowFusionFeature
        % add candidate flow fields from feature matching
        can_uv = cat(4, can_uv, data.feature_uvb{t_curr});
    end

    [matchCost, OOB tmpIndex] = compute_data_match_cost_oob(can_uv,...
        data.images(:,:,t_curr+1:-1:t_curr), params);
    
    if layer_id < nLayers
        sup1 = solution.layer_support(:,:,layer_id, t_curr+1);
        sup1 = sup1(:);
        sup2 = solution.layer_support(:,:,layer_id, t_curr);
    end;
    
    vis1  = layer_vis(:,:,layer_id, t_curr+1);
    vis1  = vis1(:);
    vis2  = layer_vis(:,:,layer_id, t_curr);

end;

[H W nF nCan]   = size(can_uv);
nStates         = nCan;


matchCost(OOB==1) = params.lambdaOcc;
% deviation from the affine flow field of the layer
dev = zeros(H,W,nCan);
for i=1:nCan
    tmp = can_uv(:,:,:,i) - aff_uv;
    tmp = evaluate(params.rho_affine, tmp);
    dev(:,:,i) = sum(tmp, 3)*params.lambdaAff;
end;

% spatial term
vert = im2col((reshape(1:H*W,H,W)),[2 1])';
hor  = im2col((reshape(1:H*W,H,W)),[1 2])';

if strcmp(params.flowPrior, 'mrfOnDev')
    can_dev_uv   = can_uv - repmat(aff_uv, [1 1 1 size(can_uv,4)]);
    spatial_func = @(u) compute_pw_spatial_flow_potential(u,...
        can_dev_uv, params);
else
    spatial_func = @(u) compute_pw_spatial_flow_potential(u,...
        can_uv, params);
end;

fp      = ones(H*W,1);
success = 0;
current_energy  = 1e32;

% start alpha expansion
while true    
    for iState=0:nStates-1
        
        current_proposal = iState *ones(size(fp));

        pairwise_terms   = generateEnergyTable_matlab([vert; hor],...
            spatial_func, current_proposal(1:H*W), fp(1:H*W));       
        
        % singleton data term        
        ind = (1:H*W)';         
        % mathcing cost        
        %   current proposal
        currInd   = ind + current_proposal*H*W;        
        curr_cost = matchCost(currInd);
        currNbInd = tmpIndex(currInd);
        currVisb2 = vis2(currNbInd);
        %   occluded
%         curr_cost(currVisb2~=vis1) = params.lambdaOcc;
        curr_cost(currVisb2==0) = params.lambdaOcc; 
        %   invisible        
        curr_cost(vis1==0) = 0; 
        
        %   previous proposal
        prevInd   = ind + fp*H*W;
        prev_cost = matchCost(prevInd);
        prevNbInd = tmpIndex(prevInd);
        prevVisb2 = vis2(prevNbInd);
        %   occluded
%         prev_cost(prevVisb2~=vis1) = params.lambdaOcc;
        prev_cost(prevVisb2==0) = params.lambdaOcc;
        %   invisible        
        prev_cost(vis1==0) = 0; 
       

        % deviation from mean affine flow fields
        curr_dev = dev(currInd);
        prev_dev = dev(prevInd);        

        cost0 = curr_cost + curr_dev;
        cost1 = prev_cost + prev_dev;

        % temporal consistency of layer support functions
        if layer_id < nLayers            
            currSup2 = sup2(currNbInd);
            prevSup2 = sup2(prevNbInd);            
        
            cost0 = cost0 + params.lambdaIncon*(sup1~=currSup2);
            cost1 = cost1 + params.lambdaIncon*(sup1~=prevSup2);
        end;
        
        data_unary = [ind'; cost0'; cost1'];
        
        [new_fp,new_energy] = QPBO_all_in_one_prealloc('p', H*W,...
            data_unary, pairwise_terms);
        
        if (new_energy - current_energy) > .5
            %if we use potentials of type double, it is
            %possible that the new energy grows, because of numerical
            %issues, see QPBO documentation
            
            %disp(strcat('WARNING: energy grows, ...
            % (new_energy - current_energy) >'...
            %    ,num2str(new_energy - current_energy)));
        elseif (new_energy-current_energy) < -1
            success = 1;
            current_energy = new_energy;
            
            %generate new labeling if it has lower energy
            fp = (1-new_fp).* current_proposal+ new_fp .* fp;
        end        
        %fprintf('%3.3f ', current_energy)
        % tmpSeg = reshape(new_fp, [H W]);
        % % [max(tmpSeg(:)) min(tmpSeg(:)) sum(isnan(tmpIm(:)))]
        % % tmpIm  = seg2color(tmpSeg);
        % fn     = [params.fdir sprintf(...
        % 'flowfusion_move_t%d_%d_%d_display%04d.png',...
        %     t_curr, isForward, layer_id, iDisplay)];
        % % imwrite(uint8(tmpIm), fn);
        % imwrite(uint8(tmpSeg*255), fn);
        % iDisplay = iDisplay+1;
    end
    %fprintf('\n');    
    if success == 0
        %disp('finished, energy did not shrink');
        break;
    end
    success = 0;
end;
% parse collapsed states to individual variables
fp = reshape(fp, [H W]);
uv = composite_final_flow_seg(can_uv, fp+1);