function [data solution params] = prune_layers_asym(data, solution, params)
%% remove layers that have visible pixels fewer than a threshold


% perform a median filtering to support function to remove speckles
layer_support = solution.layer_support;

% median filtering
for t = 1:size(layer_support,4)
    for iL = 1:size(layer_support,3)
        tmp = layer_support(:,:,iL,t);
        tmp = medfilt2(tmp, [5 5]);
        layer_support(:,:,iL,t) = tmp;
    end;
end;

% weighted median filtering
% sz = size(layer_support);
% sz = sz(1:2);
% occ = ones(sz);
% sigma_i = 7;
% for t = 1:size(layer_support,4)
%     for iL = 1:size(layer_support,3)
%         tmp = repmat(layer_support(:,:,iL,t), [1 1 2]);
%         im  = data.labImages{t};
%         tmp = denoise_color_weighted_medfilt2(tmp, im, occ, 7, [5 5], sigma_i);
%         layer_support(:,:,iL,t) = tmp(:,:,1);
%     end;
% end;

layer_seg   = support2seg(layer_support);
nLayers     = size(solution.layer_support, 3)+1;

% check/record layers to remove
remain_ids  = [];    
delete_ids  = [];    
is_remove   = false;
for iL=1: nLayers    
    if sum(layer_seg(:)==iL) > params.nIgnore
        remain_ids = [remain_ids iL];
    else
        is_remove = true;
        delete_ids = [delete_ids iL];
    end;  
  
end;



% remove layers, support functions and flow fields
if is_remove
        
    nRemain = length(remain_ids);    
    
    solution.layer_support = ...
        solution.layer_support(:,:,remain_ids(1:end-1),:);

    if params.PruneToClosest
        % Assign removed pixels to the neaest layer measured by flow distance
        for iL = 1:length(delete_ids)
            
            distFlow = zeros(1,nRemain);
            
            for t=1:params.T-1
                mask = layer_seg(:,:,t) == iL;
                
                uv = data.layer_uv{t}(:,:,:, delete_ids(iL));
                
                for i1 = 1:nRemain
                    err = sum(abs(uv-data.layer_uv{t}(:,:,:,remain_ids(i1))), 3);
                    distFlow(i1) = sum(err(mask(:)));
                end;
            end;
            
            [tmp minID] = min(distFlow);
            
            for i=1:minID-1
                for t=1:params.T-1
                    tmp = solution.layer_support(:,:,i,t);
                    tmp(mask) = 0;
                    solution.layer_support(:,:,i,t) = tmp;
                end;
            end;
            
            if minID < nRemain
                for t=1:params.T-1
                    tmp = solution.layer_support(:,:,minID,t);
                    tmp(mask) = 1;
                    solution.layer_support(:,:,minID,t) = tmp;
                end;
            end;
            
        end;
    end;
    
    for t=1:params.T-1
        %forward flow field
        data.aff_uv{t}   = data.aff_uv{t}(...
            :,:,:,remain_ids);
        data.layer_uv{t} = data.layer_uv{t}(...
            :,:,:,remain_ids);
        
        if params.is_symmetric
            %backward flow field
            data.aff_uvb{t}   = data.aff_uvb{t}(...
                :,:,:,remain_ids);
            data.layer_uvb{t} = data.layer_uvb{t}(...
                :,:,:,remain_ids);
        end;
    end



    
    
    params.nLayers = nRemain;
end;


