function solution = segment_layers_change_support_gc_si_asym(...
    solution, params, data, states)
%% 
% Given with-layer flow field, assign pixels to different layers
%   for each layer, estimate its support function with the support functions
%   for other layers fixed, it is a binary problem
%
%   use alpha expansion to solve sequential Ising model for layer
%   segmentation
%
%   if there are only two layers and one support function, it is a binary
%   problem

%   input STATES tells which layer support to change to

[H W nF nLayers] = size(data.layer_uv{1});

if nargin<4
    states = 1:nLayers-1;
end;

% no (or fixed constant 1) support for last layer, no need to change
states = sort(states, 'ascend'); 
if max(states) == nLayers
    states = states(1:end-1);
end;

nStates    = length(states);

% construct pairwise spatial term
% vert = im2col((reshape(1:H*W,H,W)),[2 1])';
% hor  = im2col((reshape(1:H*W,H,W)),[1 2])';
% spaCliques = [vert; hor];
spaCliques = data.spaCliquesSup;

% construct pairwise temporal data term
indexInc = params.startIndex-1;
for t=1:params.T-1    
    temporal{t} = [ repmat([1:H*W]', [nLayers, 1]) + indexInc(t)...
        data.tempNeighbors{t}(:)+indexInc(t+1)];
end;

if params.is_symmetric
    for t=1:params.T-1
        temporal_b{t} = [ repmat([1:H*W]', [nLayers, 1]) + indexInc(t+1)...
            data.tempNeighbors_b{t}(:)+indexInc(t)];
    end;
end;

%%%%%%%%%%%%%%% alpha expansion
% no data unary term
fp              = ones(params.T*H*W,1);
data_unary      = [1:params.T*H*W; zeros(2, params.T*H*W)];


% the change in state is updated to the prev/curr_layer_support
current_proposal = zeros(params.T*H*W,1);

% current_energy  = 1e32;
success         = 0;

% start alpha expansion
prev_layer_support = solution.layer_support; 

% isRedundant = true; % true for debug
isRedundant = false;

%% for checking energy
% solution2   = solution;
% params2     = params;
% 
% params2.lambdaAff = 0;
% params2.lambdaFlow = 0;


%while true       
for iterMove =1:params.maxMoveIters
    
    for iState=1:nStates        

        curr_layer_support = prev_layer_support;        
        curr_layer_support(:,:,states(iState), :)  =...
            1- prev_layer_support(:,:,states(iState), :);
        
%         curr_layer_support(:,:,states(iState), :)   = 0;
%         prev_layer_support(:,:,states(iState), :)   = 1;
        
        % pairwise spatial term        
        spas  = [];
        temps = [];
        
        for t=1:params.T
            if params.isMex
                if isRedundant
                    spatial_func = @(u)compute_pw_spatial_potential_si_mex(...
                        u, data.weight{t}, params.lambdaSeg,...
                        prev_layer_support(:,:,:,t),...
                        curr_layer_support(:,:,:,t));
                else
                    spatial_func = @(u)compute_pw_spatial_potential_si_mex(...
                        u, data.weight{t}, params.lambdaSeg,...
                        prev_layer_support(:,:,:,t),...
                        curr_layer_support(:,:,:,t), states(iState));
                end;
                
                spa = generateEnergyTable(spaCliques, spatial_func,...
                    current_proposal((t-1)*H*W+1:t*H*W), fp((t-1)*H*W+1:t*H*W));
            else
                if isRedundant
                    spatial_func = @(u) compute_pw_spatial_potential_si(...
                        u, data.weight{t}, params,...
                        prev_layer_support(:,:,:,t),...
                        curr_layer_support(:,:,:,t));
                else
                    spatial_func = @(u) compute_pw_spatial_potential_si(...
                        u, data.weight{t}, params,...
                        prev_layer_support(:,:,:,t),...
                        curr_layer_support(:,:,:,t), states(iState));
                end;
                spa = generateEnergyTable_matlab(spaCliques, spatial_func,...
                    current_proposal((t-1)*H*W+1:t*H*W), fp((t-1)*H*W+1:t*H*W));
            end;
            
            
            spa(1:2,:)  = spa(1:2,:) + H*W*(t-1);
            spas        = [spas spa];
            
            if t < params.T
                
                if params.isMex
                    temp_fun    = @(u) compute_pw_t_potential_si_asym_mex(u, ...
                        data.matchCost{t}, params.lambdaOcc,...
                        params.lambdaIncon, data.OOB{t}, ...
                        prev_layer_support(:,:,:,t:t+1),....
                        curr_layer_support(:,:,:,t:t+1), t);
                    temp    = generateEnergyTable(temporal{t}, temp_fun,...
                        current_proposal, fp);
                else
                    temp_fun    = @(u) compute_pw_t_potential_si_asym(u, ...
                        data.matchCost{t}, params, data.OOB{t}, ...
                        prev_layer_support(:,:,:,t:t+1),....
                        curr_layer_support(:,:,:,t:t+1), t);
                    
                    temp    = generateEnergyTable_matlab(temporal{t}, temp_fun,...
                        current_proposal, fp);
                end;
                
                
                if ~isRedundant
                    %%% only keep terms that are in effect
                    notInEffect = (temp(3,:) == temp(4,:)) &...
                        (temp(3,:) == temp(5,:)) &  (temp(3,:) == temp(6,:));
                    indR = find(~notInEffect);
                    temp = temp(:, indR);
                end;
                
                temps   = [temps temp];
                
                if params.is_symmetric
                    if params.isMex
                        temp_fun  = @(u) compute_pw_t_potential_si_asym_mex(u, ...
                            data.matchCost_b{t}, params.lambdaOcc,...
                            params.lambdaIncon, data.OOB_b{t}, ...
                            prev_layer_support(:,:,:,t:t+1),....
                            curr_layer_support(:,:,:,t:t+1), t, false);
                        temp    = generateEnergyTable(temporal_b{t}, temp_fun,...
                            current_proposal, fp);
                    else
                        temp_fun  = @(u) compute_pw_t_potential_si_asym(u, ...
                            data.matchCost_b{t}, params, data.OOB_b{t}, ...
                            prev_layer_support(:,:,:,t:t+1),....
                            curr_layer_support(:,:,:,t:t+1), t, false);
                        temp    = generateEnergyTable_matlab(temporal_b{t}, temp_fun,...
                            current_proposal, fp);
                    end;
                    
                    if ~isRedundant
                        %%% only keep terms that are in effect
                        notInEffect = (temp(3,:) == temp(4,:)) &...
                            (temp(3,:) == temp(5,:)) &  (temp(3,:) == temp(6,:));
                        indR = find(~notInEffect);
                        temp = temp(:, indR);
                    end
                    
                    temps   = [temps temp];
                end;
                
            end;
            
        end;
        
        pairwise_terms  = [spas temps];
clear spas  temps;                

% for debug
% new_fp=prev_layer_support(:,:,states(iState), :);
% new_fp = new_fp(:);
% en = evaluate_energy_gc_potential_table(data_unary, spas, new_fp);
% fprintf('sup smoothness %3.3e \n',en);
% new_fp = fp;
% en = evaluate_energy_gc_potential_table(data_unary, temps, new_fp);
% fprintf('time %3.3e \n',en);
% end of for debug

% for debug
% solution2.layer_support = prev_layer_support;
% en = evaluate_overall_energy_si_asym(solution2, data, params2);
% fprintf('(%3.5e)\t ', en);

prevEn = evaluate_energy_gc_potential_table(data_unary, pairwise_terms, fp(:));

if iterMove == 1
    pEn(iState) = prevEn;    
end;

        [new_fp,new_energy] = QPBO_all_in_one(H*W*params.T,...
            data_unary, pairwise_terms);        
%currEn = evaluate_energy_gc_potential_table(data_unary, pairwise_terms, new_fp(:));

currEn = new_energy/2;
if (iterMove == 1) && params.display
% if params.display
    fprintf('(%3.5e, %3.5e) ', prevEn, currEn);
end;

        % accept new proposal if energy goes down        
        if currEn < prevEn-1            
            new_fp = reshape(new_fp, [H W 1 params.T]);
            prev_layer_support(:,:,states(iState), :) =...
                new_fp.*prev_layer_support(:,:,states(iState), :) +...
                (1-new_fp).*curr_layer_support(:,:,states(iState), :);
            
            if (iterMove > 1) && params.display
                fprintf('(%3.5e, %3.5e) ', prevEn, currEn);
            end;
            success = 1;
        end;
    end
    
if params.display    
    fprintf('\n');
end

    
    % reach local minimum
    if (success == 0) 
        break;
    end
    success = 0;    
   
end;

solution.layer_support = prev_layer_support;

% fprintf('alpha expansion finished, energy %3.3e \n', current_energy);
% for testing energy function evaluation
% new_fp = fp;
% en = evaluate_energy_gc_potential_table(data_unary, pairwise_terms, new_fp(:));
% fprintf('\nenergy from potential table (no flow term) %3.3e',en);
% fprintf('\n');
%%
% new_fp=prev_layer_support(:,:,states(iState), :);
% new_fp = new_fp(:);
% 
% en = evaluate_energy_gc_potential_table(data_unary, spas, new_fp);
% fprintf('sup smoothness %3.3e \n',en);
% 
% en = evaluate_energy_gc_potential_table(data_unary, temps, new_fp);
% fprintf('time %3.3e \n',en);