function solution = segment_layers_gc_si_asym(solution, params, data, states)
%% Given with-layer flow field, assign pixels to different layers

% use alpha expansion to solve sequential Ising model for layer
% segmentation

[H W nF nLayers] = size(data.aff_uv{1});

if nargin<4
    states = 1:nLayers;
end;
nStates    = length(states);

% construct pairwise spatial term
vert = im2col((reshape(1:H*W,H,W)),[2 1])';
hor  = im2col((reshape(1:H*W,H,W)),[1 2])';

% construct pairwise temporal data term
indexInc = params.startIndex-1;
for t=1:params.T-1    
    temporal{t} = [ repmat([1:H*W]', [nLayers, 1]) + indexInc(t)...
        data.tempNeighbors{t}(:)+indexInc(t+1)];
end;

if params.is_symmetric
    for t=1:params.T-1
        temporal_b{t} = [ repmat([1:H*W]', [nLayers, 1]) + indexInc(t+1)...
            data.tempNeighbors_b{t}(:)+indexInc(t)];
    end;
end;

%%%%%%%%%%%%%%% alpha expansion
% no data unary term
data_unary      = [1:params.T*H*W; zeros(2, params.T*H*W)];
fp              = ones(params.T*H*W,1);

current_energy  = 1e32;
success         = 0;

% start alpha expansion

% the change in state is updated to the prev/curr_layer_support
current_proposal = zeros(params.T*H*W,1);

% spaCliques = [vert; hor];
spaCliques = data.spaCliquesSup;

% spa        = [spaCliques'; spaCliques'; spaCliques'];
% temp       = [temporal_b{t}; temporal_b{t}; temporal_b{t}];

% [tH tW]    = size(spa);
% preallocate memory
% spas  = zeros(tH, tW*params.T);
% temps = zeros(tH, tW);
% pairwise_terms = [];

% iDisplay = 1;

prev_layer_support = solution.layer_support;

% while true       
for iterMove =1:params.maxMoveIters
    
   for iState=1:nStates        
       
       % whether a pixel switches to become visible at iState layer
% clear curr_layer_support;
       curr_layer_support = change_support(states(iState),...
            prev_layer_support);        
        
        % pairwise spatial term        
clear spas  temps;
        spas  = [];
        temps = [];
        
        for t=1:params.T
clear spa;            
            if params.isMex                
                spatial_func = @(u) compute_pw_spatial_potential_si_mex(...
                    u, data.weight{t}, params.lambdaSeg,...
                    prev_layer_support(:,:,:,t),...
                    curr_layer_support(:,:,:,t), states(iState));
                spa = generateEnergyTable(spaCliques, spatial_func,...
                    current_proposal((t-1)*H*W+1:t*H*W), fp((t-1)*H*W+1:t*H*W));
                
            else
                spatial_func = @(u) compute_pw_spatial_potential_si(...
                    u, data.weight{t}, params,...
                    prev_layer_support(:,:,:,t),...
                    curr_layer_support(:,:,:,t), states(iState));                
                spa = generateEnergyTable_matlab(spaCliques, spatial_func,...
                    current_proposal((t-1)*H*W+1:t*H*W), fp((t-1)*H*W+1:t*H*W));                
            end;
           
            spa(1:2,:)  = spa(1:2,:) + H*W*(t-1);
            spas        = [spas spa];
            
            % data matching cost, modulated by visiblity states
            if t < params.T
clear temp;                
                if params.isMex
                    temp_fun    = @(u) compute_pw_t_potential_si_asym_mex(u, ...
                        data.matchCost{t}, params.lambdaOcc,...
                        params.lambdaIncon, data.OOB{t}, ...
                        prev_layer_support(:,:,:,t:t+1),....
                        curr_layer_support(:,:,:,t:t+1), t);
                    temp    = generateEnergyTable(temporal{t}, temp_fun,...
                        current_proposal, fp);
                else
                    temp_fun    = @(u) compute_pw_t_potential_si_asym(u, ...
                        data.matchCost{t}, params, data.OOB{t}, ...
                        prev_layer_support(:,:,:,t:t+1),....
                        curr_layer_support(:,:,:,t:t+1), t);
                    temp    = generateEnergyTable_matlab(temporal{t}, ...
                        temp_fun, current_proposal, fp);
                end;
               
                %%% only keep terms that are in effect                        
                notInEffect = (temp(3,:) == temp(4,:)) &...
                    (temp(3,:) == temp(5,:)) &  (temp(3,:) == temp(6,:));                                      
                indR = find(~notInEffect);
                temp = temp(:, indR);
                temps   = [temps temp];


                if params.is_symmetric
clear temp;                    
                    if params.isMex
                        
                        temp_fun  = @(u) compute_pw_t_potential_si_asym_mex(u, ...
                            data.matchCost_b{t}, params.lambdaOcc,...
                            params.lambdaIncon, data.OOB_b{t}, ...
                            prev_layer_support(:,:,:,t:t+1),....
                            curr_layer_support(:,:,:,t:t+1), t, false);
                        temp    = generateEnergyTable(temporal_b{t}, ...
                            temp_fun, current_proposal, fp);
                    else
                        
                        temp_fun  = @(u) compute_pw_t_potential_si_asym(u, ...
                            data.matchCost_b{t}, params, data.OOB_b{t}, ...
                            prev_layer_support(:,:,:,t:t+1),....
                            curr_layer_support(:,:,:,t:t+1), t, false);
                        temp    = generateEnergyTable_matlab(temporal_b{t}, ...
                            temp_fun, current_proposal, fp);                        
                    end;
                    
                    notInEffect = (temp(3,:) == temp(4,:)) &...
                        (temp(3,:) == temp(5,:)) &  (temp(3,:) == temp(6,:));
                    indR = find(~notInEffect);
                    temp = temp(:, indR);
                    
                    temps   = [temps temp];
                end;
                
            end;
            
        end;
        
        pairwise_terms  = [spas temps];
clear spas  temps;        

% if iterMove == 1
%     prevEn(iState) = evaluate_energy_gc_potential_table(data_unary, pairwise_terms, fp(:));    
% end;

prevEn = evaluate_energy_gc_potential_table(data_unary, pairwise_terms, fp(:));    

        [new_fp,new_energy] = QPBO_all_in_one(H*W*params.T,...
            data_unary, pairwise_terms);
currEn = new_energy/2;        
%currEn = evaluate_energy_gc_potential_table(data_unary, pairwise_terms, new_fp(:));

if (iterMove == 1) && params.display
    fprintf('(%3.5e, %3.5e) ', prevEn, currEn);
end;

        
        % accept new proposal if energy goes down        
        if currEn < prevEn-1; %prevEn(iState)-1            
            prev_layer_support = update_layer_support(...
                prev_layer_support, curr_layer_support, new_fp);
            success = 1;            
            if (iterMove > 1) && params.display
                fprintf('(%3.5e, %3.5e) ', prevEn, currEn);
            end;
        end;
                
    end
    
    if success == 0
%         disp('finished, energy did not shrink');
        break;
    end
    success = 0;
end;

solution.layer_support = prev_layer_support;
