function [solution data] = segment_layers_select_flow_gc_si_asym(...
           solution, params, data, states)
%% 
% A baisc move includes flip the visible pixel from layer A to layer B
%   together with its flow seleciton variable. The corresponding flow
%   selection variable at A will be set to its affine flow field.

% use alpha expansion to solve sequential Ising model for layer
% segmentation

[H W nF nLayers] = size(data.aff_uv{1});

if nargin<4
    states = 1:nLayers;
end;
nStates    = length(states);

tmp         = reshape(1:nLayers, [1 1 nLayers]);
flow_sel    = repmat(tmp, [H W 1])+nLayers;

% put affine flow field first for function change_flow_selection
for t=1:params.T-1
    can_uv{t}        = cat(4, data.aff_uv{t}, data.layer_uv{t});
    prev_flow_sel{t} = flow_sel;
end;

prev_flow_sel{params.T} = flow_sel;

% nCan = size(can_uv{1}, 4);
% compute match Cost & OOB mask, temporal neighbors

% forward flow field    
for t=1:params.T-1    
    [matchCost{t}, OOB{t}, tempNeighborsAll{t}] =...
        compute_data_match_cost_oob(can_uv{t},...
        data.images(:,:,t:t+1), params);
end;

% backward flow field
if params.is_symmetric
    for t=1:params.T-1
        can_uv_b{t} = cat(4, data.aff_uvb{t}, data.layer_uvb{t});
        [matchCost_b{t}, OOB_b{t}, tempNeighborsAll_b{t}] =...
            compute_data_match_cost_oob(can_uv_b{t},...
            data.images(:,:,t+1:-1:t), params);
    end;
end;

% construct pairwise spatial term
vert = im2col((reshape(1:H*W,H,W)),[2 1])';
hor  = im2col((reshape(1:H*W,H,W)),[1 2])';
% spaCliques = [vert; hor];
spaCliques = data.spaCliquesSup;

% construct pairwise temporal data term
indexInc = params.startIndex-1;

%%%%%%%%%%%%%%% alpha expansion

% the change in state is updated to the
% curr_layer_support/solution.flow_sel 
%1: select previous solution
fp         = ones(params.T*H*W,1);  
%0: select current proposal
current_proposal= zeros(params.T*H*W,1);

current_energy  = 1e32;
success         = 0;

% preallocate memory
% spas  = zeros(tH, tW*params.T);
% temps = zeros(tH, tW);
% pairwise_terms = [];

prev_layer_support = solution.layer_support;

% iDisplay = 1;

% start alpha expansion
% while true    
for iterMove =1:params.maxMoveIters    
    for iState= 1:nStates 
        curr_layer_support = prev_layer_support;
        curr_layer_support = change_support(states(iState),...
            prev_layer_support, curr_layer_support);                
        
        layer_seg     = support2seg(prev_layer_support);        
        curr_flow_sel = change_flow_selection(...
            states(iState), prev_flow_sel, layer_seg);        
% TODO: preallocate spas temps        
        % pairwise spatial term
        clear spas temps;        
        spas  = [];
        temps = [];
        
        for t=1:params.T
            
clear spa;
            if params.isMex
                spatial_func = @(u) compute_pw_spatial_potential_si_mex(...
                    u, data.weight{t}, params.lambdaSeg,...
                    prev_layer_support(:,:,:,t),...
                    curr_layer_support(:,:,:,t), states(iState));                
                spa = generateEnergyTable(spaCliques, spatial_func,...
                    current_proposal((t-1)*H*W+1:t*H*W),...
                    fp((t-1)*H*W+1:t*H*W));
                
            else
                spatial_func = @(u) compute_pw_spatial_potential_si(...
                    u, data.weight{t}, params,...
                    prev_layer_support(:,:,:,t),...
                    curr_layer_support(:,:,:,t), states(iState));       
                spa = generateEnergyTable_matlab(spaCliques, spatial_func,...
                    current_proposal((t-1)*H*W+1:t*H*W),...
                    fp((t-1)*H*W+1:t*H*W));
                
            end;                

            % add the index shift for each frame
            spa(1:2,:) = spa(1:2,:) + H*W*(t-1);
            
            if t<params.T
                % forward flow selection smoothness                
clear spa_flow;          
                if strcmp(params.flowPrior, 'mrfOnDev')
                    tmp_can_uv = can_uv{t} - repmat(...
                        data.aff_uv{t}(:,:,:,iState),...
                        [1 1 1 size(can_uv{t},4)]);
                else
                    tmp_can_uv = can_uv{t};
                end;
                
                if params.isMex && params.isFlowL1Norm
                    spatial_func = @(u) compute_pw_flow_spatial_potential_si_mex(...
                        u, tmp_can_uv, params.lambdaFlow,...
                        prev_flow_sel{t}, curr_flow_sel{t});
                    
                    spa_flow = generateEnergyTable(spaCliques, spatial_func,...
                        current_proposal((t-1)*H*W+1:t*H*W), fp((t-1)*H*W+1:t*H*W));
                else
                    spatial_func = @(u) compute_pw_flow_spatial_potential_si(...
                        u, tmp_can_uv, params,...
                        prev_flow_sel{t}, curr_flow_sel{t});
                    spa_flow = generateEnergyTable_matlab(spaCliques, spatial_func,...
                        current_proposal((t-1)*H*W+1:t*H*W), fp((t-1)*H*W+1:t*H*W));
                end;

                % add together
                spa(3:end,:) = spa(3:end,:) + spa_flow(3:end,:);
            end;
            
            if params.is_symmetric && t > 1
                % backward flow selection smoothness                
clear spa_flow;               

                if strcmp(params.flowPrior, 'mrfOnDev')
                    tmp_can_uv = can_uv_b{t-1} - repmat(...
                        data.aff_uvb{t-1}(:,:,:,iState),...
                        [1 1 1 size(can_uv_b{t-1},4)]);
                else
                    tmp_can_uv = can_uv_b{t-1};
                end;
                
                if params.isMex && params.isFlowL1Norm
                    spatial_func = @(u) compute_pw_flow_spatial_potential_si_mex(...
                        u, tmp_can_uv, params.lambdaFlow,...
                        prev_flow_sel{t}, curr_flow_sel{t});
                    spa_flow = generateEnergyTable(spaCliques, spatial_func,...
                        current_proposal((t-1)*H*W+1:t*H*W), fp((t-1)*H*W+1:t*H*W));
                else                    
                    spatial_func = @(u) compute_pw_flow_spatial_potential_si(...
                        u, tmp_can_uv, params,...
                        prev_flow_sel{t}, curr_flow_sel{t});

                    spa_flow = generateEnergyTable_matlab(spaCliques, spatial_func,...
                        current_proposal((t-1)*H*W+1:t*H*W), fp((t-1)*H*W+1:t*H*W));                    
                end;
           


                % add together
                spa(3:end,:) = spa(3:end,:) + spa_flow(3:end,:);
                
            end;            
            

            spas = [spas spa];

            % temporal term from forward flow 
            if t<params.T                
                for state = 0:1
%                    
                    % state 0/1 on temporal cliques: on only when sate of node at
                    % frame t is 0/1
                    ind     = reshape(1:H*W, [H W]);
                    
                    if state==0
                        tmp_ind = repmat(ind, [1 1 nLayers])+...
                            (curr_flow_sel{t}-1)*H*W;
                    else
                        tmp_ind = repmat(ind, [1 1 nLayers]) +...
                            (prev_flow_sel{t}-1)*H*W;
                    end;
                    
                    tmp_matchCost = matchCost{t}(tmp_ind);
                    tmp_OOB       = OOB{t}(tmp_ind);
                    tempNeighbors = tempNeighborsAll{t}(tmp_ind);
                    temporalN =[repmat((1:H*W)', [nLayers, 1])+indexInc(t)...
                        tempNeighbors(:)+indexInc(t+1)];
%                    
clear temp;
                    if params.isMex
                        temp_fun  = @(u)compute_pw_t_potential_state_si_asym_mex(...
                            u, tmp_matchCost, params.lambdaOcc,...
                            params.lambdaIncon, tmp_OOB, ...
                            prev_layer_support(:,:,:,t:t+1),...
                            curr_layer_support(:,:,:,t:t+1), state, t);
                        temp  = generateEnergyTable(temporalN, temp_fun,...
                            current_proposal, fp);
                    else
                        temp_fun  = @(u)compute_pw_t_potential_state_si_asym(u, ...
                            tmp_matchCost, params, tmp_OOB, ...
                            prev_layer_support(:,:,:,t:t+1),...
                            curr_layer_support(:,:,:,t:t+1), state, t);
                        
                        temp  = generateEnergyTable_matlab(temporalN, temp_fun,...
                            current_proposal, fp);                        
                    end;
                    
                    %%% only keep terms that are in effect
                    if state ==0
                        inEffect = temp(3,:) ~= temp(4,:);
                    else
                        inEffect = temp(5,:) ~= temp(6,:);
                    end;
                    
                    indR = find(inEffect);
                    temp = temp(:, indR);

                    temps = [temps temp];
                    
                end;                 
                
                if params.is_symmetric
                    for state = 0:1
                        
                        % state 0/1 on temporal cliques: on only when sate
                        %  of node at frame t is 0/1
                        ind     = reshape(1:H*W, [H W]);
                        
                        if state==0
                            tmp_ind = repmat(ind, [1 1 nLayers])+...
                                (curr_flow_sel{t+1}-1)*H*W;
                        else
                            tmp_ind = repmat(ind, [1 1 nLayers]) +...
                                (prev_flow_sel{t+1}-1)*H*W;
                        end;
                        
                        tmp_matchCost = matchCost_b{t}(tmp_ind);
                        tmp_OOB       = OOB_b{t}(tmp_ind);
                        tempNeighbors = tempNeighborsAll_b{t}(tmp_ind);
                        temporalN =[repmat((1:H*W)', [nLayers, 1])+indexInc(t+1)...
                            tempNeighbors(:)+indexInc(t)];
                        
clear temp;                                     
                        if params.isMex
                            temp_fun  = @(u)compute_pw_t_potential_state_si_asym_mex(...
                                u, tmp_matchCost, params.lambdaOcc,...
                                params.lambdaIncon, tmp_OOB, ...
                                prev_layer_support(:,:,:,t:t+1),...
                                curr_layer_support(:,:,:,t:t+1), state, t, false);                            
                            temp  = generateEnergyTable(temporalN, temp_fun,...
                                current_proposal, fp);
                        else
                            temp_fun  = @(u)compute_pw_t_potential_state_si_asym(u, ...
                                tmp_matchCost, params, tmp_OOB, ...
                                prev_layer_support(:,:,:,t:t+1),...
                                curr_layer_support(:,:,:,t:t+1), state, t, false);                                                        
                            temp  = generateEnergyTable_matlab(temporalN, temp_fun,...
                                current_proposal, fp);
                        end;
           
                        
                        %%% only keep terms that are in effect                        
                        % remove useless temporal terms: support functions
                        % doesn't change in either case
                        % state 0 row 3 (00) = row 4 (01)
                        % state 1 row 5 (10) = row 6 (11)
                    
                        if state ==0
                            inEffect = temp(3,:) ~= temp(4,:);
                        else
                            inEffect = temp(5,:) ~= temp(6,:);
                        end;
                        
                        indR = find(inEffect);
                        temp = temp(:, indR);
                    
                        temps = [temps temp];
                    end;
                end;
            end;           
            
        end;        
        
        % better than [spa1 temp1 spa2 temp2], TODO: examine reasons
        pairwise_terms  = [spas temps]; 
        clear temps spas
        % data term: deviation of flow field from the affine flow field of
        % each layer

        dev0= zeros(H,W,params.T);
        dev1= dev0;
        
        % forward flow field
        for t=1:params.T-1;
            for iL=1:nLayers
                
                uv0 = composite_final_flow_seg(...
                    can_uv{t}, curr_flow_sel{t}(:,:,iL));
                dev0(:,:,t) = dev0(:,:,t)+sum(evaluate(params.rho_affine,...
                    uv0-data.aff_uv{t}(:,:,:,iL) ), 3);
                
                uv1 = composite_final_flow_seg(...
                    can_uv{t}, prev_flow_sel{t}(:,:,iL));
                dev1(:,:,t) = dev1(:,:,t)+sum(evaluate(params.rho_affine,...
                    uv1-data.aff_uv{t}(:,:,:,iL) ), 3);
                
            end;
        end;
        
        % backward flow field
        if params.is_symmetric
            for t=2:params.T;
                for iL=1:nLayers
                    
                    uv0 = composite_final_flow_seg(...
                        can_uv_b{t-1}, curr_flow_sel{t}(:,:,iL));
                    dev0(:,:,t) = dev0(:,:,t)+sum(evaluate(params.rho_affine,...
                        uv0-data.aff_uvb{t-1}(:,:,:,iL) ), 3);
                    
                    uv1 = composite_final_flow_seg(...
                        can_uv_b{t-1}, prev_flow_sel{t}(:,:,iL));
                    dev1(:,:,t) = dev1(:,:,t)+sum(evaluate(params.rho_affine,...
                        uv1-data.aff_uvb{t-1}(:,:,:,iL) ), 3);
                    
                end;
            end;
        
        end;
        
        dev0 = dev0*params.lambdaAff;
        dev1 = dev1*params.lambdaAff;
        
        data_unary = [1:params.T*H*W; dev0(:)'; dev1(:)'];
        

% if iterMove == 1
%     prevEn(iState) = evaluate_energy_gc_potential_table(data_unary, pairwise_terms, fp(:));    
% end;

pEn = evaluate_energy_gc_potential_table(data_unary, pairwise_terms, fp(:));    

        [new_fp,new_energy] = QPBO_all_in_one(H*W*params.T,...
            data_unary, pairwise_terms);              
        
% currEn = evaluate_energy_gc_potential_table(data_unary, pairwise_terms, new_fp(:));
currEn = new_energy/2;        
if params.display
%     fprintf('(%3.5e, %3.5e, %3.5e, %3.5e %d %d)\n', pEn, prevEn(iState), currEn, ...
%         new_energy/2-currEn, pEn-1 > currEn, prevEn(iState)-1 > currEn);
    fprintf('(%3.5e, %3.5e %d)\n', pEn, currEn, ...
        pEn-1 > currEn);
end;


        % accept new proposal if energy goes down        
        if currEn < pEn-1            
            %generate new support function and flow selection
            % 0: current proposal; 1: previous proposal
            
            prev_layer_support = update_layer_support(prev_layer_support,...
                curr_layer_support, new_fp);
            prev_flow_sel      = update_flow_selection(prev_flow_sel,...
                curr_flow_sel, new_fp );
            
            success = 1;                        
        end;

% if (iterMove == 1) && params.display
%     fprintf('(%3.5e, %3.5e) ', prevEn(iState), currEn);
% end;

%         % accept new proposal if energy goes down        
%         if currEn < prevEn(iState)-1            
%             %generate new support function and flow selection
%             % 0: current proposal; 1: previous proposal
%             
%             prev_layer_support = update_layer_support(prev_layer_support,...
%                 curr_layer_support, new_fp);
%             prev_flow_sel      = update_flow_selection(prev_flow_sel,...
%                 curr_flow_sel, new_fp );
%             
%             success = 1;            
% %             if (iterMove > 1) && params.display
% %                 fprintf('(%3.5e, %3.5e) ', prevEn(iState), currEn);
% %             end;
%             prevEn(iState) = currEn;
%         end;
        
   

% if (new_energy - current_energy) > .5
%     %if we use potentials of type double, it is
%     %possible that the new energy grows, because of numerical
%     %issues, see QPBO documentation
% 
%     %disp(strcat('WARNING: energy grows, (new_energy - current_energy) >'...
%     %    ,num2str(new_energy - current_energy)));
% elseif (new_energy - current_energy) < -1
%     success = 1;
%     current_energy=new_energy;
% end

    end
if params.display    
    fprintf('\n');
end
    
    if success == 0
        break;
    end
    success = 0;
end;
% fprintf('%3.3e \n', current_energy);

% composite new layer_flow and update matchCost
clear data_unary pairwise_terms



% forward
for t=1:params.T-1;
    for iL=1:nLayers
        data.layer_uv{t}(:,:,:,iL) = composite_final_flow_seg(...
            can_uv{t}, prev_flow_sel{t}(:,:,iL));
    end;
end;

% backward
if params.is_symmetric
    for t=1:params.T-1;
        for iL=1:nLayers
            data.layer_uvb{t}(:,:,:,iL) = composite_final_flow_seg(...
                can_uv_b{t}, prev_flow_sel{t+1}(:,:,iL));
        end;
    end;
end;
data.flow_sel = prev_flow_sel;
solution.layer_support = prev_layer_support ;

% evaluate energy according to potential tables
% for testing energy function evaluation
%new_fp = fp;
% en = evaluate_energy_gc_potential_table(data_unary, pairwise_terms, new_fp);
% fprintf('energy from potential table %3.3e \n', en);