function out = compute_pw_spatial_potential_si(vals, ...
               weight, params, prevP, currP, nSelected)
%% 
% sequential Ising model for the binary layer support, the input states,
% vals(3:4,:), select which group of layer support (3D array) to use 
% 0: current proposal; 1: previous proposal

% test code (for code optimization)
%
% params.lambdaSeg    =1;
% H = 500; W = 500; nP = 10; T = 2;
% prevP = round(rand(H, W, nP));
% currP = round(rand(H, W, nP));
% vals = round(rand(4, H*W*2))+1;
% vals(1:2,:) = round(rand(2, H*W*2)*(H*W-2))+1;
% vals(3:4, :) = double(vals(3:4, :) ==1);
% weight = rand(1, H*W*2);
% % 
% out = compute_pw_spatial_potential_si_mex(vals, weight, 1, prevP, currP);
% out2 = compute_pw_spatial_potential_si(vals, weight, params, prevP, currP);
%

lambdaSeg   = params.lambdaSeg;

[H W n]     = size(prevP);

%%%%%%%%%%%%%%%%%%%%%
if exist('nSelected', 'var')
    n = min(n, nSelected);
end;
%%%%%%%%%%%%%%%%%%%%%

% convert to states of each layer
% 0: curr proposal used for each layer
% 1: prev proposal used for each layer

out  = zeros(size(vals(3,:)));

ind1 = vals(1,:);
ind2 = vals(2,:);

% specifiy n by input can reduce computation by half in total
% prevP and currP are the same for layers behind n
for i=1:n 
    
    tmp_ind1 = (i-1)*H*W + ind1;
    tmp_ind2 = (i-1)*H*W + ind2;

%     p1 = prevP(tmp_ind1).*vals(3,:) + currP(tmp_ind1).*(1-vals(3,:));
%     p2 = prevP(tmp_ind2).*vals(4,:) + currP(tmp_ind2).*(1-vals(4,:));

    p1 = currP(tmp_ind1);
    pP = prevP(tmp_ind1);
%     p1(vals(3,:)==1) = pP(vals(3,:)==1) ;
    p1 = pP.*vals(3,:) + p1.*(1-vals(3,:));
    
    p2 = currP(tmp_ind2);
    pP = prevP(tmp_ind2);
%     p2(vals(4,:)==1) = pP(vals(4,:)==1) ;
    p2 = pP.*vals(4,:) + p2.*(1-vals(4,:));
    
    out = out + double(p1~=p2);
end;    

% modulated by color weight 
out = out.*weight*lambdaSeg;