function out = compute_pw_t_potential_si(vals, ...
                matchCost, params, OOB, prevP, currP)
%%
% Compute the temporal pairwise term for layer segmentation with the
% sequential Ising model. the input states, vals(3:4,:), select which group
% of layer support (3D array) to use  

% 0: curr proposal used for each layer
% 1: prev proposal used for each layer

%% for debug
% figure; imagesc([prevP(:,:,1,1) prevP(:,:,2,1) ]);
% figure; imagesc([currP(:,:,1,1) currP(:,:,2,1) ]);
% 
% %%
% figure; imagesc([prevV(:,:,1,1) prevV(:,:,2,1) prevV(:,:,3,1) ]);
% figure; imagesc([currV(:,:,1,1) currV(:,:,2,1) currV(:,:,3,1) ]);
% 
% %%
% figure; imagesc([prevSeg(:,:,1) prevSeg(:,:,2)]);
% figure; imagesc([currSeg(:,:,1) currSeg(:,:,2)]);
% 
% %%
% seg1 = prevSeg(vals(1,:));
% tmp = reshape(seg1(1:4:end), [H W nLayers]);
% figure; imagesc([tmp(:,:,1) tmp(:,:,2) tmp(:,:, 3) ]);
% %%
% tmp = reshape(seg2(1:4:end), [H W nLayers]);
% figure; imagesc([tmp(:,:,1) tmp(:,:,2) tmp(:,:, 3) ]);
%%

[H W nP T] = size(prevP);
% n Layers have n-1 support functions
nLayers    = nP+1;

HP    = size(vals,2)/nLayers;

prevV = support2visibility(prevP);
currV = support2visibility(currP);

% prevSeg = support2seg(prevP);
% currSeg = support2seg(currP);

ind1 = vals(1,:);

if (ind1(1,1)>H*W)
    % from t+1 to t data term (2 image pair symmetric flow case)
    ind1 = ind1-H*W;
    % for indexing visibility masks
    indV1 = ind1 + nLayers*H*W; 
    % for indexing support
    indP1 = ind1(1:HP*(nLayers-1)) + (nLayers-1)*H*W; 
else
    indV1 = ind1;
    indP1 = ind1(1:HP*(nLayers-1));
end;

ind2 = vals(2,:);
if (ind2(1,1)>H*W)
    % from t to t+1 data term
    ind2 = ind2-H*W;            
    % for indexing visibility masks
    indV2 = ind2 + nLayers*H*W; 
    % for indexing support
    indP2 = ind2(1:HP*(nLayers-1)) + (nLayers-1)*H*W; 
else
    indV2 = ind2;
    indP2 = ind2(1:HP*(nLayers-1));
end;

% obtain layer IDs for each state
% seg1 = prevSeg(vals(1,:)).*vals(3,:) + ...
%        currSeg(vals(1,:)).*(1-vals(3,:));
%    
% seg2 = prevSeg(vals(2,:)).*vals(4,:) + ...
%        currSeg(vals(2,:)).*(1-vals(4,:));
layerID1 = repmat([1:nLayers], [H*W*4 1]);
layerID1 = layerID1(:)';
layerID2 = layerID1;
  
% data matching cost
tmp_ind  = ind1 + (layerID1-1)*H*W;  
out      = matchCost(tmp_ind);
oob      = OOB(tmp_ind);

% out of boundary
out(oob==1) = params.lambdaOcc;

tmp_ind = indV2 + (layerID2-1)*H*W;  
vis2    = prevV(tmp_ind).*vals(4,:) + ...
          currV(tmp_ind).*(1-vals(4,:));
            
% invisible at other frame
out(vis2==0) = params.lambdaOcc;

% invisible at current frame
tmp_ind = indV1 + (layerID1-1)*H*W;  
vis1    = prevV(tmp_ind).*vals(3,:) + ...
          currV(tmp_ind).*(1-vals(3,:));

out(vis1==0) = 0;      
%
%
% tmp = reshape(vis1(1:4:end), [H W nLayers]);
% figure; imagesc([tmp(:,:,1) tmp(:,:,2) tmp(:,:, 3) ]);
% 
% tmp = reshape(vis1(4:4:end), [H W nLayers]);
% figure; imagesc([tmp(:,:,1) tmp(:,:,2) tmp(:,:, 3) ]);
%
% add temporal inconsistency bewteen support functions

%
% tmp = reshape(vis1(1:4:end), [H W nLayers]);
% figure; imagesc([tmp(:,:,1) tmp(:,:,2) tmp(:,:, 3) ]);



% corresponding nodes at frame t+1
tmp_ind = indP2;
for iL =1:nLayers-1
    tmp_ind( (iL-1)*HP+1:iL*HP ) = ...
        tmp_ind( (iL-1)*HP+1:iL*HP ) + (iL-1)*H*W;
end;
sup2    = prevP(tmp_ind).*vals(4,1:HP*(nLayers-1)) + ...
          currP(tmp_ind).*(1-vals(4,1:HP*(nLayers-1)));

% corresponding nodes at frame t      
tmp_ind = indP1;
for iL =1:nLayers-1
    tmp_ind( (iL-1)*HP+1:iL*HP ) =...
        tmp_ind( (iL-1)*HP+1:iL*HP ) + (iL-1)*H*W;
end;
sup1    = prevP(tmp_ind).*vals(3,1:HP*(nLayers-1)) + ...
          currP(tmp_ind).*(1-vals(3,1:HP*(nLayers-1)));      
      
% add temporal consistency term for the nLayers-1 support functions
out(1:HP*(nLayers-1)) = out(1:HP*(nLayers-1)) +...
    params.lambdaIncon*(sup1~=sup2);
