function layer_uv = fit_affine_layer_motion_robust(uv, seg, rho, nRLSiters)
%%
% fit affine motion to the visible pixels of each layer

nL = max(seg(:));
sz = size(uv); sz = sz(1:2);

if nargin < 4
    nRLSiters = 3; 
end;


nData = prod(sz);
[X Y] = meshgrid(1:sz(2), 1:sz(1));
x = X(:); y = Y(:);
xy1 = [x y ones(nData, 1)];

for i = 1:nL
    
    u = uv(:,:,1); u = u(:);    
    v = uv(:,:,2); v = v(:);
    
    % fit affine motion to visible pixels
    tmp_u = u(seg==i);
    tmp_v = v(seg==i);
    tmp_xy1 = xy1(seg==i, :);
    
    A(:,1) = L2_fit(tmp_u, tmp_xy1);
    A(:,2) = L2_fit(tmp_v, tmp_xy1);
    
    A(:,1) = RLS_fit(tmp_u, tmp_xy1, rho, nRLSiters, A(:,1));
    A(:,2) = RLS_fit(tmp_v, tmp_xy1, rho, nRLSiters, A(:,2));

    
    % interpolate invisble pixels by affine motion model
    u = X*A(1,1)+Y*A(2,1)+A(3,1);
    v = X*A(1,2)+Y*A(2,2)+A(3,2);
    
    u = reshape(u, sz);
    v = reshape(v, sz);
    
    layer_uv(:,:,:,i) = cat(3, u, v);

end;

function a = L2_fit(u, xy1)    
% least square fit u = a1x + a2y + a3;
a = (xy1'*xy1)\(xy1'*u);

function s = RLS_fit(u, xy1, rho, niters, s)

for iter = 1:niters;
    err = u - s(1)*xy1(:,1) -s(2)*xy1(:,2)-s(3);
    w = deriv_over_x(rho, err);
    s = (xy1'* [w.*xy1(:,1) w.*xy1(:,2) w] )\(xy1'* (w.*u));
end;
