function [outputData layer_support flow_sel] = parse_data_seq_si(...
    ini_layer_uv, ini_layer_uvb, pyramid_images, color_pyramid_images,...
    layer_g, seg, params)
%%

inputDownRatio  = params.inputDownRatio;
inputSZ         = size(seg{1});
downSZ          = round(inputSZ/inputDownRatio);


layer_uv  = ini_layer_uv; 
layer_uvb = ini_layer_uvb; 

% fit affine mean flow field (robustly)
layer_mean_affine_uv = fit_affine_motion_robust(...
                        layer_uv{1}, params.rho_affine);
layer_mean_affine_uvb= fit_affine_motion_robust(...
                        layer_uvb{1}, params.rho_affine);
                    

% down sample segmentation, flow field, and support functions
sseg{1} = imresize(seg{1}, downSZ, 'nearest');
sseg{2} = imresize(seg{2}, downSZ, 'nearest');

for i=1:length(layer_uv{1})
    layer_uv_s{i} = imresize(layer_uv{1}{i}, downSZ)/inputDownRatio;
    layer_mean_affine_uv_s{i} = imresize(layer_mean_affine_uv{i},...
        downSZ)/inputDownRatio;
    layer_uvb_s{i} = imresize(layer_uvb{1}{i}, downSZ)/inputDownRatio;
    layer_mean_affine_uvb_s{i} = imresize(layer_mean_affine_uvb{i},...
        downSZ)/inputDownRatio;
end;

for t=1:length(layer_g);
    layer_g_s{t} = imresize(layer_g{t}, downSZ);
end;

% convert continuous support function to binary support functions
layer_support = zeros([size(layer_g_s{1}) length(layer_g_s)]);
for t=1:length(layer_g_s)
    layer_support(:,:,:,t) = layer_g_s{t} > 0;
end;

% down sample image features and color images
images = imresize(pyramid_images{1}, downSZ);

if size(color_pyramid_images{1}, 4)>1
    labImages = {imresize(color_pyramid_images{1}(:,:,:,1), downSZ),...
        imresize(color_pyramid_images{1}(:,:,:,2), downSZ)};
else
    labImages = {imresize(color_pyramid_images{1}(:,:,1), downSZ),...
        imresize(color_pyramid_images{1}(:,:,2), downSZ)};
end;


layer_seg = cat(3, sseg{1}, sseg{2});

uvf = composite_final_flow_seg(layer_uv_s,  layer_seg(:,:,1));
uvb = composite_final_flow_seg(layer_uvb_s, layer_seg(:,:,2));

% candidate flow : affine plus single-layered result
can_uvf = [];
can_uvb = [];
for i=1:length(layer_uv_s)    
    can_uvf = cat(4, can_uvf, layer_mean_affine_uv_s{i});
    can_uvb = cat(4, can_uvb, layer_mean_affine_uvb_s{i});
end;    

% affine mean flow field
outputData.aff_uv = {can_uvf, can_uvb};

can_uvf = cat(4, can_uvf, uvf);
can_uvb = cat(4, can_uvb, uvb);

% % add patch matching results to candidate set
% pmDir = '/data/vision/dqsun/program/iccv11_flow/result/precompute/PM_flow/Grove/';
% pmuvf = readFlowFile([pmDir 'forward_003.flo']);
% pmuvb = readFlowFile([pmDir 'backward_003.flo']);
% pmuvf = imresize(pmuvf, downSZ)/inputDownRatio;
% pmuvb = imresize(pmuvb, downSZ)/inputDownRatio;
% pmuvf(isnan(pmuvf)) = 0;
% pmuvb(isnan(pmuvb)) = 0;
% 
% can_uvf = cat(4, can_uvf, pmuvf);
% can_uvb = cat(4, can_uvb, pmuvb);

outputData.can_uv = {can_uvf, can_uvb};

% luv = [];luv2 = [];
% for i=1:length(layer_uv_s)
%     luv = cat(4, luv, layer_uv_s{i});
%     luv2 = cat(4, luv2, layer_uvb_s{i});
% end;    

% assign initial flow selection variables: pick up single-layer output for
% visible pixels, affine mean flow field for invisible pixels
nLayers = length(layer_uv_s);
for t=1:2;      
    flow_sel{t} = [];
    for i=1:nLayers
        tmp = ones(downSZ)*i;        
        tmp(layer_seg(:,:,t)==i) = nLayers+1;
        flow_sel{t} = cat(3, flow_sel{t}, tmp);
    end;
end;

% two frame symmetric case
outputData.images    = {images(:,:,1:2), images(:,:,2:-1:1)};
outputData.labImages = labImages;