function flow_sel = select_flow_gc_si(layer_support, t, layer_id,...
                    params, data, flow_sel_init)            
%%
% given layer segmentation, select flow field for each layer
% sequential Ising model,
% temporal consistency of layer support, deviation from affine flow fields


layer_vis   = support2visibility(layer_support);
nLayers     = size(layer_vis,3);
[H W nCan]  = size(data.OOB{t});
nStates     = nCan;

matchCost   = data.matchCost{t};
OOB         = data.OOB{t};

if layer_id < nLayers
    sup1 = layer_support(:,:,layer_id, t);
    sup1 = sup1(:);
    
    if t < params.T
        sup2  = layer_support(:,:,layer_id, t+1);
    elseif t==params.T
        sup2  = layer_support(:,:,layer_id, t-1);
    end;    
end;

vis1        = layer_vis(:,:,layer_id, t);
vis1        = vis1(:);
tmpIndex    = data.tempNeighbors{t};

if t < params.T
    vis2  = layer_vis(:,:,layer_id, t+1);
elseif t==params.T
    vis2  = layer_vis(:,:,layer_id, t-1);
end;

matchCost(OOB==1) = params.lambdaOcc;

% deviation from the affine flow field of the layer
for i=1:size(data.can_uv{t},4)
    tmp = data.can_uv{t}(:,:,:,i) - data.aff_uv{t}(:,:,:,layer_id);
    tmp = evaluate(params.rho_affine, tmp);
    dev(:,:,i) = sum(tmp, 3)*params.lambdaAff;
end;

% spatial term
vert = im2col((reshape(1:H*W,H,W)),[2 1])';
hor  = im2col((reshape(1:H*W,H,W)),[1 2])';

spatial_func    = @(u) compute_pw_spatial_flow_potential(u,...
    data.can_uv{t}, params);
    
fp              = flow_sel_init(:)-1;

current_energy  = 1e32;
success         = 0;

% start alpha expansion
while true    
    for iState=0:nStates-1
        
        current_proposal = iState *ones(size(fp));

        % pairwise spatial term
        spa1 = generateEnergyTable([vert; hor], spatial_func,...
            current_proposal(1:H*W), fp(1:H*W));       
           
        pairwise_terms  = [spa1];
        
        % singleton data term
        
        ind = (1:H*W)'; 
        
        % mathcing cost        
        % current proposal
        currInd   = ind + current_proposal*H*W;        
        curr_cost = matchCost(currInd);
        currNbInd = tmpIndex(currInd);
        currVisb2 = vis2(currNbInd);
         % occluded
        curr_cost(currVisb2==0) = params.lambdaOcc; 
        % invisible        
        curr_cost(vis1==0) = 0; 
        
        % previous proposal
        prevInd   = ind + fp*H*W;
        prev_cost = matchCost(prevInd);
        prevNbInd = tmpIndex(prevInd);
        prevVisb2 = vis2(prevNbInd);
        % occluded
        prev_cost(prevVisb2==0) = params.lambdaOcc;
        % invisible        
        prev_cost(vis1==0) = 0; 
       

        % deviation from mean affine flow fields
        curr_dev = dev(currInd);
        prev_dev = dev(prevInd);        

        cost0 = curr_cost + curr_dev;
        cost1 = prev_cost + prev_dev;

        % temporal consistency of layer support functions
        if layer_id < nLayers            
            currSup2 = sup2(currNbInd);
            prevSup2 = sup2(prevNbInd);            
        
            cost0 = cost0 + params.lambdaIncon*(sup1~=currSup2);
            cost1 = cost1 + params.lambdaIncon*(sup1~=prevSup2);
        end;
                
        
        data_unary = [ind'; cost0'; cost1'];
        
        [new_fp,new_energy] = QPBO_all_in_one(H*W,...
            data_unary, pairwise_terms);
        
        if (new_energy - current_energy) > .5
            %if we use potentials of type double, it is
            %possible that the new energy grows, because of numerical
            %issues, see QPBO documentation
            
            %disp(strcat('WARNING: energy grows, (new_energy - current_energy) >'...
            %    ,num2str(new_energy - current_energy)));
        elseif (new_energy - current_energy) < -1
            success = 1;
            current_energy=new_energy;
        end
        
        %fprintf('%3.3f ', current_energy);

        %generate new labeling
        fp = (1-new_fp).* current_proposal+ new_fp .* fp;
        
    end
    %fprintf('\n');
    
    if success == 0
        %disp('finished, energy did not shrink');
        break;
    end
    success = 0;
end;
% fprintf('%3.3f \n', current_energy);

% parse collapsed states to individual variables
fp = reshape(fp, [H W]);

flow_sel = fp+1;

    
