function seg= support2seg(support)
%% convert the binary support function to segmentation sequentially
% test code
%
% H = 100; W = 100;
% support = zeros([H W 2]);
% support(1:H/4, 1:W/4, 1) = 1;
% support(1:H/2, 1:W/2, 2) = 1;
% seg = support2seg(support);
% figure; imagesc(seg); 
% segs = support2seg({support, 1-support});
% figure; imagesc(segs{1}); 
% figure; imagesc(segs{2});

% segs = support2seg( cat(4, support, 1-support) );
% figure; imagesc(segs(:,:,1)); 
% figure; imagesc(segs(:,:,2));


% sanity check
% segs = support2seg({support, 2-support});
% seg = support2seg(support+1);
%%

if iscell(support)
    
    H = size(support{1},1);
    W = size(support{1},2);
    nLayers = size(support{1},3)+1;
    for t=1:length(support)
        
        % sanity check
        if sum(support{t}(:)>1) >0 | sum(support{t}(:)<0) >0
            error('support2seg: binary support should only takes values in {0, 1}!');
        end;
        
        seg{t} = zeros(H,W);       
        for i=1:nLayers-1
            vis = (support{t}(:,:,i) ==1) & (seg{t} == 0);
            seg{t}(vis) = i;
        end;
        % all the remaining pixels are explained by background layer
        seg{t}(seg{t}==0) = nLayers;
    end;
    
else
    
    % sanity check
    if sum(support(:)>1) >0 | sum(support(:)<0) >0
        error('support2seg: binary support should only takes values in {0, 1}!');
    end;
    
    [H W nLayers T] = size(support);
    nLayers = nLayers+1;
    
    seg = zeros(H,W, T);
    
    for t=1:T;
    
        tmp_seg = ones(H,W)*nLayers;
        for i=1:nLayers-1
            vis = (support(:,:,i, t) ==1) & (tmp_seg == nLayers);
            tmp_seg(vis) = i;
        end;
        
        seg(:,:,t) = tmp_seg;
        
    end;
    
end;





