function support = visibility2support(vis)
%%
% change the sequetial binary visibility mask to binary support functions

% this is usually not unique mapping and minimum/maximum support is adopted, i.e.
% if a pixel is occluded by pixels of front layers, its support function is
% set to be 0/1

% this function is use when switchcing layer orders: visibility of layers
% are retained while

% test code
% H = 100; W = 100;
% vis = zeros([H W 3]);
% vis(1:H/4, 1:W/4, 1) = 1;
% vis(:,:, 2) = 1-vis(:,:, 1);
% %
% support = visibility2support( cat(4, vis, vis) );
% %
% figure; 
% t = 2;
% i=1; subplot(1,3,i); imagesc(support(:,:,i,t)); 
% i=2; subplot(1,3,i); imagesc(support(:,:,i,t)); 


% sanity check
if sum(vis(:)>1) >0 || sum(vis(:)<0) >0
    error('support2seg: visibility should only take values in {0, 1}!');
end;

[H W nLayers T] = size(vis);

support = zeros(H,W,nLayers-1,T);

for t=1:T;
    
    if sum(reshape(vis(:,:,:,t), 1, []) ) ~= H*W
        error('visibility2support: at each pixel, one and only one layer is visible!');
    end;
    
    % indicator whether pixels at front layers have been observed or not
    tmp_obs = zeros(H,W);
    
    for i=1:nLayers-1

        % minimum support
        support(:,:,i,t) = (vis(:,:,i,t) == 1);                
        
        % maximum support
%         support(:,:,i,t) = (vis(:,:,i,t) == 1) | (tmp_obs==1);        
%         tmp_obs(support(:,:,i,t)==1) = 1;
    end;    
    
end;


