
// input image IM and filter F
// output, filtered result, "corr", "same", "replicate"

#include "math.h"
#include "mex.h"

#define TOL 1e-6
#define MAX(A,B) (A)>(B)?(A):(B)
#define MIN(A,B) (A)<(B)?(A):(B)

// Compile using:  mex imfilter_mex.cpp

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{

    double *out, *im, *F;

    int rowLen, colLen, rowLenF, colLenF, hr, hc;

    int rStart, rEnd, cStart, cEnd;

    //Get data
    im = mxGetPr(prhs[0]);
    F  = mxGetPr(prhs[1]);
    
    colLen  = mxGetN(prhs[0]);	// number of columns or width
    rowLen  = mxGetM(prhs[0]);	// number of rows or height

    //printf("%d\t%d\n", colLen, rowLen);
    
    colLenF = mxGetN(prhs[1]);	// number of columns
    rowLenF = mxGetM(prhs[1]);	// number of rows

    rStart = MAX( (rowLenF-1)>>1, 0);
    rEnd   = rowLenF>>1;
    cStart = MAX( (colLenF-1)>>1, 0);
    cEnd   = colLenF>>1;
    
    //cout<<rStart<<"\t"<<rEnd<<"\t"<<cStart<<"\t"<<cEnd<<endl;    
    //printf("%d\t%d\t%d\t%d\n", rStart, rEnd, cStart, cEnd);
    //Allocate memory and assign output pointer
    plhs[0] = mxCreateDoubleMatrix(rowLen, colLen, mxREAL); //mxReal is our data-type

    //Get a pointer to the data space in our newly allocated memory
    out = mxGetPr(plhs[0]);

    int offset, i1, j1;
    float w;
    // the matrix is column indexed as in Matlab
    for(int i=0;i<colLen;i++)
    {
        for(int j=0;j<rowLen;j++)
        {
           offset = i*rowLen+j;
           out[offset] =0;
    	   for (int c=cStart; c<=cEnd; c++)
		for (int r=rStart; r<=rEnd; r++)
                {
                    i1 = MIN(MAX(i+c, 0), colLen-1);
                    j1 = MIN(MAX(j+r, 0), rowLen-1);
                    w  = F[ (c-cStart)*rowLenF+r-rStart  ];
                    out[offset] += w*im[i1*rowLen+j1];     
//                     if (j == 0 && i==colLen-1)               
//                     {
//                         printf("%d\t%d\t", i1, j1);
//                         printf("%3.3f\t%3.3f\n", w, im[i1*rowLen+j1]);
//                     }
                }
	}

    }
//     for(int i=0;i<colLen-1;i++)
//     {
//         for(int j=0;j<rowLen;j++)
//         {
//             offset = i*rowLen+j;
//             out[offset] = F[0]*im[i*rowLen+j] + F[1]*im[(i+1)*rowLen+j];     
//         }
// 
//     }
//     for(int i=colLen-1;i<colLen;i++)
//         for(int j=0;j<rowLen;j++)
//         {
//             offset = i*rowLen+j;
//             out[offset] = F[0]*im[i*rowLen+j] + F[1]*im[i*rowLen+j];     
//         }
   return;
}

