
// input ndimensional arary IM, 
// type of sigmoid function 1 sigmoid 2 derivative of sigmoid -1 inverse function
// lambda slope parameter for sigmoid function

// y = 1/( 1+exp(-lambda*x) )

#include "math.h"
#include "mex.h"

#define TOL 1e-6

// Compile using:  mex sigmoid_mex.cpp

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    const mxArray *imData;
    double *outMatrix, *imValues, *type, *lambda;
    int typei;
    const mwSize *dim_array;       
    mwSize elements, number_of_dims;
    int j;
    
    
    //Copy input pointer
    imData = prhs[0];
    if (nrhs >1)
        type   = mxGetPr(prhs[1]);
    else
        type[0] = 1;
    
    if (nrhs >2)
        lambda = mxGetPr(prhs[2]);
    else
        lambda[0] = 2;
    
    //Get data
    imValues = mxGetPr(imData);

    // Get the number of dimensions in the input argument. 
    dim_array=mxGetDimensions(prhs[0]);
        
    // Get the number of elements in the input argument 
    elements=mxGetNumberOfElements(prhs[0]);
    
    //Allocate memory and assign output pointer
    number_of_dims=mxGetNumberOfDimensions(prhs[0]);
    //plhs[0]=mxCreateDoubleMatrix(elements,number_of_dims,mxREAL);
    plhs[0]=mxCreateNumericArray(number_of_dims, dim_array,mxDOUBLE_CLASS, mxREAL);
    
    
    //Get a pointer to the data space in our newly allocated memory
    outMatrix = mxGetPr(plhs[0]);

    typei = (int) type[0];
    for(j=0;j<elements;j++) 
    {
        switch (typei)
        {
            case 1:
                // sigmoid
                outMatrix[j]= 1.0/ (1+exp(-lambda[0]*imValues[j]));
                break;
            case 2:
                // derivative
                outMatrix[j]= lambda[0]/(1+exp(-lambda[0]*imValues[j]))/(1+exp(lambda[0]*imValues[j]));
                break;
            case -1:
                // inverse function
                double tmp;
                tmp = imValues[j];
                if (tmp < 0)
                    tmp = 0;
                if (tmp > 1)
                    tmp = 1;                
                    
                outMatrix[j] = -log( (1-tmp)/tmp) / lambda[0];
                break;
            default:
                printf("sigmmoid_mex: unknown input type %d\n", type[0]);
                break;
                
        }

    }

    
   return;
}