
function [positions, labels, layers_field] = segmentation2bboxes(segmentations, num_classes)

num_layers = 0;
positions = cell(1);
labels = [];
im_size = [size(segmentations, 1), size(segmentations, 2)];
layers_field = zeros(im_size(1), im_size(2));
pad_size = 40;
load class_names_segmentation.mat

% compute initial set of layers 
init_segmentation = segmentations(:, :, 1);
end_segmentation = segmentations(:, :, 2);
se = strel('disk',8,4);

isobject = [1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 0, 0, 1, 1, 0, 1, 0, 0, 1, 0, 1];
 

for i=1:num_classes
	class_mask = (init_segmentation==i);
   
    
	if (sum(class_mask(:)) > 200) %&& valid_class
		
		if isobject(i)
	
			% clean segmentation
			class_mask = imdilate(class_mask,se);
			class_mask = imerode(class_mask,se);
			class_mask = remove_small_blobs(class_mask);
			
			% make a layer per blob 
			[L, num_objects] = bwlabel(class_mask);
			for j=1:num_objects
			
				% check that the object didn't disappear 
				bb_next = track_blob(L == j, i, end_segmentation); 
                
				if ~isempty(bb_next)

                    num_layers = num_layers+1;
		
					% add layer to field 
					layers_field(L == j) = num_layers;
		
					% compute bounding box 
					[row, col] = find(L == j); 
				        bb = [min(row(:)), min(col(:)), (max(row(:)) - min(row(:))), (max(col(:)) - min(col(:)))];
	
					% add bounding box in next frame
					start_bb_row = min(bb(1), bb_next(1));
					start_bb_col = min(bb(2), bb_next(2));
					end_bb_row = max(bb(1)+bb(3), bb_next(1)+bb_next(3));
					end_bb_col = max(bb(2)+bb(4), bb_next(2)+bb_next(4));
					bb = [start_bb_row, start_bb_col, (end_bb_row - start_bb_row), (end_bb_col - start_bb_col)];
	
                    % add pad around it 
                    start_pos_1 = max(1, round(bb(1)-pad_size));
                    start_pos_2 = max(1, round(bb(2)-pad_size));
                    end_pos_1 = min(im_size(1), bb(1)+bb(3)+(pad_size));
                    end_pos_2 = min(im_size(2), bb(2)+bb(4)+(pad_size));
                    positions{num_layers} = [start_pos_1, start_pos_2, (end_pos_1-start_pos_1), (end_pos_2-start_pos_2)];
					labels = [labels, i];
				
				end
			end
		else

			num_layers = num_layers+1;

			% compute bounding box 
			[row, col] = find(init_segmentation == i); 
			positions{num_layers} = [min(row(:)), min(col(:)), (max(row(:)) - min(row(:))), (max(col(:)) - min(col(:)))];
			labels = [labels, i];
		end
	end
end
	

% track each of the layers (for now I assume they don't move)

for i=1:num_layers 
	positions{i} = repmat(positions{i}, [size(segmentations, 3), 1]);
end


