function y = charbonnier(x, sigma, type)
%%
%CHARBONNIER   Charbonnier robust function.
%   CHARBONNIER(X, SIGMA, TYPE) evaluates the Charbonnier robust function
%   with sigma SIGMA at point(s) X.  
%   TYPE selects the evaluation type:
%    - 0: function value
%    - 1: first derivative
%    - 2: second derivative
%  
%   This is a private member function of the class 'robust_function'. 
%
%   Author:  Stefan Roth, Department of Computer Science, Brown University
%   Contact: roth@cs.brown.edu
%   $Date: $
%   $Revision: $

% Copyright 2004-2006 Brown University, Providence, RI.
% 
%                         All Rights Reserved
% 
% Permission to use, copy, modify, and distribute this software and its
% documentation for any purpose other than its incorporation into a
% commercial product is hereby granted without fee, provided that the
% above copyright notice appear in all copies and that both that
% copyright notice and this permission notice appear in supporting
% documentation, and that the name of Brown University not be used in
% advertising or publicity pertaining to distribution of the software
% without specific, written prior permission.
% 
% BROWN UNIVERSITY DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
% INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR ANY
% PARTICULAR PURPOSE.  IN NO EVENT SHALL BROWN UNIVERSITY BE LIABLE FOR
% ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
% WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
% ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
% OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

  
  switch (type)
   case 0
%     y = sigma^2 * sqrt(1 + x.^2 / sigma^2);   % not original charbonnier
    %y = sigma * sqrt(1 + x.^2 / sigma^2);   % modified by dqsun (2009-09-12) 
    y = abs(x);   % modified by dqsun (2011-09-30) 
   case 1
%     y = x ./ sqrt(1 + x.^2 / sigma^2);
    y = x ./ sqrt(1 + x.^2 / sigma^2)/sigma;   % modified by dqsun (2009-09-12) 
   case 2
%     y = 1 ./ sqrt(1 + x.^2 / sigma^2);
    %y = 1 ./ sqrt(1 + x.^2 / sigma^2)/sigma;   % modified by dqsun (2009-09-12) 
    
    
    % modified by dqsun (2011-09-30) 
%     y = 1./(x+eps);   
%     y(y>sigma) = sigma;
%     %y = 1./(x+sigma);   
    y = 1 ./ sqrt(sigma^2 + x.^2);   % modified by dqsun (2011-10-03) 
  end