function out = compute_pairwise_spatial_potential(vals, can_uv, ...
    selected, weight, params)
%%
% nStates = params.nStates;
rho        = params.rho_spatial;
lambdaFlow = params.lambdaFlow;
lambdaSeg  = params.lambdaSeg;

H = size(can_uv,1);
W = size(can_uv,2);
nCan = size(can_uv,4);


% obtain layer segmentation and flow selection from collapsed state

[segIndex f0 f1] = convert_collapsed_state_to_org(vals(3:4,:), nCan);

% compute flow smoothness term

[out1 out2 ] = evluate_flow_smoothness(vals(1:2,:), H, W, can_uv, rho, f0);
[out3 out4 ] = evluate_flow_smoothness(vals(1:2,:), H, W, can_uv, rho, f1);

% layer segmentation
% set label of unselected pixels to be different from one or two
segIndex( ~selected(vals(1:2,:)) ) = -1; 

out = double(segIndex(1,:) ~= segIndex(2,:));

% modulated by color weight 
out = out.*weight*lambdaSeg;

out = out+ (out1+out2+out3+out4)*lambdaFlow;
    

function [out1 out2 ] = evluate_flow_smoothness(vals, H, W, can_uv, rho, f0)

% horizontal flow
ind1 = vals(1,:);
ind2 = vals(2,:);

ind1 = ind1(:)' + f0(1,:)*2*H*W;
ind2 = ind2(:)' + f0(2,:)*2*H*W;
u_   = can_uv(ind1)-can_uv(ind2);           
out1 = evaluate(rho, u_);  

% vertical flow
ind1 = ind1 +H*W;
ind2 = ind2 +H*W;
u_   = can_uv(ind1)-can_uv(ind2);
out2 = evaluate(rho, u_);  



% % first flow field, horizontal flow
% ind1 = vals(1,:);
% ind2 = vals(2,:);
% 
% ind1 = ind1(:)' + f0(1,:)*2*H*W;
% ind2 = ind2(:)' + f0(2,:)*2*H*W;
% u_   = can_uv(ind1)-can_uv(ind2);           
% out1 = evaluate(rho, u_)*lambdaFlow;  
% 
% % first flow field, vertical flow
% ind1 = ind1 +H*W;
% ind2 = ind2 +H*W;
% u_   = can_uv(ind1)-can_uv(ind2);
% out2 = evaluate(rho, u_)*lambdaFlow;  
% 
% % second flow field, horizontal flow
% ind1 = vals(1,:);
% ind2 = vals(2,:);
% 
% ind1 = ind1(:)' + f1(1,:)*2*H*W;
% ind2 = ind2(:)' + f1(2,:)*2*H*W;
% u_   = can_uv(ind1)-can_uv(ind2);           
% out3 = evaluate(rho, u_)*lambdaFlow;  
% 
% % second flow field, vertical flow
% ind1 = ind1 +H*W;
% ind2 = ind2 +H*W;
% u_   = can_uv(ind1)-can_uv(ind2);
% out4 = evaluate(rho, u_)*lambdaFlow;  