function uo = denoise_color_weighted_medfilt2_single(uv, im, bfhsz, mfsz, sigma_i, fullVersion)
%%
% edge region: weighted median filtering, the weights are determined by
%  spatial distance, intensity distance, occlusion state
% smooth region: 

sigma_x = 7;   %  spatial distance (7)

dilate_sz = 5*[1 1];  % dilation window size for flow edge region [5 5]

sz = size(im);
sz = sz(1:2);

if nargin < 3
    bfhsz = 10; % half window size
end;

if nargin < 4 || isempty(mfsz)
    uvo = uv; 
else
    uvo = medfilt2(uv, mfsz, 'symmetric');    
end;

if nargin < 5
    sigma_i = 5; 
end;

if nargin < 6
    fullVersion = false;
end;

e    = edge(uv(:,:,1), 'sobel');
mask = imdilate(e, ones(dilate_sz) );

% figure; imagesc(mask);

% below to apply WMF to all regions
if fullVersion
    mask = ones(size(mask));
end;

[indx_row, indx_col] = find(mask ==1);
 
pad_u  = padarray(uv(:,:,1), bfhsz*[1 1], 'symmetric', 'both');        
pad_im = padarray(im, bfhsz*[1 1], 'symmetric', 'both');        
%pad_occ= padarray(occ, bfhsz*[1 1], 'symmetric', 'both');        


[H W] = size(pad_u);

% Divide into several groups for memory reasons ~70,000 causes out of memory
Indx_Row = indx_row;
Indx_Col = indx_col;
N        = length(Indx_Row); % number of elements to process
n        = 4e4;              % number of elements per batch
nB       = ceil(N/n);

for ib = 1:nB;
    istart = (ib-1)*n + 1;
    iend   = min(ib*n, N);
    indx_row = Indx_Row(istart:iend);
    indx_col = Indx_Col(istart:iend);    

    [C R] = meshgrid(-bfhsz:bfhsz, -bfhsz:bfhsz);
    nindx = R + C*H;    
    cindx = indx_row +bfhsz  + (indx_col+bfhsz-1)*H;
    
    pad_indx = repmat(nindx(:), [1 length(indx_row)]) + ...
               repmat(cindx(:)', [(bfhsz*2+1)^2, 1] );
           
    % spatial weight
    tmp = exp(- (C.^2 + R.^2) /2/sigma_x^2 );
    weights = repmat(tmp(:), [1 length(indx_row)]);    
    
    % %Uncomment below: no spatial weight for test
    % weights = ones(size(weights));    
    
    % Intensity weight
    tmp_w = zeros(size(weights));
    
    for i = 1:size(pad_im,3)
        tmp = pad_im(:,:,i);
        tmp_w = tmp_w + (tmp(pad_indx) - repmat(tmp(cindx(:))', [(bfhsz*2+1)^2, 1])).^2;
    end;    
    tmp_w = tmp_w/size(pad_im,3);

    % truncated L2
%     Ti2 = 20^2;
%     tmp_w(tmp_w>Ti2) = Ti2; fprintf('.');
        
    weights = weights.* exp(-tmp_w/2/sigma_i^2);

    % Occlusion weight    
    % weights = weights.*pad_occ(pad_indx);
    
    % Normalize
    weights = weights./repmat(sum(weights, 1), [(bfhsz*2+1)^2, 1]);
    
    neighbors_u = pad_u(pad_indx);
    

    % solve weighted median filtering
    indx = sub2ind(sz, indx_row, indx_col);
    uo   = uvo(:,:,1);
    u    = weighted_median(weights, neighbors_u);
%     u    = weighted_median_iter(weights, neighbors_u);
    uo(indx) = u;    
end;
