function layer_uv = fit_affine_layer_motion_g(layer_uv, layer_g, rho)
% Fit affine motion to the visible pixels of each layer

nL = length(layer_uv);
sz = size(layer_uv{1}); sz = sz(1:2);

nData = prod(sz);
[X Y] = meshgrid(1:sz(2), 1:sz(1));
x   = X(:); 
y   = Y(:);
xy1 = [x y ones(nData, 1)];

% Obtain hard segmentation by thresholding the hidden fields
ind = threshold_g(layer_g);

for i = 1:nL
    
    if sum(ind(:)==i) < 10
        % not enough pixels, assign zero flow
        A = zeros(3,2);    
    else
        uv = layer_uv{i};
        u = uv(:,:,1); u = u(:);
        v = uv(:,:,2); v = v(:);
        
        % fit affine motion to visible pixels
        tmp_u = u(ind==i);
        tmp_v = v(ind==i);
        tmp_xy1 = xy1(ind==i, :);
        
%         A(:,1) = L2_fit(tmp_u, tmp_xy1);
%         A(:,2) = L2_fit(tmp_v, tmp_xy1);
        A(:,1) = RLS_fit(tmp_u, tmp_xy1, rho);
        A(:,2) = RLS_fit(tmp_v, tmp_xy1, rho);
    end;
    
    % interpolate invisble pixels by affine motion model
    u = X*A(1,1)+Y*A(2,1)+A(3,1);
    v = X*A(1,2)+Y*A(2,2)+A(3,2);
    
    u = reshape(u, sz);
    v = reshape(v, sz);
    
    layer_uv{i} = cat(3, u, v);
end;


