function flow_sel = select_flow_gc(layer_seg, t, seg_id,...
                    can_uv, params, data, flow_sel_init)            
%%
% given layer segmentation, select flow field for each layer


[H W nCan] = size(data.OOB{t});

nStates  = nCan;

ind = reshape(1:H*W, [H W]);

matchCost = data.matchCost{t};
OOB       = data.OOB{t};

if t ==1
    tmpIndex = data.tempNeighbors{1}+H*W;
elseif t==2
    tmpIndex = data.tempNeighbors{2};
end;
    
warpSeg = layer_seg(tmpIndex);

occ = repmat(layer_seg(:,:,t), [1 1 nStates]) > warpSeg;
forb= repmat(layer_seg(:,:,t), [1 1 nStates]) < warpSeg;

matchCost(occ) = params.lambdaOcc;
matchCost(forb)= params.lambdaForbid;

matchCost(OOB==1) = params.lambdaOcc;

% temporal consistency of layer support functions


validInd = find(layer_seg(:,:,t) == seg_id); 

% spatial term
vert = im2col((reshape(1:H*W,H,W)),[2 1])';
hor  = im2col((reshape(1:H*W,H,W)),[1 2])';

spatial_func   = @(u) compute_pw_spatial_flow_potential(u,...
                         can_uv, params);
    

    
fp = flow_sel_init(:)-1;


current_energy = 1e32;
success = 0;
% start alpha expansion
while true    
    for iState=0:nStates-1
        current_proposal = iState *ones(size(fp));

        % pairwise spatial term
        spa1 = generateEnergyTable([vert; hor], spatial_func,...
            current_proposal(1:H*W), fp(1:H*W));       
           
        pairwise_terms  = [spa1];
        
        % singleton data term
        
        % Data term : mathcing cost
        currInd = validInd + current_proposal(validInd)*H*W;
        fpInd   = validInd + fp(validInd)*H*W;
        
        data_unary = [validInd'; matchCost(currInd)'; matchCost(fpInd)'];

        
        [new_fp,new_energy] = QPBO_all_in_one(H*W, data_unary, pairwise_terms);
        
        if (new_energy - current_energy) > .5
            %if we use potentials of type double, it is
            %possible that the new energy grows, because of numerical
            %issues, see QPBO documentation
            
            %             disp(strcat('WARNING: energy grows, (new_energy - current_energy) >'...
            %                 ,num2str(new_energy - current_energy)));
        elseif (new_energy - current_energy) < -1
            success = 1;
            current_energy=new_energy;
        end
        
%         fprintf('%3.3f ', current_energy);
        
        fp = (1-new_fp).* current_proposal+ new_fp .* fp;%generate new labeling
        
    end
%     fprintf('\n');
    
    if success == 0
%         disp('finished, energy did not shrink');
        break;
    end
    success = 0;
end;
% fprintf('%3.3f \n', current_energy);

% parse collapsed states to individual variables
fp = reshape(fp, [H W]);


flow_sel = fp+1;

    
